/****************************************************************************
** Meta object code from reading C++ file 'OneMessageModel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Model/OneMessageModel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'OneMessageModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Mailbox__OneMessageModel_t {
    QByteArrayData data[35];
    char stringdata0[419];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__OneMessageModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__OneMessageModel_t qt_meta_stringdata_Imap__Mailbox__OneMessageModel = {
    {
QT_MOC_LITERAL(0, 0, 30), // "Imap::Mailbox::OneMessageModel"
QT_MOC_LITERAL(1, 31, 15), // "envelopeChanged"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 12), // "flagsChanged"
QT_MOC_LITERAL(4, 61, 18), // "mainPartUrlChanged"
QT_MOC_LITERAL(5, 80, 22), // "handleModelDataChanged"
QT_MOC_LITERAL(6, 103, 11), // "QModelIndex"
QT_MOC_LITERAL(7, 115, 7), // "topLeft"
QT_MOC_LITERAL(8, 123, 11), // "bottomRight"
QT_MOC_LITERAL(9, 135, 10), // "setMessage"
QT_MOC_LITERAL(10, 146, 7), // "mailbox"
QT_MOC_LITERAL(11, 154, 3), // "uid"
QT_MOC_LITERAL(12, 158, 7), // "subject"
QT_MOC_LITERAL(13, 166, 4), // "date"
QT_MOC_LITERAL(14, 171, 12), // "receivedDate"
QT_MOC_LITERAL(15, 184, 4), // "from"
QT_MOC_LITERAL(16, 189, 2), // "to"
QT_MOC_LITERAL(17, 192, 2), // "cc"
QT_MOC_LITERAL(18, 195, 3), // "bcc"
QT_MOC_LITERAL(19, 199, 6), // "sender"
QT_MOC_LITERAL(20, 206, 7), // "replyTo"
QT_MOC_LITERAL(21, 214, 9), // "inReplyTo"
QT_MOC_LITERAL(22, 224, 9), // "messageId"
QT_MOC_LITERAL(23, 234, 15), // "isMarkedDeleted"
QT_MOC_LITERAL(24, 250, 12), // "isMarkedRead"
QT_MOC_LITERAL(25, 263, 17), // "isMarkedForwarded"
QT_MOC_LITERAL(26, 281, 15), // "isMarkedReplied"
QT_MOC_LITERAL(27, 297, 14), // "isMarkedRecent"
QT_MOC_LITERAL(28, 312, 15), // "isMarkedFlagged"
QT_MOC_LITERAL(29, 328, 12), // "isMarkedJunk"
QT_MOC_LITERAL(30, 341, 15), // "isMarkedNotJunk"
QT_MOC_LITERAL(31, 357, 11), // "mainPartUrl"
QT_MOC_LITERAL(32, 369, 16), // "attachmentsModel"
QT_MOC_LITERAL(33, 386, 13), // "hasValidIndex"
QT_MOC_LITERAL(34, 400, 18) // "mainPartModelIndex"

    },
    "Imap::Mailbox::OneMessageModel\0"
    "envelopeChanged\0\0flagsChanged\0"
    "mainPartUrlChanged\0handleModelDataChanged\0"
    "QModelIndex\0topLeft\0bottomRight\0"
    "setMessage\0mailbox\0uid\0subject\0date\0"
    "receivedDate\0from\0to\0cc\0bcc\0sender\0"
    "replyTo\0inReplyTo\0messageId\0isMarkedDeleted\0"
    "isMarkedRead\0isMarkedForwarded\0"
    "isMarkedReplied\0isMarkedRecent\0"
    "isMarkedFlagged\0isMarkedJunk\0"
    "isMarkedNotJunk\0mainPartUrl\0"
    "attachmentsModel\0hasValidIndex\0"
    "mainPartModelIndex"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__OneMessageModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
      23,   52, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,
       3,    0,   40,    2, 0x06 /* Public */,
       4,    0,   41,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    2,   42,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
       9,    2,   47,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 6,    7,    8,

 // methods: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::UInt,   10,   11,

 // properties: name, type, flags
      12, QMetaType::QString, 0x00495001,
      13, QMetaType::QDateTime, 0x00495001,
      14, QMetaType::QDateTime, 0x00495001,
      15, QMetaType::QVariantList, 0x00495001,
      16, QMetaType::QVariantList, 0x00495001,
      17, QMetaType::QVariantList, 0x00495001,
      18, QMetaType::QVariantList, 0x00495001,
      19, QMetaType::QVariantList, 0x00495001,
      20, QMetaType::QVariantList, 0x00495001,
      21, QMetaType::QByteArray, 0x00495001,
      22, QMetaType::QByteArray, 0x00495001,
      23, QMetaType::Bool, 0x00495003,
      24, QMetaType::Bool, 0x00495003,
      25, QMetaType::Bool, 0x00495001,
      26, QMetaType::Bool, 0x00495001,
      27, QMetaType::Bool, 0x00495001,
      28, QMetaType::Bool, 0x00495001,
      29, QMetaType::Bool, 0x00495001,
      30, QMetaType::Bool, 0x00495001,
      31, QMetaType::QUrl, 0x00495001,
      32, QMetaType::QObjectStar, 0x00495001,
      33, QMetaType::Bool, 0x00495001,
      34, 0x80000000 | 6, 0x00095009,

 // properties: notify_signal_id
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       1,
       2,
       2,
       0,
       0,

       0        // eod
};

void Imap::Mailbox::OneMessageModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OneMessageModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->envelopeChanged(); break;
        case 1: _t->flagsChanged(); break;
        case 2: _t->mainPartUrlChanged(); break;
        case 3: _t->handleModelDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 4: _t->setMessage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const uint(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (OneMessageModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OneMessageModel::envelopeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (OneMessageModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OneMessageModel::flagsChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (OneMessageModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OneMessageModel::mainPartUrlChanged)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<OneMessageModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->subject(); break;
        case 1: *reinterpret_cast< QDateTime*>(_v) = _t->date(); break;
        case 2: *reinterpret_cast< QDateTime*>(_v) = _t->receivedDate(); break;
        case 3: *reinterpret_cast< QVariantList*>(_v) = _t->from(); break;
        case 4: *reinterpret_cast< QVariantList*>(_v) = _t->to(); break;
        case 5: *reinterpret_cast< QVariantList*>(_v) = _t->cc(); break;
        case 6: *reinterpret_cast< QVariantList*>(_v) = _t->bcc(); break;
        case 7: *reinterpret_cast< QVariantList*>(_v) = _t->sender(); break;
        case 8: *reinterpret_cast< QVariantList*>(_v) = _t->replyTo(); break;
        case 9: *reinterpret_cast< QByteArray*>(_v) = _t->inReplyTo(); break;
        case 10: *reinterpret_cast< QByteArray*>(_v) = _t->messageId(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->isMarkedDeleted(); break;
        case 12: *reinterpret_cast< bool*>(_v) = _t->isMarkedRead(); break;
        case 13: *reinterpret_cast< bool*>(_v) = _t->isMarkedForwarded(); break;
        case 14: *reinterpret_cast< bool*>(_v) = _t->isMarkedReplied(); break;
        case 15: *reinterpret_cast< bool*>(_v) = _t->isMarkedRecent(); break;
        case 16: *reinterpret_cast< bool*>(_v) = _t->isMarkedFlagged(); break;
        case 17: *reinterpret_cast< bool*>(_v) = _t->isMarkedJunk(); break;
        case 18: *reinterpret_cast< bool*>(_v) = _t->isMarkedNotJunk(); break;
        case 19: *reinterpret_cast< QUrl*>(_v) = _t->mainPartUrl(); break;
        case 20: *reinterpret_cast< QObject**>(_v) = _t->attachmentsModel(); break;
        case 21: *reinterpret_cast< bool*>(_v) = _t->hasValidIndex(); break;
        case 22: *reinterpret_cast< QModelIndex*>(_v) = _t->mainPartModelIndex(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<OneMessageModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 11: _t->setMarkedDeleted(*reinterpret_cast< bool*>(_v)); break;
        case 12: _t->setMarkedRead(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::OneMessageModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__OneMessageModel.data,
    qt_meta_data_Imap__Mailbox__OneMessageModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::OneMessageModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::OneMessageModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__OneMessageModel.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Imap::Mailbox::OneMessageModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 23;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 23;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 23;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 23;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 23;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Imap::Mailbox::OneMessageModel::envelopeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Imap::Mailbox::OneMessageModel::flagsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Imap::Mailbox::OneMessageModel::mainPartUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
