/****************************************************************************
** Meta object code from reading C++ file 'GpgMe++.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Cryptography/GpgMe++.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'GpgMe++.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Cryptography__GpgMePart_t {
    QByteArrayData data[13];
    char stringdata0[179];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Cryptography__GpgMePart_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Cryptography__GpgMePart_t qt_meta_stringdata_Cryptography__GpgMePart = {
    {
QT_MOC_LITERAL(0, 0, 23), // "Cryptography::GpgMePart"
QT_MOC_LITERAL(1, 24, 14), // "forwardFailure"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 10), // "statusTLDR"
QT_MOC_LITERAL(4, 51, 10), // "statusLong"
QT_MOC_LITERAL(5, 62, 10), // "statusIcon"
QT_MOC_LITERAL(6, 73, 17), // "handleDataChanged"
QT_MOC_LITERAL(7, 91, 11), // "QModelIndex"
QT_MOC_LITERAL(8, 103, 7), // "topLeft"
QT_MOC_LITERAL(9, 111, 11), // "bottomRight"
QT_MOC_LITERAL(10, 123, 19), // "internalUpdateState"
QT_MOC_LITERAL(11, 143, 33), // "Cryptography::SignatureDataBu..."
QT_MOC_LITERAL(12, 177, 1) // "d"

    },
    "Cryptography::GpgMePart\0forwardFailure\0"
    "\0statusTLDR\0statusLong\0statusIcon\0"
    "handleDataChanged\0QModelIndex\0topLeft\0"
    "bottomRight\0internalUpdateState\0"
    "Cryptography::SignatureDataBundle\0d"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Cryptography__GpgMePart[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   29,    2, 0x09 /* Protected */,
       6,    2,   36,    2, 0x09 /* Protected */,
      10,    1,   41,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString,    3,    4,    5,
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 7,    8,    9,
    QMetaType::Void, 0x80000000 | 11,   12,

       0        // eod
};

void Cryptography::GpgMePart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GpgMePart *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->forwardFailure((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 1: _t->handleDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 2: _t->internalUpdateState((*reinterpret_cast< const Cryptography::SignatureDataBundle(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Cryptography::SignatureDataBundle >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Cryptography::GpgMePart::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Cryptography__GpgMePart.data,
    qt_meta_data_Cryptography__GpgMePart,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Cryptography::GpgMePart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Cryptography::GpgMePart::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Cryptography__GpgMePart.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "LocalMessagePart"))
        return static_cast< LocalMessagePart*>(this);
    return QObject::qt_metacast(_clname);
}

int Cryptography::GpgMePart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_Cryptography__GpgMeSigned_t {
    QByteArrayData data[6];
    char stringdata0[77];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Cryptography__GpgMeSigned_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Cryptography__GpgMeSigned_t qt_meta_stringdata_Cryptography__GpgMeSigned = {
    {
QT_MOC_LITERAL(0, 0, 25), // "Cryptography::GpgMeSigned"
QT_MOC_LITERAL(1, 26, 17), // "handleDataChanged"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 57, 7), // "topLeft"
QT_MOC_LITERAL(5, 65, 11) // "bottomRight"

    },
    "Cryptography::GpgMeSigned\0handleDataChanged\0"
    "\0QModelIndex\0topLeft\0bottomRight"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Cryptography__GpgMeSigned[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   19,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    4,    5,

       0        // eod
};

void Cryptography::GpgMeSigned::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GpgMeSigned *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Cryptography::GpgMeSigned::staticMetaObject = { {
    QMetaObject::SuperData::link<GpgMePart::staticMetaObject>(),
    qt_meta_stringdata_Cryptography__GpgMeSigned.data,
    qt_meta_data_Cryptography__GpgMeSigned,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Cryptography::GpgMeSigned::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Cryptography::GpgMeSigned::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Cryptography__GpgMeSigned.stringdata0))
        return static_cast<void*>(this);
    return GpgMePart::qt_metacast(_clname);
}

int Cryptography::GpgMeSigned::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GpgMePart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_Cryptography__GpgMeEncrypted_t {
    QByteArrayData data[9];
    char stringdata0[109];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Cryptography__GpgMeEncrypted_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Cryptography__GpgMeEncrypted_t qt_meta_stringdata_Cryptography__GpgMeEncrypted = {
    {
QT_MOC_LITERAL(0, 0, 28), // "Cryptography::GpgMeEncrypted"
QT_MOC_LITERAL(1, 29, 17), // "handleDataChanged"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 60, 7), // "topLeft"
QT_MOC_LITERAL(5, 68, 11), // "bottomRight"
QT_MOC_LITERAL(6, 80, 20), // "processDecryptedData"
QT_MOC_LITERAL(7, 101, 2), // "ok"
QT_MOC_LITERAL(8, 104, 4) // "data"

    },
    "Cryptography::GpgMeEncrypted\0"
    "handleDataChanged\0\0QModelIndex\0topLeft\0"
    "bottomRight\0processDecryptedData\0ok\0"
    "data"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Cryptography__GpgMeEncrypted[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x08 /* Private */,
       6,    2,   29,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    4,    5,
    QMetaType::Void, QMetaType::Bool, QMetaType::QByteArray,    7,    8,

       0        // eod
};

void Cryptography::GpgMeEncrypted::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GpgMeEncrypted *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 1: _t->processDecryptedData((*reinterpret_cast< const bool(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Cryptography::GpgMeEncrypted::staticMetaObject = { {
    QMetaObject::SuperData::link<GpgMePart::staticMetaObject>(),
    qt_meta_stringdata_Cryptography__GpgMeEncrypted.data,
    qt_meta_data_Cryptography__GpgMeEncrypted,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Cryptography::GpgMeEncrypted::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Cryptography::GpgMeEncrypted::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Cryptography__GpgMeEncrypted.stringdata0))
        return static_cast<void*>(this);
    return GpgMePart::qt_metacast(_clname);
}

int Cryptography::GpgMeEncrypted::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GpgMePart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
