/********************************************************************************
** Form generated from reading UI file 'CreateMailboxDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CREATEMAILBOXDIALOG_H
#define UI_CREATEMAILBOXDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include "LineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_CreateMailboxDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    LineEdit *mailboxName;
    QHBoxLayout *horizontalLayout;
    QRadioButton *otherMailboxes;
    QRadioButton *justMessages;
    QCheckBox *subscribe;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *CreateMailboxDialog)
    {
        if (CreateMailboxDialog->objectName().isEmpty())
            CreateMailboxDialog->setObjectName(QString::fromUtf8("CreateMailboxDialog"));
        CreateMailboxDialog->resize(276, 110);
        verticalLayout = new QVBoxLayout(CreateMailboxDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label = new QLabel(CreateMailboxDialog);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout_2->addWidget(label);

        mailboxName = new LineEdit(CreateMailboxDialog);
        mailboxName->setObjectName(QString::fromUtf8("mailboxName"));

        horizontalLayout_2->addWidget(mailboxName);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        otherMailboxes = new QRadioButton(CreateMailboxDialog);
        otherMailboxes->setObjectName(QString::fromUtf8("otherMailboxes"));

        horizontalLayout->addWidget(otherMailboxes);

        justMessages = new QRadioButton(CreateMailboxDialog);
        justMessages->setObjectName(QString::fromUtf8("justMessages"));
        justMessages->setChecked(true);

        horizontalLayout->addWidget(justMessages);


        verticalLayout->addLayout(horizontalLayout);

        subscribe = new QCheckBox(CreateMailboxDialog);
        subscribe->setObjectName(QString::fromUtf8("subscribe"));

        verticalLayout->addWidget(subscribe);

        buttonBox = new QDialogButtonBox(CreateMailboxDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(mailboxName);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(mailboxName, otherMailboxes);
        QWidget::setTabOrder(otherMailboxes, justMessages);
        QWidget::setTabOrder(justMessages, buttonBox);

        retranslateUi(CreateMailboxDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), CreateMailboxDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), CreateMailboxDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(CreateMailboxDialog);
    } // setupUi

    void retranslateUi(QDialog *CreateMailboxDialog)
    {
        CreateMailboxDialog->setWindowTitle(QCoreApplication::translate("CreateMailboxDialog", "Create Mailbox", nullptr));
        label->setText(QCoreApplication::translate("CreateMailboxDialog", "&Mailbox:", nullptr));
        otherMailboxes->setText(QCoreApplication::translate("CreateMailboxDialog", "Other Mai&lboxes", nullptr));
        justMessages->setText(QCoreApplication::translate("CreateMailboxDialog", "Messages Only", nullptr));
#if QT_CONFIG(tooltip)
        subscribe->setToolTip(QCoreApplication::translate("CreateMailboxDialog", "Check to add this mailbox to the server-side list of subscribed mailboxes", nullptr));
#endif // QT_CONFIG(tooltip)
        subscribe->setText(QCoreApplication::translate("CreateMailboxDialog", "&Subscribe", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CreateMailboxDialog: public Ui_CreateMailboxDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CREATEMAILBOXDIALOG_H
