/****************************************************************************
** Meta object code from reading C++ file 'KeepMailboxOpenTask.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Tasks/KeepMailboxOpenTask.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KeepMailboxOpenTask.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Mailbox__KeepMailboxOpenTask_t {
    QByteArrayData data[26];
    char stringdata0[512];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__KeepMailboxOpenTask_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__KeepMailboxOpenTask_t qt_meta_stringdata_Imap__Mailbox__KeepMailboxOpenTask = {
    {
QT_MOC_LITERAL(0, 0, 34), // "Imap::Mailbox::KeepMailboxOpe..."
QT_MOC_LITERAL(1, 35, 15), // "slotTaskDeleted"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 6), // "object"
QT_MOC_LITERAL(4, 59, 21), // "slotPerformConnection"
QT_MOC_LITERAL(5, 81, 20), // "slotSyncHasCompleted"
QT_MOC_LITERAL(6, 102, 20), // "handleNumberResponse"
QT_MOC_LITERAL(7, 123, 37), // "Imap::Responses::NumberRespon..."
QT_MOC_LITERAL(8, 161, 4), // "resp"
QT_MOC_LITERAL(9, 166, 11), // "handleFetch"
QT_MOC_LITERAL(10, 178, 28), // "Imap::Responses::Fetch*const"
QT_MOC_LITERAL(11, 207, 17), // "handleStateHelper"
QT_MOC_LITERAL(12, 225, 28), // "Imap::Responses::State*const"
QT_MOC_LITERAL(13, 254, 11), // "handleFlags"
QT_MOC_LITERAL(14, 266, 28), // "Imap::Responses::Flags*const"
QT_MOC_LITERAL(15, 295, 14), // "handleVanished"
QT_MOC_LITERAL(16, 310, 25), // "Responses::Vanished*const"
QT_MOC_LITERAL(17, 336, 29), // "handleResponseCodeInsideState"
QT_MOC_LITERAL(18, 366, 15), // "slotPerformNoop"
QT_MOC_LITERAL(19, 382, 17), // "slotActivateTasks"
QT_MOC_LITERAL(20, 400, 23), // "slotFetchRequestedParts"
QT_MOC_LITERAL(21, 424, 27), // "slotFetchRequestedEnvelopes"
QT_MOC_LITERAL(22, 452, 14), // "slotUnselected"
QT_MOC_LITERAL(23, 467, 9), // "terminate"
QT_MOC_LITERAL(24, 477, 19), // "finalizeTermination"
QT_MOC_LITERAL(25, 497, 14) // "syncingTimeout"

    },
    "Imap::Mailbox::KeepMailboxOpenTask\0"
    "slotTaskDeleted\0\0object\0slotPerformConnection\0"
    "slotSyncHasCompleted\0handleNumberResponse\0"
    "Imap::Responses::NumberResponse*const\0"
    "resp\0handleFetch\0Imap::Responses::Fetch*const\0"
    "handleStateHelper\0Imap::Responses::State*const\0"
    "handleFlags\0Imap::Responses::Flags*const\0"
    "handleVanished\0Responses::Vanished*const\0"
    "handleResponseCodeInsideState\0"
    "slotPerformNoop\0slotActivateTasks\0"
    "slotFetchRequestedParts\0"
    "slotFetchRequestedEnvelopes\0slotUnselected\0"
    "terminate\0finalizeTermination\0"
    "syncingTimeout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__KeepMailboxOpenTask[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   99,    2, 0x08 /* Private */,
       4,    0,  102,    2, 0x08 /* Private */,
       5,    0,  103,    2, 0x08 /* Private */,
       6,    1,  104,    2, 0x08 /* Private */,
       9,    1,  107,    2, 0x08 /* Private */,
      11,    1,  110,    2, 0x08 /* Private */,
      13,    1,  113,    2, 0x08 /* Private */,
      15,    1,  116,    2, 0x08 /* Private */,
      17,    1,  119,    2, 0x08 /* Private */,
      18,    0,  122,    2, 0x08 /* Private */,
      19,    0,  123,    2, 0x08 /* Private */,
      20,    0,  124,    2, 0x08 /* Private */,
      21,    0,  125,    2, 0x08 /* Private */,
      22,    0,  126,    2, 0x08 /* Private */,
      23,    0,  127,    2, 0x08 /* Private */,
      24,    0,  128,    2, 0x08 /* Private */,
      25,    0,  129,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::QObjectStar,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, 0x80000000 | 7,    8,
    QMetaType::Bool, 0x80000000 | 10,    8,
    QMetaType::Bool, 0x80000000 | 12,    8,
    QMetaType::Bool, 0x80000000 | 14,    8,
    QMetaType::Bool, 0x80000000 | 16,    8,
    QMetaType::Bool, 0x80000000 | 12,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Imap::Mailbox::KeepMailboxOpenTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<KeepMailboxOpenTask *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotTaskDeleted((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 1: _t->slotPerformConnection(); break;
        case 2: _t->slotSyncHasCompleted(); break;
        case 3: { bool _r = _t->handleNumberResponse((*reinterpret_cast< const Imap::Responses::NumberResponse*const(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->handleFetch((*reinterpret_cast< const Imap::Responses::Fetch*const(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->handleStateHelper((*reinterpret_cast< const Imap::Responses::State*const(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->handleFlags((*reinterpret_cast< const Imap::Responses::Flags*const(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->handleVanished((*reinterpret_cast< const Responses::Vanished*const(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->handleResponseCodeInsideState((*reinterpret_cast< const Imap::Responses::State*const(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->slotPerformNoop(); break;
        case 10: _t->slotActivateTasks(); break;
        case 11: _t->slotFetchRequestedParts(); break;
        case 12: _t->slotFetchRequestedEnvelopes(); break;
        case 13: _t->slotUnselected(); break;
        case 14: _t->terminate(); break;
        case 15: _t->finalizeTermination(); break;
        case 16: _t->syncingTimeout(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::KeepMailboxOpenTask::staticMetaObject = { {
    QMetaObject::SuperData::link<ImapTask::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__KeepMailboxOpenTask.data,
    qt_meta_data_Imap__Mailbox__KeepMailboxOpenTask,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::KeepMailboxOpenTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::KeepMailboxOpenTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__KeepMailboxOpenTask.stringdata0))
        return static_cast<void*>(this);
    return ImapTask::qt_metacast(_clname);
}

int Imap::Mailbox::KeepMailboxOpenTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ImapTask::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
