/* Copyright (C) 2006 - 2014 Jan Kundrát <jkt@flaska.net>

   This file is part of the Trojita Qt IMAP e-mail client,
   http://trojita.flaska.net/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef COMMON_SQLTRANSACTIONAUTOABORTER_H
#define COMMON_SQLTRANSACTIONAUTOABORTER_H

#include <QSqlDatabase>

namespace Common
{
/** @short An auto-commiter

  A utility class using the RAII idiom -- when its instance goes out of scope,
  it aborts the current transaction
*/
class SqlTransactionAutoAborter
{
public:
    SqlTransactionAutoAborter(QSqlDatabase *db): db(db), committed(false)
    {
        db->transaction();
    }

    ~SqlTransactionAutoAborter()
    {
        if (! committed)
            db->rollback();
    }

    bool commit()
    {
        committed = db->commit();
        return committed;
    }

private:
    QSqlDatabase *db;
    bool committed;
};
}

#endif // COMMON_SQLTRANSACTIONAUTOABORTER_H
