/****************************************************************************
** Meta object code from reading C++ file 'MsgListView.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/MsgListView.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MsgListView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gui__MsgListView_t {
    QByteArrayData data[23];
    char stringdata0[364];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__MsgListView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__MsgListView_t qt_meta_stringdata_Gui__MsgListView = {
    {
QT_MOC_LITERAL(0, 0, 16), // "Gui::MsgListView"
QT_MOC_LITERAL(1, 17, 11), // "slotFixSize"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 22), // "slotExpandWholeSubtree"
QT_MOC_LITERAL(4, 53, 11), // "QModelIndex"
QT_MOC_LITERAL(5, 65, 9), // "rootIndex"
QT_MOC_LITERAL(6, 75, 23), // "slotUpdateHeaderActions"
QT_MOC_LITERAL(7, 99, 34), // "slotHeaderSectionVisibilityTo..."
QT_MOC_LITERAL(8, 134, 7), // "section"
QT_MOC_LITERAL(9, 142, 36), // "slotMsgListModelRowsAboutToBe..."
QT_MOC_LITERAL(10, 179, 6), // "parent"
QT_MOC_LITERAL(11, 186, 5), // "start"
QT_MOC_LITERAL(12, 192, 3), // "end"
QT_MOC_LITERAL(13, 196, 44), // "slotThreadingMsgListModelRowA..."
QT_MOC_LITERAL(14, 241, 5), // "index"
QT_MOC_LITERAL(15, 247, 29), // "slotHandleSortCriteriaChanged"
QT_MOC_LITERAL(16, 277, 6), // "column"
QT_MOC_LITERAL(17, 284, 13), // "Qt::SortOrder"
QT_MOC_LITERAL(18, 298, 5), // "order"
QT_MOC_LITERAL(19, 304, 20), // "slotCurrentActivated"
QT_MOC_LITERAL(20, 325, 20), // "slotHandleNewColumns"
QT_MOC_LITERAL(21, 346, 8), // "oldCount"
QT_MOC_LITERAL(22, 355, 8) // "newCount"

    },
    "Gui::MsgListView\0slotFixSize\0\0"
    "slotExpandWholeSubtree\0QModelIndex\0"
    "rootIndex\0slotUpdateHeaderActions\0"
    "slotHeaderSectionVisibilityToggled\0"
    "section\0slotMsgListModelRowsAboutToBeRemoved\0"
    "parent\0start\0end\0"
    "slotThreadingMsgListModelRowAboutToBeRemoved\0"
    "index\0slotHandleSortCriteriaChanged\0"
    "column\0Qt::SortOrder\0order\0"
    "slotCurrentActivated\0slotHandleNewColumns\0"
    "oldCount\0newCount"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__MsgListView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x08 /* Private */,
       3,    1,   60,    2, 0x08 /* Private */,
       6,    0,   63,    2, 0x08 /* Private */,
       7,    1,   64,    2, 0x08 /* Private */,
       9,    3,   67,    2, 0x08 /* Private */,
      13,    1,   74,    2, 0x08 /* Private */,
      15,    2,   77,    2, 0x08 /* Private */,
      19,    0,   82,    2, 0x08 /* Private */,
      20,    2,   83,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Int,   10,   11,   12,
    QMetaType::Void, 0x80000000 | 4,   14,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 17,   16,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   21,   22,

       0        // eod
};

void Gui::MsgListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MsgListView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotFixSize(); break;
        case 1: _t->slotExpandWholeSubtree((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 2: _t->slotUpdateHeaderActions(); break;
        case 3: _t->slotHeaderSectionVisibilityToggled((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->slotMsgListModelRowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 5: _t->slotThreadingMsgListModelRowAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 6: _t->slotHandleSortCriteriaChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Qt::SortOrder(*)>(_a[2]))); break;
        case 7: _t->slotCurrentActivated(); break;
        case 8: _t->slotHandleNewColumns((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::MsgListView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_meta_stringdata_Gui__MsgListView.data,
    qt_meta_data_Gui__MsgListView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::MsgListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::MsgListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__MsgListView.stringdata0))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int Gui::MsgListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
