/****************************************************************************
** Meta object code from reading C++ file 'PasswordWatcher.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/UiUtils/PasswordWatcher.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PasswordWatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_UiUtils__PasswordWatcher_t {
    QByteArrayData data[25];
    char stringdata0[375];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UiUtils__PasswordWatcher_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UiUtils__PasswordWatcher_t qt_meta_stringdata_UiUtils__PasswordWatcher = {
    {
QT_MOC_LITERAL(0, 0, 24), // "UiUtils::PasswordWatcher"
QT_MOC_LITERAL(1, 25, 19), // "backendMaybeChanged"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 12), // "stateChanged"
QT_MOC_LITERAL(4, 59, 13), // "readingFailed"
QT_MOC_LITERAL(5, 73, 7), // "message"
QT_MOC_LITERAL(6, 81, 12), // "savingFailed"
QT_MOC_LITERAL(7, 94, 11), // "readingDone"
QT_MOC_LITERAL(8, 106, 10), // "savingDone"
QT_MOC_LITERAL(9, 117, 14), // "reloadPassword"
QT_MOC_LITERAL(10, 132, 11), // "setPassword"
QT_MOC_LITERAL(11, 144, 8), // "password"
QT_MOC_LITERAL(12, 153, 17), // "passwordRetrieved"
QT_MOC_LITERAL(13, 171, 21), // "passwordReadingFailed"
QT_MOC_LITERAL(14, 193, 27), // "Plugins::PasswordJob::Error"
QT_MOC_LITERAL(15, 221, 9), // "errorCode"
QT_MOC_LITERAL(16, 231, 12), // "errorMessage"
QT_MOC_LITERAL(17, 244, 15), // "passwordWritten"
QT_MOC_LITERAL(18, 260, 21), // "passwordWritingFailed"
QT_MOC_LITERAL(19, 282, 18), // "isStorageEncrypted"
QT_MOC_LITERAL(20, 301, 18), // "isWaitingForPlugin"
QT_MOC_LITERAL(21, 320, 17), // "isPluginAvailable"
QT_MOC_LITERAL(22, 338, 15), // "progressMessage"
QT_MOC_LITERAL(23, 354, 9), // "didReadOk"
QT_MOC_LITERAL(24, 364, 10) // "didWriteOk"

    },
    "UiUtils::PasswordWatcher\0backendMaybeChanged\0"
    "\0stateChanged\0readingFailed\0message\0"
    "savingFailed\0readingDone\0savingDone\0"
    "reloadPassword\0setPassword\0password\0"
    "passwordRetrieved\0passwordReadingFailed\0"
    "Plugins::PasswordJob::Error\0errorCode\0"
    "errorMessage\0passwordWritten\0"
    "passwordWritingFailed\0isStorageEncrypted\0"
    "isWaitingForPlugin\0isPluginAvailable\0"
    "progressMessage\0didReadOk\0didWriteOk"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UiUtils__PasswordWatcher[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       7,  102, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x06 /* Public */,
       3,    0,   75,    2, 0x06 /* Public */,
       4,    1,   76,    2, 0x06 /* Public */,
       6,    1,   79,    2, 0x06 /* Public */,
       7,    0,   82,    2, 0x06 /* Public */,
       8,    0,   83,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   84,    2, 0x0a /* Public */,
      10,    1,   85,    2, 0x0a /* Public */,
      12,    1,   88,    2, 0x08 /* Private */,
      13,    2,   91,    2, 0x08 /* Private */,
      17,    0,   96,    2, 0x08 /* Private */,
      18,    2,   97,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, 0x80000000 | 14, QMetaType::QString,   15,   16,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14, QMetaType::QString,   15,   16,

 // properties: name, type, flags
      19, QMetaType::Bool, 0x00495001,
      20, QMetaType::Bool, 0x00495001,
      21, QMetaType::Bool, 0x00495001,
      22, QMetaType::QString, 0x00495001,
      11, QMetaType::QString, 0x00495001,
      23, QMetaType::Bool, 0x00495001,
      24, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       0,
       1,
       1,
       1,
       1,
       1,
       1,

       0        // eod
};

void UiUtils::PasswordWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PasswordWatcher *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->backendMaybeChanged(); break;
        case 1: _t->stateChanged(); break;
        case 2: _t->readingFailed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->savingFailed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->readingDone(); break;
        case 5: _t->savingDone(); break;
        case 6: _t->reloadPassword(); break;
        case 7: _t->setPassword((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->passwordRetrieved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->passwordReadingFailed((*reinterpret_cast< const Plugins::PasswordJob::Error(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 10: _t->passwordWritten(); break;
        case 11: _t->passwordWritingFailed((*reinterpret_cast< const Plugins::PasswordJob::Error(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PasswordWatcher::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PasswordWatcher::backendMaybeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PasswordWatcher::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PasswordWatcher::stateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PasswordWatcher::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PasswordWatcher::readingFailed)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PasswordWatcher::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PasswordWatcher::savingFailed)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (PasswordWatcher::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PasswordWatcher::readingDone)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (PasswordWatcher::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PasswordWatcher::savingDone)) {
                *result = 5;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<PasswordWatcher *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isStorageEncrypted(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->isWaitingForPlugin(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->isPluginAvailable(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->progressMessage(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->password(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->didReadOk(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->didWriteOk(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject UiUtils::PasswordWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_UiUtils__PasswordWatcher.data,
    qt_meta_data_UiUtils__PasswordWatcher,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UiUtils::PasswordWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UiUtils::PasswordWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UiUtils__PasswordWatcher.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int UiUtils::PasswordWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void UiUtils::PasswordWatcher::backendMaybeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void UiUtils::PasswordWatcher::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void UiUtils::PasswordWatcher::readingFailed(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void UiUtils::PasswordWatcher::savingFailed(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void UiUtils::PasswordWatcher::readingDone()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void UiUtils::PasswordWatcher::savingDone()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
