/****************************************************************************
** Meta object code from reading C++ file 'AbstractMSA.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/MSA/AbstractMSA.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AbstractMSA.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MSA__AbstractMSA_t {
    QByteArrayData data[21];
    char stringdata0[176];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MSA__AbstractMSA_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MSA__AbstractMSA_t qt_meta_stringdata_MSA__AbstractMSA = {
    {
QT_MOC_LITERAL(0, 0, 16), // "MSA::AbstractMSA"
QT_MOC_LITERAL(1, 17, 10), // "connecting"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 17), // "passwordRequested"
QT_MOC_LITERAL(4, 47, 4), // "user"
QT_MOC_LITERAL(5, 52, 4), // "host"
QT_MOC_LITERAL(6, 57, 7), // "sending"
QT_MOC_LITERAL(7, 65, 4), // "sent"
QT_MOC_LITERAL(8, 70, 5), // "error"
QT_MOC_LITERAL(9, 76, 7), // "message"
QT_MOC_LITERAL(10, 84, 11), // "progressMax"
QT_MOC_LITERAL(11, 96, 3), // "max"
QT_MOC_LITERAL(12, 100, 8), // "progress"
QT_MOC_LITERAL(13, 109, 3), // "num"
QT_MOC_LITERAL(14, 113, 6), // "logged"
QT_MOC_LITERAL(15, 120, 15), // "Common::LogKind"
QT_MOC_LITERAL(16, 136, 4), // "kind"
QT_MOC_LITERAL(17, 141, 6), // "source"
QT_MOC_LITERAL(18, 148, 6), // "cancel"
QT_MOC_LITERAL(19, 155, 11), // "setPassword"
QT_MOC_LITERAL(20, 167, 8) // "password"

    },
    "MSA::AbstractMSA\0connecting\0\0"
    "passwordRequested\0user\0host\0sending\0"
    "sent\0error\0message\0progressMax\0max\0"
    "progress\0num\0logged\0Common::LogKind\0"
    "kind\0source\0cancel\0setPassword\0password"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MSA__AbstractMSA[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,
       3,    2,   65,    2, 0x06 /* Public */,
       6,    0,   70,    2, 0x06 /* Public */,
       7,    0,   71,    2, 0x06 /* Public */,
       8,    1,   72,    2, 0x06 /* Public */,
      10,    1,   75,    2, 0x06 /* Public */,
      12,    1,   78,    2, 0x06 /* Public */,
      14,    3,   81,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    0,   88,    2, 0x0a /* Public */,
      19,    1,   89,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    4,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, 0x80000000 | 15, QMetaType::QString, QMetaType::QString,   16,   17,    9,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   20,

       0        // eod
};

void MSA::AbstractMSA::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AbstractMSA *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->connecting(); break;
        case 1: _t->passwordRequested((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 2: _t->sending(); break;
        case 3: _t->sent(); break;
        case 4: _t->error((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->progressMax((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->progress((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->logged((*reinterpret_cast< const Common::LogKind(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 8: _t->cancel(); break;
        case 9: _t->setPassword((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AbstractMSA::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractMSA::connecting)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (AbstractMSA::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractMSA::passwordRequested)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (AbstractMSA::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractMSA::sending)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (AbstractMSA::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractMSA::sent)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (AbstractMSA::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractMSA::error)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (AbstractMSA::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractMSA::progressMax)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (AbstractMSA::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractMSA::progress)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (AbstractMSA::*)(const Common::LogKind , const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AbstractMSA::logged)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MSA::AbstractMSA::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_MSA__AbstractMSA.data,
    qt_meta_data_MSA__AbstractMSA,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MSA::AbstractMSA::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MSA::AbstractMSA::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MSA__AbstractMSA.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MSA::AbstractMSA::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void MSA::AbstractMSA::connecting()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MSA::AbstractMSA::passwordRequested(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void MSA::AbstractMSA::sending()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MSA::AbstractMSA::sent()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MSA::AbstractMSA::error(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void MSA::AbstractMSA::progressMax(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void MSA::AbstractMSA::progress(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void MSA::AbstractMSA::logged(const Common::LogKind _t1, const QString & _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
