/****************************************************************************
** Meta object code from reading C++ file 'PasswordPlugin.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Plugins/PasswordPlugin.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PasswordPlugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Plugins__PasswordJob_t {
    QByteArrayData data[9];
    char stringdata0[127];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Plugins__PasswordJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Plugins__PasswordJob_t qt_meta_stringdata_Plugins__PasswordJob = {
    {
QT_MOC_LITERAL(0, 0, 20), // "Plugins::PasswordJob"
QT_MOC_LITERAL(1, 21, 17), // "passwordAvailable"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 8), // "password"
QT_MOC_LITERAL(4, 49, 14), // "passwordStored"
QT_MOC_LITERAL(5, 64, 15), // "passwordDeleted"
QT_MOC_LITERAL(6, 80, 5), // "error"
QT_MOC_LITERAL(7, 86, 27), // "Plugins::PasswordJob::Error"
QT_MOC_LITERAL(8, 114, 12) // "errorMessage"

    },
    "Plugins::PasswordJob\0passwordAvailable\0"
    "\0password\0passwordStored\0passwordDeleted\0"
    "error\0Plugins::PasswordJob::Error\0"
    "errorMessage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Plugins__PasswordJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x06 /* Public */,
       4,    0,   37,    2, 0x06 /* Public */,
       5,    0,   38,    2, 0x06 /* Public */,
       6,    2,   39,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7, QMetaType::QString,    6,    8,

       0        // eod
};

void Plugins::PasswordJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PasswordJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->passwordAvailable((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->passwordStored(); break;
        case 2: _t->passwordDeleted(); break;
        case 3: _t->error((*reinterpret_cast< const Plugins::PasswordJob::Error(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PasswordJob::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PasswordJob::passwordAvailable)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PasswordJob::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PasswordJob::passwordStored)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PasswordJob::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PasswordJob::passwordDeleted)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PasswordJob::*)(const Plugins::PasswordJob::Error , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PasswordJob::error)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Plugins::PasswordJob::staticMetaObject = { {
    QMetaObject::SuperData::link<PluginJob::staticMetaObject>(),
    qt_meta_stringdata_Plugins__PasswordJob.data,
    qt_meta_data_Plugins__PasswordJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Plugins::PasswordJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Plugins::PasswordJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Plugins__PasswordJob.stringdata0))
        return static_cast<void*>(this);
    return PluginJob::qt_metacast(_clname);
}

int Plugins::PasswordJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PluginJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Plugins::PasswordJob::passwordAvailable(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Plugins::PasswordJob::passwordStored()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Plugins::PasswordJob::passwordDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Plugins::PasswordJob::error(const Plugins::PasswordJob::Error _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
struct qt_meta_stringdata_Plugins__PasswordPlugin_t {
    QByteArrayData data[9];
    char stringdata0[114];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Plugins__PasswordPlugin_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Plugins__PasswordPlugin_t qt_meta_stringdata_Plugins__PasswordPlugin = {
    {
QT_MOC_LITERAL(0, 0, 23), // "Plugins::PasswordPlugin"
QT_MOC_LITERAL(1, 24, 15), // "requestPassword"
QT_MOC_LITERAL(2, 40, 12), // "PasswordJob*"
QT_MOC_LITERAL(3, 53, 0), // ""
QT_MOC_LITERAL(4, 54, 9), // "accountId"
QT_MOC_LITERAL(5, 64, 11), // "accountType"
QT_MOC_LITERAL(6, 76, 13), // "storePassword"
QT_MOC_LITERAL(7, 90, 8), // "password"
QT_MOC_LITERAL(8, 99, 14) // "deletePassword"

    },
    "Plugins::PasswordPlugin\0requestPassword\0"
    "PasswordJob*\0\0accountId\0accountType\0"
    "storePassword\0password\0deletePassword"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Plugins__PasswordPlugin[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   29,    3, 0x0a /* Public */,
       6,    3,   34,    3, 0x0a /* Public */,
       8,    2,   41,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2, QMetaType::QString, QMetaType::QString,    4,    5,
    0x80000000 | 2, QMetaType::QString, QMetaType::QString, QMetaType::QString,    4,    5,    7,
    0x80000000 | 2, QMetaType::QString, QMetaType::QString,    4,    5,

       0        // eod
};

void Plugins::PasswordPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PasswordPlugin *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { PasswordJob* _r = _t->requestPassword((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< PasswordJob**>(_a[0]) = std::move(_r); }  break;
        case 1: { PasswordJob* _r = _t->storePassword((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< PasswordJob**>(_a[0]) = std::move(_r); }  break;
        case 2: { PasswordJob* _r = _t->deletePassword((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< PasswordJob**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Plugins::PasswordPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Plugins__PasswordPlugin.data,
    qt_meta_data_Plugins__PasswordPlugin,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Plugins::PasswordPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Plugins::PasswordPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Plugins__PasswordPlugin.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Plugins::PasswordPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
