/********************************************************************************
** Form generated from reading UI file 'ShortcutConfigWidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHORTCUTCONFIGWIDGET_H
#define UI_SHORTCUTCONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QWidget>
#include "LineEdit.h"

QT_BEGIN_NAMESPACE

class Ui_ShortcutConfigWidget
{
public:
    QGridLayout *gridLayout;
    QLabel *descriptionLabel;
    QTreeWidget *shortcutTreeWidget;
    QHBoxLayout *searchHorizontalLayout;
    QLabel *searchLabel;
    LineEdit *searchLineEdit;
    QPushButton *clearPushButton;
    QPushButton *useDefaultPushButton;

    void setupUi(QWidget *ShortcutConfigWidget)
    {
        if (ShortcutConfigWidget->objectName().isEmpty())
            ShortcutConfigWidget->setObjectName(QString::fromUtf8("ShortcutConfigWidget"));
        ShortcutConfigWidget->resize(400, 350);
        gridLayout = new QGridLayout(ShortcutConfigWidget);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        descriptionLabel = new QLabel(ShortcutConfigWidget);
        descriptionLabel->setObjectName(QString::fromUtf8("descriptionLabel"));

        gridLayout->addWidget(descriptionLabel, 0, 0, 1, 3);

        shortcutTreeWidget = new QTreeWidget(ShortcutConfigWidget);
        shortcutTreeWidget->setObjectName(QString::fromUtf8("shortcutTreeWidget"));

        gridLayout->addWidget(shortcutTreeWidget, 1, 0, 1, 3);

        searchHorizontalLayout = new QHBoxLayout();
        searchHorizontalLayout->setObjectName(QString::fromUtf8("searchHorizontalLayout"));
        searchLabel = new QLabel(ShortcutConfigWidget);
        searchLabel->setObjectName(QString::fromUtf8("searchLabel"));

        searchHorizontalLayout->addWidget(searchLabel);

        searchLineEdit = new LineEdit(ShortcutConfigWidget);
        searchLineEdit->setObjectName(QString::fromUtf8("searchLineEdit"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(searchLineEdit->sizePolicy().hasHeightForWidth());
        searchLineEdit->setSizePolicy(sizePolicy);

        searchHorizontalLayout->addWidget(searchLineEdit);


        gridLayout->addLayout(searchHorizontalLayout, 2, 0, 1, 3);

        clearPushButton = new QPushButton(ShortcutConfigWidget);
        clearPushButton->setObjectName(QString::fromUtf8("clearPushButton"));

        gridLayout->addWidget(clearPushButton, 3, 0, 1, 1);

        useDefaultPushButton = new QPushButton(ShortcutConfigWidget);
        useDefaultPushButton->setObjectName(QString::fromUtf8("useDefaultPushButton"));

        gridLayout->addWidget(useDefaultPushButton, 3, 1, 1, 1);

#if QT_CONFIG(shortcut)
        searchLabel->setBuddy(searchLineEdit);
#endif // QT_CONFIG(shortcut)

        retranslateUi(ShortcutConfigWidget);

        QMetaObject::connectSlotsByName(ShortcutConfigWidget);
    } // setupUi

    void retranslateUi(QWidget *ShortcutConfigWidget)
    {
        ShortcutConfigWidget->setWindowTitle(QCoreApplication::translate("ShortcutConfigWidget", "Configure Shortcuts", nullptr));
        descriptionLabel->setText(QCoreApplication::translate("ShortcutConfigWidget", "Select an action and press a key combination to change its shortcut:", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = shortcutTreeWidget->headerItem();
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("ShortcutConfigWidget", "Shortcut", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("ShortcutConfigWidget", "Action", nullptr));
#if QT_CONFIG(whatsthis)
        shortcutTreeWidget->setWhatsThis(QCoreApplication::translate("ShortcutConfigWidget", "<p>Select an action (with the mouse or the arrow keys). Its shortcut can then be changed by simply pressing the new shortcut.</p><p>Note that a few shortcuts cannot be used: \"Up\", \"Down\", \"Left\", \"Right\", \"Page Up\", \"Page Down\", \"Tab\", \"Shift+Tab\", \"Enter\" and \"Escape\".</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        searchLabel->setWhatsThis(QCoreApplication::translate("ShortcutConfigWidget", "<p>Search interactively for shortcut names (e.g. Copy) or combination of keys (e.g. Ctrl+C) by typing them.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        searchLabel->setText(QCoreApplication::translate("ShortcutConfigWidget", "Search:", "Label of search field"));
#if QT_CONFIG(whatsthis)
        searchLineEdit->setWhatsThis(QCoreApplication::translate("ShortcutConfigWidget", "<p>Search interactively for shortcut names (e.g. Copy) or combination of keys (e.g. Ctrl+C) by typing them.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        clearPushButton->setWhatsThis(QCoreApplication::translate("ShortcutConfigWidget", "<p>Remove the shortcut of the currently highlighted action.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        clearPushButton->setText(QCoreApplication::translate("ShortcutConfigWidget", "Clear", "Action for clearing shortcut"));
#if QT_CONFIG(whatsthis)
        useDefaultPushButton->setWhatsThis(QCoreApplication::translate("ShortcutConfigWidget", "<p>Restore the default shortcut for the currently highlighted action.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        useDefaultPushButton->setText(QCoreApplication::translate("ShortcutConfigWidget", "Default", "Action for restoring default shortcut"));
    } // retranslateUi

};

namespace Ui {
    class ShortcutConfigWidget: public Ui_ShortcutConfigWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SHORTCUTCONFIGWIDGET_H
