/****************************************************************************
** Meta object code from reading C++ file 'SubtreeModel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Model/SubtreeModel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SubtreeModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Mailbox__SubtreeModel_t {
    QByteArrayData data[24];
    char stringdata0[352];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__SubtreeModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__SubtreeModel_t qt_meta_stringdata_Imap__Mailbox__SubtreeModel = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Imap::Mailbox::SubtreeModel"
QT_MOC_LITERAL(1, 28, 15), // "validityChanged"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 17), // "handleDataChanged"
QT_MOC_LITERAL(4, 63, 11), // "QModelIndex"
QT_MOC_LITERAL(5, 75, 7), // "topLeft"
QT_MOC_LITERAL(6, 83, 11), // "bottomRight"
QT_MOC_LITERAL(7, 95, 25), // "handleModelAboutToBeReset"
QT_MOC_LITERAL(8, 121, 16), // "handleModelReset"
QT_MOC_LITERAL(9, 138, 26), // "handleRowsAboutToBeRemoved"
QT_MOC_LITERAL(10, 165, 6), // "parent"
QT_MOC_LITERAL(11, 172, 5), // "first"
QT_MOC_LITERAL(12, 178, 4), // "last"
QT_MOC_LITERAL(13, 183, 17), // "handleRowsRemoved"
QT_MOC_LITERAL(14, 201, 27), // "handleRowsAboutToBeInserted"
QT_MOC_LITERAL(15, 229, 18), // "handleRowsInserted"
QT_MOC_LITERAL(16, 248, 11), // "setRootItem"
QT_MOC_LITERAL(17, 260, 9), // "rootIndex"
QT_MOC_LITERAL(18, 270, 19), // "setRootItemByOffset"
QT_MOC_LITERAL(19, 290, 3), // "row"
QT_MOC_LITERAL(20, 294, 17), // "setRootOneLevelUp"
QT_MOC_LITERAL(21, 312, 15), // "setOriginalRoot"
QT_MOC_LITERAL(22, 328, 12), // "parentOfRoot"
QT_MOC_LITERAL(23, 341, 10) // "itemsValid"

    },
    "Imap::Mailbox::SubtreeModel\0validityChanged\0"
    "\0handleDataChanged\0QModelIndex\0topLeft\0"
    "bottomRight\0handleModelAboutToBeReset\0"
    "handleModelReset\0handleRowsAboutToBeRemoved\0"
    "parent\0first\0last\0handleRowsRemoved\0"
    "handleRowsAboutToBeInserted\0"
    "handleRowsInserted\0setRootItem\0rootIndex\0"
    "setRootItemByOffset\0row\0setRootOneLevelUp\0"
    "setOriginalRoot\0parentOfRoot\0itemsValid"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__SubtreeModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       1,  130, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   84,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    2,   85,    2, 0x08 /* Private */,
       7,    0,   90,    2, 0x08 /* Private */,
       8,    0,   91,    2, 0x08 /* Private */,
       9,    3,   92,    2, 0x08 /* Private */,
      13,    3,   99,    2, 0x08 /* Private */,
      14,    3,  106,    2, 0x08 /* Private */,
      15,    3,  113,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      16,    1,  120,    2, 0x02 /* Public */,
      18,    1,  123,    2, 0x02 /* Public */,
      20,    0,  126,    2, 0x02 /* Public */,
      21,    0,  127,    2, 0x02 /* Public */,
      22,    0,  128,    2, 0x02 /* Public */,
      23,    0,  129,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 4,    5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Int,   10,   11,   12,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Int,   10,   11,   12,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Int,   10,   11,   12,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Int,   10,   11,   12,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 4,   17,
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void,
    QMetaType::Void,
    0x80000000 | 4,
    QMetaType::Bool,

 // properties: name, type, flags
      23, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       0,

       0        // eod
};

void Imap::Mailbox::SubtreeModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SubtreeModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->validityChanged(); break;
        case 1: _t->handleDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 2: _t->handleModelAboutToBeReset(); break;
        case 3: _t->handleModelReset(); break;
        case 4: _t->handleRowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 5: _t->handleRowsRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 6: _t->handleRowsAboutToBeInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 7: _t->handleRowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 8: _t->setRootItem((*reinterpret_cast< QModelIndex(*)>(_a[1]))); break;
        case 9: _t->setRootItemByOffset((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 10: _t->setRootOneLevelUp(); break;
        case 11: _t->setOriginalRoot(); break;
        case 12: { QModelIndex _r = _t->parentOfRoot();
            if (_a[0]) *reinterpret_cast< QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->itemsValid();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SubtreeModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SubtreeModel::validityChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<SubtreeModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->itemsValid(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::SubtreeModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractProxyModel::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__SubtreeModel.data,
    qt_meta_data_Imap__Mailbox__SubtreeModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::SubtreeModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::SubtreeModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__SubtreeModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractProxyModel::qt_metacast(_clname);
}

int Imap::Mailbox::SubtreeModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Imap::Mailbox::SubtreeModel::validityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfModel_t {
    QByteArrayData data[1];
    char stringdata0[35];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfModel_t qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfModel = {
    {
QT_MOC_LITERAL(0, 0, 34) // "Imap::Mailbox::SubtreeModelOf..."

    },
    "Imap::Mailbox::SubtreeModelOfModel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__SubtreeModelOfModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Imap::Mailbox::SubtreeModelOfModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::SubtreeModelOfModel::staticMetaObject = { {
    QMetaObject::SuperData::link<SubtreeModel::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfModel.data,
    qt_meta_data_Imap__Mailbox__SubtreeModelOfModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::SubtreeModelOfModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::SubtreeModelOfModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfModel.stringdata0))
        return static_cast<void*>(this);
    return SubtreeModel::qt_metacast(_clname);
}

int Imap::Mailbox::SubtreeModelOfModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SubtreeModel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfMailboxModel_t {
    QByteArrayData data[1];
    char stringdata0[42];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfMailboxModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfMailboxModel_t qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfMailboxModel = {
    {
QT_MOC_LITERAL(0, 0, 41) // "Imap::Mailbox::SubtreeModelOf..."

    },
    "Imap::Mailbox::SubtreeModelOfMailboxModel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__SubtreeModelOfMailboxModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Imap::Mailbox::SubtreeModelOfMailboxModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::SubtreeModelOfMailboxModel::staticMetaObject = { {
    QMetaObject::SuperData::link<SubtreeModel::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfMailboxModel.data,
    qt_meta_data_Imap__Mailbox__SubtreeModelOfMailboxModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::SubtreeModelOfMailboxModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::SubtreeModelOfMailboxModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__SubtreeModelOfMailboxModel.stringdata0))
        return static_cast<void*>(this);
    return SubtreeModel::qt_metacast(_clname);
}

int Imap::Mailbox::SubtreeModelOfMailboxModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SubtreeModel::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
