// This file is autogenerated. Changes will be overwritten.
#include "VNTOB5ESJV/moc_AddressbookPlugin.cpp"
#include "VNTOB5ESJV/moc_PasswordPlugin.cpp"
#include "VNTOB5ESJV/moc_PluginJob.cpp"
#include "VNTOB5ESJV/moc_PluginManager.cpp"
#include "VNTOB5ESJV/moc_SpellcheckerPlugin.cpp"
