/********************************************************************************
** Form generated from reading UI file 'ImapInfoDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAPINFODIALOG_H
#define UI_IMAPINFODIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ImapInfoDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *main;
    QVBoxLayout *column;
    QLabel *version;
    QScrollArea *scrollableCapabilities;
    QLabel *capabilities;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ImapInfoDialog)
    {
        if (ImapInfoDialog->objectName().isEmpty())
            ImapInfoDialog->setObjectName(QString::fromUtf8("ImapInfoDialog"));
        ImapInfoDialog->resize(350, 400);
        verticalLayout = new QVBoxLayout(ImapInfoDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(-1, -1, 0, -1);
        main = new QHBoxLayout();
        main->setSpacing(10);
        main->setObjectName(QString::fromUtf8("main"));
        column = new QVBoxLayout();
        column->setSpacing(20);
        column->setObjectName(QString::fromUtf8("column"));
        version = new QLabel(ImapInfoDialog);
        version->setObjectName(QString::fromUtf8("version"));
        version->setWordWrap(true);

        column->addWidget(version);

        scrollableCapabilities = new QScrollArea(ImapInfoDialog);
        scrollableCapabilities->setObjectName(QString::fromUtf8("scrollableCapabilities"));
        scrollableCapabilities->setFrameShape(QFrame::NoFrame);
        scrollableCapabilities->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        scrollableCapabilities->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        scrollableCapabilities->setWidgetResizable(true);
        capabilities = new QLabel();
        capabilities->setObjectName(QString::fromUtf8("capabilities"));
        capabilities->setGeometry(QRect(0, 0, 340, 306));
        QFont font;
        font.setFamily(QString::fromUtf8("Monospace"));
        capabilities->setFont(font);
        capabilities->setText(QString::fromUtf8("CAPABILITIES placeholder"));
        capabilities->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        capabilities->setWordWrap(true);
        capabilities->setProperty("readOnly", QVariant(true));
        scrollableCapabilities->setWidget(capabilities);

        column->addWidget(scrollableCapabilities);

        column->setStretch(1, 1);

        main->addLayout(column);


        verticalLayout->addLayout(main);

        buttonBox = new QDialogButtonBox(ImapInfoDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(true);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(ImapInfoDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), ImapInfoDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), ImapInfoDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(ImapInfoDialog);
    } // setupUi

    void retranslateUi(QDialog *ImapInfoDialog)
    {
        ImapInfoDialog->setWindowTitle(QCoreApplication::translate("ImapInfoDialog", "IMAP Server Info", nullptr));
        version->setText(QCoreApplication::translate("ImapInfoDialog", "<html><head/><body><p>SERVERINFO Placeholder</p></body></html>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ImapInfoDialog: public Ui_ImapInfoDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMAPINFODIALOG_H
