/****************************************************************************
** Meta object code from reading C++ file 'Parser.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Parser/Parser.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Parser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Parser_t {
    QByteArrayData data[123];
    char stringdata0[1399];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Parser_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Parser_t qt_meta_stringdata_Imap__Parser = {
    {
QT_MOC_LITERAL(0, 0, 12), // "Imap::Parser"
QT_MOC_LITERAL(1, 13, 16), // "responseReceived"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 13), // "Imap::Parser*"
QT_MOC_LITERAL(4, 45, 6), // "parser"
QT_MOC_LITERAL(5, 52, 12), // "lineReceived"
QT_MOC_LITERAL(6, 65, 4), // "line"
QT_MOC_LITERAL(7, 70, 8), // "lineSent"
QT_MOC_LITERAL(8, 79, 13), // "parserWarning"
QT_MOC_LITERAL(9, 93, 7), // "message"
QT_MOC_LITERAL(10, 101, 17), // "const QByteArray*"
QT_MOC_LITERAL(11, 119, 8), // "position"
QT_MOC_LITERAL(12, 128, 13), // "commandQueued"
QT_MOC_LITERAL(13, 142, 22), // "connectionStateChanged"
QT_MOC_LITERAL(14, 165, 21), // "Imap::ConnectionState"
QT_MOC_LITERAL(15, 187, 10), // "capability"
QT_MOC_LITERAL(16, 198, 13), // "CommandHandle"
QT_MOC_LITERAL(17, 212, 4), // "noop"
QT_MOC_LITERAL(18, 217, 6), // "logout"
QT_MOC_LITERAL(19, 224, 8), // "startTls"
QT_MOC_LITERAL(20, 233, 5), // "login"
QT_MOC_LITERAL(21, 239, 4), // "user"
QT_MOC_LITERAL(22, 244, 4), // "pass"
QT_MOC_LITERAL(23, 249, 6), // "select"
QT_MOC_LITERAL(24, 256, 7), // "mailbox"
QT_MOC_LITERAL(25, 264, 6), // "params"
QT_MOC_LITERAL(26, 271, 13), // "selectQresync"
QT_MOC_LITERAL(27, 285, 11), // "uidValidity"
QT_MOC_LITERAL(28, 297, 13), // "highestModSeq"
QT_MOC_LITERAL(29, 311, 8), // "Sequence"
QT_MOC_LITERAL(30, 320, 9), // "knownUids"
QT_MOC_LITERAL(31, 330, 16), // "sequenceSnapshot"
QT_MOC_LITERAL(32, 347, 11), // "uidSnapshot"
QT_MOC_LITERAL(33, 359, 7), // "examine"
QT_MOC_LITERAL(34, 367, 6), // "create"
QT_MOC_LITERAL(35, 374, 13), // "deleteMailbox"
QT_MOC_LITERAL(36, 388, 6), // "rename"
QT_MOC_LITERAL(37, 395, 7), // "oldName"
QT_MOC_LITERAL(38, 403, 7), // "newName"
QT_MOC_LITERAL(39, 411, 9), // "subscribe"
QT_MOC_LITERAL(40, 421, 11), // "unSubscribe"
QT_MOC_LITERAL(41, 433, 4), // "list"
QT_MOC_LITERAL(42, 438, 9), // "reference"
QT_MOC_LITERAL(43, 448, 13), // "returnOptions"
QT_MOC_LITERAL(44, 462, 4), // "lSub"
QT_MOC_LITERAL(45, 467, 6), // "status"
QT_MOC_LITERAL(46, 474, 6), // "fields"
QT_MOC_LITERAL(47, 481, 6), // "append"
QT_MOC_LITERAL(48, 488, 5), // "flags"
QT_MOC_LITERAL(49, 494, 9), // "timestamp"
QT_MOC_LITERAL(50, 504, 14), // "appendCatenate"
QT_MOC_LITERAL(51, 519, 34), // "QList<Imap::Mailbox::Catenate..."
QT_MOC_LITERAL(52, 554, 4), // "data"
QT_MOC_LITERAL(53, 559, 5), // "check"
QT_MOC_LITERAL(54, 565, 5), // "close"
QT_MOC_LITERAL(55, 571, 7), // "expunge"
QT_MOC_LITERAL(56, 579, 6), // "search"
QT_MOC_LITERAL(57, 586, 8), // "criteria"
QT_MOC_LITERAL(58, 595, 7), // "charset"
QT_MOC_LITERAL(59, 603, 5), // "fetch"
QT_MOC_LITERAL(60, 609, 3), // "seq"
QT_MOC_LITERAL(61, 613, 5), // "items"
QT_MOC_LITERAL(62, 619, 24), // "QMap<QByteArray,quint64>"
QT_MOC_LITERAL(63, 644, 15), // "uint64Modifiers"
QT_MOC_LITERAL(64, 660, 5), // "store"
QT_MOC_LITERAL(65, 666, 4), // "item"
QT_MOC_LITERAL(66, 671, 5), // "value"
QT_MOC_LITERAL(67, 677, 4), // "copy"
QT_MOC_LITERAL(68, 682, 8), // "uidFetch"
QT_MOC_LITERAL(69, 691, 8), // "uidStore"
QT_MOC_LITERAL(70, 700, 7), // "uidCopy"
QT_MOC_LITERAL(71, 708, 7), // "uidMove"
QT_MOC_LITERAL(72, 716, 10), // "uidExpunge"
QT_MOC_LITERAL(73, 727, 9), // "uidSearch"
QT_MOC_LITERAL(74, 737, 12), // "uidSearchUid"
QT_MOC_LITERAL(75, 750, 8), // "sequence"
QT_MOC_LITERAL(76, 759, 13), // "uidESearchUid"
QT_MOC_LITERAL(77, 773, 5), // "xAtom"
QT_MOC_LITERAL(78, 779, 17), // "Commands::Command"
QT_MOC_LITERAL(79, 797, 8), // "commands"
QT_MOC_LITERAL(80, 806, 8), // "unSelect"
QT_MOC_LITERAL(81, 815, 4), // "idle"
QT_MOC_LITERAL(82, 820, 8), // "idleDone"
QT_MOC_LITERAL(83, 829, 24), // "idleContinuationWontCome"
QT_MOC_LITERAL(84, 854, 31), // "idleMagicallyTerminatedByServer"
QT_MOC_LITERAL(85, 886, 16), // "namespaceCommand"
QT_MOC_LITERAL(86, 903, 4), // "sort"
QT_MOC_LITERAL(87, 908, 12), // "sortCriteria"
QT_MOC_LITERAL(88, 921, 14), // "searchCriteria"
QT_MOC_LITERAL(89, 936, 7), // "uidSort"
QT_MOC_LITERAL(90, 944, 6), // "thread"
QT_MOC_LITERAL(91, 951, 4), // "algo"
QT_MOC_LITERAL(92, 956, 9), // "uidThread"
QT_MOC_LITERAL(93, 966, 8), // "uidESort"
QT_MOC_LITERAL(94, 975, 10), // "uidESearch"
QT_MOC_LITERAL(95, 986, 10), // "uidEThread"
QT_MOC_LITERAL(96, 997, 12), // "cancelUpdate"
QT_MOC_LITERAL(97, 1010, 3), // "tag"
QT_MOC_LITERAL(98, 1014, 9), // "idCommand"
QT_MOC_LITERAL(99, 1024, 27), // "QMap<QByteArray,QByteArray>"
QT_MOC_LITERAL(100, 1052, 4), // "args"
QT_MOC_LITERAL(101, 1057, 6), // "enable"
QT_MOC_LITERAL(102, 1064, 10), // "extensions"
QT_MOC_LITERAL(103, 1075, 15), // "compressDeflate"
QT_MOC_LITERAL(104, 1091, 10), // "genUrlAuth"
QT_MOC_LITERAL(105, 1102, 3), // "url"
QT_MOC_LITERAL(106, 1106, 9), // "mechanism"
QT_MOC_LITERAL(107, 1116, 11), // "uidSendmail"
QT_MOC_LITERAL(108, 1128, 3), // "uid"
QT_MOC_LITERAL(109, 1132, 29), // "Mailbox::UidSubmitOptionsList"
QT_MOC_LITERAL(110, 1162, 17), // "submissionOptions"
QT_MOC_LITERAL(111, 1180, 22), // "slotSocketStateChanged"
QT_MOC_LITERAL(112, 1203, 9), // "connState"
QT_MOC_LITERAL(113, 1213, 23), // "unfreezeAfterEncryption"
QT_MOC_LITERAL(114, 1237, 15), // "closeConnection"
QT_MOC_LITERAL(115, 1253, 15), // "handleReadyRead"
QT_MOC_LITERAL(116, 1269, 18), // "handleDisconnected"
QT_MOC_LITERAL(117, 1288, 6), // "reason"
QT_MOC_LITERAL(118, 1295, 15), // "executeACommand"
QT_MOC_LITERAL(119, 1311, 15), // "executeCommands"
QT_MOC_LITERAL(120, 1327, 14), // "finishStartTls"
QT_MOC_LITERAL(121, 1342, 21), // "handleSocketEncrypted"
QT_MOC_LITERAL(122, 1364, 34) // "handleCompressionPossibleActi..."

    },
    "Imap::Parser\0responseReceived\0\0"
    "Imap::Parser*\0parser\0lineReceived\0"
    "line\0lineSent\0parserWarning\0message\0"
    "const QByteArray*\0position\0commandQueued\0"
    "connectionStateChanged\0Imap::ConnectionState\0"
    "capability\0CommandHandle\0noop\0logout\0"
    "startTls\0login\0user\0pass\0select\0mailbox\0"
    "params\0selectQresync\0uidValidity\0"
    "highestModSeq\0Sequence\0knownUids\0"
    "sequenceSnapshot\0uidSnapshot\0examine\0"
    "create\0deleteMailbox\0rename\0oldName\0"
    "newName\0subscribe\0unSubscribe\0list\0"
    "reference\0returnOptions\0lSub\0status\0"
    "fields\0append\0flags\0timestamp\0"
    "appendCatenate\0QList<Imap::Mailbox::CatenatePair>\0"
    "data\0check\0close\0expunge\0search\0"
    "criteria\0charset\0fetch\0seq\0items\0"
    "QMap<QByteArray,quint64>\0uint64Modifiers\0"
    "store\0item\0value\0copy\0uidFetch\0uidStore\0"
    "uidCopy\0uidMove\0uidExpunge\0uidSearch\0"
    "uidSearchUid\0sequence\0uidESearchUid\0"
    "xAtom\0Commands::Command\0commands\0"
    "unSelect\0idle\0idleDone\0idleContinuationWontCome\0"
    "idleMagicallyTerminatedByServer\0"
    "namespaceCommand\0sort\0sortCriteria\0"
    "searchCriteria\0uidSort\0thread\0algo\0"
    "uidThread\0uidESort\0uidESearch\0uidEThread\0"
    "cancelUpdate\0tag\0idCommand\0"
    "QMap<QByteArray,QByteArray>\0args\0"
    "enable\0extensions\0compressDeflate\0"
    "genUrlAuth\0url\0mechanism\0uidSendmail\0"
    "uid\0Mailbox::UidSubmitOptionsList\0"
    "submissionOptions\0slotSocketStateChanged\0"
    "connState\0unfreezeAfterEncryption\0"
    "closeConnection\0handleReadyRead\0"
    "handleDisconnected\0reason\0executeACommand\0"
    "executeCommands\0finishStartTls\0"
    "handleSocketEncrypted\0"
    "handleCompressionPossibleActivated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Parser[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      83,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  429,    2, 0x06 /* Public */,
       5,    2,  432,    2, 0x06 /* Public */,
       7,    2,  437,    2, 0x06 /* Public */,
       8,    4,  442,    2, 0x06 /* Public */,
      12,    0,  451,    2, 0x06 /* Public */,
      13,    2,  452,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    0,  457,    2, 0x0a /* Public */,
      17,    0,  458,    2, 0x0a /* Public */,
      18,    0,  459,    2, 0x0a /* Public */,
      19,    0,  460,    2, 0x0a /* Public */,
      20,    2,  461,    2, 0x0a /* Public */,
      23,    2,  466,    2, 0x0a /* Public */,
      23,    1,  471,    2, 0x2a /* Public | MethodCloned */,
      26,    6,  474,    2, 0x0a /* Public */,
      26,    5,  487,    2, 0x2a /* Public | MethodCloned */,
      26,    4,  498,    2, 0x2a /* Public | MethodCloned */,
      26,    3,  507,    2, 0x2a /* Public | MethodCloned */,
      33,    2,  514,    2, 0x0a /* Public */,
      33,    1,  519,    2, 0x2a /* Public | MethodCloned */,
      34,    1,  522,    2, 0x0a /* Public */,
      35,    1,  525,    2, 0x0a /* Public */,
      36,    2,  528,    2, 0x0a /* Public */,
      39,    1,  533,    2, 0x0a /* Public */,
      40,    1,  536,    2, 0x0a /* Public */,
      41,    3,  539,    2, 0x0a /* Public */,
      41,    2,  546,    2, 0x2a /* Public | MethodCloned */,
      44,    2,  551,    2, 0x0a /* Public */,
      45,    2,  556,    2, 0x0a /* Public */,
      47,    4,  561,    2, 0x0a /* Public */,
      47,    3,  570,    2, 0x2a /* Public | MethodCloned */,
      47,    2,  577,    2, 0x2a /* Public | MethodCloned */,
      50,    4,  582,    2, 0x0a /* Public */,
      50,    3,  591,    2, 0x2a /* Public | MethodCloned */,
      50,    2,  598,    2, 0x2a /* Public | MethodCloned */,
      53,    0,  603,    2, 0x0a /* Public */,
      54,    0,  604,    2, 0x0a /* Public */,
      55,    0,  605,    2, 0x0a /* Public */,
      56,    2,  606,    2, 0x0a /* Public */,
      56,    1,  611,    2, 0x2a /* Public | MethodCloned */,
      59,    3,  614,    2, 0x0a /* Public */,
      59,    2,  621,    2, 0x2a /* Public | MethodCloned */,
      64,    3,  626,    2, 0x0a /* Public */,
      67,    2,  633,    2, 0x0a /* Public */,
      68,    2,  638,    2, 0x0a /* Public */,
      69,    3,  643,    2, 0x0a /* Public */,
      70,    2,  650,    2, 0x0a /* Public */,
      71,    2,  655,    2, 0x0a /* Public */,
      72,    1,  660,    2, 0x0a /* Public */,
      73,    2,  663,    2, 0x0a /* Public */,
      73,    1,  668,    2, 0x2a /* Public | MethodCloned */,
      74,    1,  671,    2, 0x0a /* Public */,
      76,    1,  674,    2, 0x0a /* Public */,
      77,    1,  677,    2, 0x0a /* Public */,
      80,    0,  680,    2, 0x0a /* Public */,
      81,    0,  681,    2, 0x0a /* Public */,
      82,    0,  682,    2, 0x0a /* Public */,
      83,    0,  683,    2, 0x0a /* Public */,
      84,    0,  684,    2, 0x0a /* Public */,
      85,    0,  685,    2, 0x0a /* Public */,
      86,    3,  686,    2, 0x0a /* Public */,
      89,    3,  693,    2, 0x0a /* Public */,
      90,    3,  700,    2, 0x0a /* Public */,
      92,    3,  707,    2, 0x0a /* Public */,
      93,    4,  714,    2, 0x0a /* Public */,
      94,    3,  723,    2, 0x0a /* Public */,
      95,    4,  730,    2, 0x0a /* Public */,
      96,    1,  739,    2, 0x0a /* Public */,
      98,    0,  742,    2, 0x0a /* Public */,
      98,    1,  743,    2, 0x0a /* Public */,
     101,    1,  746,    2, 0x0a /* Public */,
     103,    0,  749,    2, 0x0a /* Public */,
     104,    2,  750,    2, 0x0a /* Public */,
     107,    2,  755,    2, 0x0a /* Public */,
     111,    2,  760,    2, 0x0a /* Public */,
     113,    0,  765,    2, 0x0a /* Public */,
     114,    0,  766,    2, 0x0a /* Public */,
     115,    0,  767,    2, 0x08 /* Private */,
     116,    1,  768,    2, 0x08 /* Private */,
     118,    0,  771,    2, 0x08 /* Private */,
     119,    0,  772,    2, 0x08 /* Private */,
     120,    0,  773,    2, 0x08 /* Private */,
     121,    0,  774,    2, 0x08 /* Private */,
     122,    0,  775,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QByteArray,    4,    6,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QByteArray,    4,    6,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString, 0x80000000 | 10, QMetaType::UInt,    4,    9,    6,   11,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 14,    4,    2,

 // slots: parameters
    0x80000000 | 16,
    0x80000000 | 16,
    0x80000000 | 16,
    0x80000000 | 16,
    0x80000000 | 16, QMetaType::QString, QMetaType::QString,   21,   22,
    0x80000000 | 16, QMetaType::QString, QMetaType::QByteArrayList,   24,   25,
    0x80000000 | 16, QMetaType::QString,   24,
    0x80000000 | 16, QMetaType::QString, QMetaType::UInt, QMetaType::ULongLong, 0x80000000 | 29, 0x80000000 | 29, 0x80000000 | 29,   24,   27,   28,   30,   31,   32,
    0x80000000 | 16, QMetaType::QString, QMetaType::UInt, QMetaType::ULongLong, 0x80000000 | 29, 0x80000000 | 29,   24,   27,   28,   30,   31,
    0x80000000 | 16, QMetaType::QString, QMetaType::UInt, QMetaType::ULongLong, 0x80000000 | 29,   24,   27,   28,   30,
    0x80000000 | 16, QMetaType::QString, QMetaType::UInt, QMetaType::ULongLong,   24,   27,   28,
    0x80000000 | 16, QMetaType::QString, QMetaType::QByteArrayList,   24,   25,
    0x80000000 | 16, QMetaType::QString,   24,
    0x80000000 | 16, QMetaType::QString,   24,
    0x80000000 | 16, QMetaType::QString,   24,
    0x80000000 | 16, QMetaType::QString, QMetaType::QString,   37,   38,
    0x80000000 | 16, QMetaType::QString,   24,
    0x80000000 | 16, QMetaType::QString,   24,
    0x80000000 | 16, QMetaType::QString, QMetaType::QString, QMetaType::QStringList,   42,   24,   43,
    0x80000000 | 16, QMetaType::QString, QMetaType::QString,   42,   24,
    0x80000000 | 16, QMetaType::QString, QMetaType::QString,   42,   24,
    0x80000000 | 16, QMetaType::QString, QMetaType::QStringList,   24,   46,
    0x80000000 | 16, QMetaType::QString, QMetaType::QByteArray, QMetaType::QStringList, QMetaType::QDateTime,   24,    9,   48,   49,
    0x80000000 | 16, QMetaType::QString, QMetaType::QByteArray, QMetaType::QStringList,   24,    9,   48,
    0x80000000 | 16, QMetaType::QString, QMetaType::QByteArray,   24,    9,
    0x80000000 | 16, QMetaType::QString, 0x80000000 | 51, QMetaType::QStringList, QMetaType::QDateTime,   24,   52,   48,   49,
    0x80000000 | 16, QMetaType::QString, 0x80000000 | 51, QMetaType::QStringList,   24,   52,   48,
    0x80000000 | 16, QMetaType::QString, 0x80000000 | 51,   24,   52,
    0x80000000 | 16,
    0x80000000 | 16,
    0x80000000 | 16,
    0x80000000 | 16, QMetaType::QStringList, QMetaType::QByteArray,   57,   58,
    0x80000000 | 16, QMetaType::QStringList,   57,
    0x80000000 | 16, 0x80000000 | 29, QMetaType::QStringList, 0x80000000 | 62,   60,   61,   63,
    0x80000000 | 16, 0x80000000 | 29, QMetaType::QStringList,   60,   61,
    0x80000000 | 16, 0x80000000 | 29, QMetaType::QString, QMetaType::QString,   60,   65,   66,
    0x80000000 | 16, 0x80000000 | 29, QMetaType::QString,   60,   24,
    0x80000000 | 16, 0x80000000 | 29, QMetaType::QByteArrayList,   60,   61,
    0x80000000 | 16, 0x80000000 | 29, QMetaType::QString, QMetaType::QString,   60,   65,   66,
    0x80000000 | 16, 0x80000000 | 29, QMetaType::QString,   60,   24,
    0x80000000 | 16, 0x80000000 | 29, QMetaType::QString,   60,   24,
    0x80000000 | 16, 0x80000000 | 29,   60,
    0x80000000 | 16, QMetaType::QStringList, QMetaType::QByteArray,   57,   58,
    0x80000000 | 16, QMetaType::QStringList,   57,
    0x80000000 | 16, QMetaType::QByteArray,   75,
    0x80000000 | 16, QMetaType::QByteArray,   75,
    0x80000000 | 16, 0x80000000 | 78,   79,
    0x80000000 | 16,
    0x80000000 | 16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    0x80000000 | 16,
    0x80000000 | 16, QMetaType::QStringList, QMetaType::QByteArray, QMetaType::QStringList,   87,   58,   88,
    0x80000000 | 16, QMetaType::QStringList, QMetaType::QByteArray, QMetaType::QStringList,   87,   58,   88,
    0x80000000 | 16, QMetaType::QByteArray, QMetaType::QByteArray, QMetaType::QStringList,   91,   58,   88,
    0x80000000 | 16, QMetaType::QByteArray, QMetaType::QByteArray, QMetaType::QStringList,   91,   58,   88,
    0x80000000 | 16, QMetaType::QStringList, QMetaType::QByteArray, QMetaType::QStringList, QMetaType::QStringList,   87,   58,   88,   43,
    0x80000000 | 16, QMetaType::QByteArray, QMetaType::QStringList, QMetaType::QStringList,   58,   88,   43,
    0x80000000 | 16, QMetaType::QByteArray, QMetaType::QByteArray, QMetaType::QStringList, QMetaType::QStringList,   91,   58,   88,   43,
    0x80000000 | 16, 0x80000000 | 16,   97,
    0x80000000 | 16,
    0x80000000 | 16, 0x80000000 | 99,  100,
    0x80000000 | 16, QMetaType::QByteArrayList,  102,
    0x80000000 | 16,
    0x80000000 | 16, QMetaType::QByteArray, QMetaType::QByteArray,  105,  106,
    0x80000000 | 16, QMetaType::UInt, 0x80000000 | 109,  108,  110,
    QMetaType::Void, 0x80000000 | 14, QMetaType::QString,  112,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,  117,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Imap::Parser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Parser *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->responseReceived((*reinterpret_cast< Imap::Parser*(*)>(_a[1]))); break;
        case 1: _t->lineReceived((*reinterpret_cast< Imap::Parser*(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 2: _t->lineSent((*reinterpret_cast< Imap::Parser*(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 3: _t->parserWarning((*reinterpret_cast< Imap::Parser*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QByteArray*(*)>(_a[3])),(*reinterpret_cast< uint(*)>(_a[4]))); break;
        case 4: _t->commandQueued(); break;
        case 5: _t->connectionStateChanged((*reinterpret_cast< Imap::Parser*(*)>(_a[1])),(*reinterpret_cast< Imap::ConnectionState(*)>(_a[2]))); break;
        case 6: { CommandHandle _r = _t->capability();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 7: { CommandHandle _r = _t->noop();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 8: { CommandHandle _r = _t->logout();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 9: { CommandHandle _r = _t->startTls();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 10: { CommandHandle _r = _t->login((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 11: { CommandHandle _r = _t->select((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<QByteArray>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 12: { CommandHandle _r = _t->select((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 13: { CommandHandle _r = _t->selectQresync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const uint(*)>(_a[2])),(*reinterpret_cast< const quint64(*)>(_a[3])),(*reinterpret_cast< const Sequence(*)>(_a[4])),(*reinterpret_cast< const Sequence(*)>(_a[5])),(*reinterpret_cast< const Sequence(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 14: { CommandHandle _r = _t->selectQresync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const uint(*)>(_a[2])),(*reinterpret_cast< const quint64(*)>(_a[3])),(*reinterpret_cast< const Sequence(*)>(_a[4])),(*reinterpret_cast< const Sequence(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 15: { CommandHandle _r = _t->selectQresync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const uint(*)>(_a[2])),(*reinterpret_cast< const quint64(*)>(_a[3])),(*reinterpret_cast< const Sequence(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 16: { CommandHandle _r = _t->selectQresync((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const uint(*)>(_a[2])),(*reinterpret_cast< const quint64(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 17: { CommandHandle _r = _t->examine((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<QByteArray>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 18: { CommandHandle _r = _t->examine((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 19: { CommandHandle _r = _t->create((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 20: { CommandHandle _r = _t->deleteMailbox((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 21: { CommandHandle _r = _t->rename((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 22: { CommandHandle _r = _t->subscribe((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 23: { CommandHandle _r = _t->unSubscribe((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 24: { CommandHandle _r = _t->list((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 25: { CommandHandle _r = _t->list((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 26: { CommandHandle _r = _t->lSub((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 27: { CommandHandle _r = _t->status((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 28: { CommandHandle _r = _t->append((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])),(*reinterpret_cast< const QDateTime(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 29: { CommandHandle _r = _t->append((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 30: { CommandHandle _r = _t->append((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 31: { CommandHandle _r = _t->appendCatenate((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<Imap::Mailbox::CatenatePair>(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])),(*reinterpret_cast< const QDateTime(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 32: { CommandHandle _r = _t->appendCatenate((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<Imap::Mailbox::CatenatePair>(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 33: { CommandHandle _r = _t->appendCatenate((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QList<Imap::Mailbox::CatenatePair>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 34: { CommandHandle _r = _t->check();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 35: { CommandHandle _r = _t->close();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 36: { CommandHandle _r = _t->expunge();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 37: { CommandHandle _r = _t->search((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 38: { CommandHandle _r = _t->search((*reinterpret_cast< const QStringList(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 39: { CommandHandle _r = _t->fetch((*reinterpret_cast< const Sequence(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])),(*reinterpret_cast< const QMap<QByteArray,quint64>(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 40: { CommandHandle _r = _t->fetch((*reinterpret_cast< const Sequence(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 41: { CommandHandle _r = _t->store((*reinterpret_cast< const Sequence(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 42: { CommandHandle _r = _t->copy((*reinterpret_cast< const Sequence(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 43: { CommandHandle _r = _t->uidFetch((*reinterpret_cast< const Sequence(*)>(_a[1])),(*reinterpret_cast< const QList<QByteArray>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 44: { CommandHandle _r = _t->uidStore((*reinterpret_cast< const Sequence(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 45: { CommandHandle _r = _t->uidCopy((*reinterpret_cast< const Sequence(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 46: { CommandHandle _r = _t->uidMove((*reinterpret_cast< const Sequence(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 47: { CommandHandle _r = _t->uidExpunge((*reinterpret_cast< const Sequence(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 48: { CommandHandle _r = _t->uidSearch((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 49: { CommandHandle _r = _t->uidSearch((*reinterpret_cast< const QStringList(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 50: { CommandHandle _r = _t->uidSearchUid((*reinterpret_cast< const QByteArray(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 51: { CommandHandle _r = _t->uidESearchUid((*reinterpret_cast< const QByteArray(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 52: { CommandHandle _r = _t->xAtom((*reinterpret_cast< const Commands::Command(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 53: { CommandHandle _r = _t->unSelect();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 54: { CommandHandle _r = _t->idle();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 55: _t->idleDone(); break;
        case 56: _t->idleContinuationWontCome(); break;
        case 57: _t->idleMagicallyTerminatedByServer(); break;
        case 58: { CommandHandle _r = _t->namespaceCommand();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 59: { CommandHandle _r = _t->sort((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 60: { CommandHandle _r = _t->uidSort((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 61: { CommandHandle _r = _t->thread((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 62: { CommandHandle _r = _t->uidThread((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 63: { CommandHandle _r = _t->uidESort((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])),(*reinterpret_cast< const QStringList(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 64: { CommandHandle _r = _t->uidESearch((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 65: { CommandHandle _r = _t->uidEThread((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])),(*reinterpret_cast< const QStringList(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 66: { CommandHandle _r = _t->cancelUpdate((*reinterpret_cast< const CommandHandle(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 67: { CommandHandle _r = _t->idCommand();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 68: { CommandHandle _r = _t->idCommand((*reinterpret_cast< const QMap<QByteArray,QByteArray>(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 69: { CommandHandle _r = _t->enable((*reinterpret_cast< const QList<QByteArray>(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 70: { CommandHandle _r = _t->compressDeflate();
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 71: { CommandHandle _r = _t->genUrlAuth((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 72: { CommandHandle _r = _t->uidSendmail((*reinterpret_cast< const uint(*)>(_a[1])),(*reinterpret_cast< const Mailbox::UidSubmitOptionsList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< CommandHandle*>(_a[0]) = std::move(_r); }  break;
        case 73: _t->slotSocketStateChanged((*reinterpret_cast< const Imap::ConnectionState(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 74: _t->unfreezeAfterEncryption(); break;
        case 75: _t->closeConnection(); break;
        case 76: _t->handleReadyRead(); break;
        case 77: _t->handleDisconnected((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 78: _t->executeACommand(); break;
        case 79: _t->executeCommands(); break;
        case 80: _t->finishStartTls(); break;
        case 81: _t->handleSocketEncrypted(); break;
        case 82: _t->handleCompressionPossibleActivated(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Imap::Parser* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Imap::Parser* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Imap::Parser* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Imap::Parser* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Imap::Parser* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Parser::*)(Imap::Parser * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Parser::responseReceived)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Parser::*)(Imap::Parser * , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Parser::lineReceived)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Parser::*)(Imap::Parser * , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Parser::lineSent)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Parser::*)(Imap::Parser * , const QString & , const QByteArray * , uint );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Parser::parserWarning)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Parser::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Parser::commandQueued)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Parser::*)(Imap::Parser * , Imap::ConnectionState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Parser::connectionStateChanged)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Imap::Parser::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Imap__Parser.data,
    qt_meta_data_Imap__Parser,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Parser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Parser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Parser.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Imap::Parser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 83)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 83;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 83)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 83;
    }
    return _id;
}

// SIGNAL 0
void Imap::Parser::responseReceived(Imap::Parser * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Imap::Parser::lineReceived(Imap::Parser * _t1, const QByteArray & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Imap::Parser::lineSent(Imap::Parser * _t1, const QByteArray & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Imap::Parser::parserWarning(Imap::Parser * _t1, const QString & _t2, const QByteArray * _t3, uint _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Imap::Parser::commandQueued()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Imap::Parser::connectionStateChanged(Imap::Parser * _t1, Imap::ConnectionState _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
