/****************************************************************************
** Meta object code from reading C++ file 'AddressbookPlugin.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Plugins/AddressbookPlugin.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AddressbookPlugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Plugins__AddressbookJob_t {
    QByteArrayData data[4];
    char stringdata0[62];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Plugins__AddressbookJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Plugins__AddressbookJob_t qt_meta_stringdata_Plugins__AddressbookJob = {
    {
QT_MOC_LITERAL(0, 0, 23), // "Plugins::AddressbookJob"
QT_MOC_LITERAL(1, 24, 5), // "error"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 30) // "Plugins::AddressbookJob::Error"

    },
    "Plugins::AddressbookJob\0error\0\0"
    "Plugins::AddressbookJob::Error"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Plugins__AddressbookJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    1,

       0        // eod
};

void Plugins::AddressbookJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AddressbookJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->error((*reinterpret_cast< Plugins::AddressbookJob::Error(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AddressbookJob::*)(Plugins::AddressbookJob::Error );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AddressbookJob::error)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Plugins::AddressbookJob::staticMetaObject = { {
    QMetaObject::SuperData::link<PluginJob::staticMetaObject>(),
    qt_meta_stringdata_Plugins__AddressbookJob.data,
    qt_meta_data_Plugins__AddressbookJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Plugins::AddressbookJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Plugins::AddressbookJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Plugins__AddressbookJob.stringdata0))
        return static_cast<void*>(this);
    return PluginJob::qt_metacast(_clname);
}

int Plugins::AddressbookJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PluginJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Plugins::AddressbookJob::error(Plugins::AddressbookJob::Error _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_Plugins__AddressbookCompletionJob_t {
    QByteArrayData data[5];
    char stringdata0[89];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Plugins__AddressbookCompletionJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Plugins__AddressbookCompletionJob_t qt_meta_stringdata_Plugins__AddressbookCompletionJob = {
    {
QT_MOC_LITERAL(0, 0, 33), // "Plugins::AddressbookCompletio..."
QT_MOC_LITERAL(1, 34, 19), // "completionAvailable"
QT_MOC_LITERAL(2, 54, 0), // ""
QT_MOC_LITERAL(3, 55, 22), // "Plugins::NameEmailList"
QT_MOC_LITERAL(4, 78, 10) // "completion"

    },
    "Plugins::AddressbookCompletionJob\0"
    "completionAvailable\0\0Plugins::NameEmailList\0"
    "completion"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Plugins__AddressbookCompletionJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void Plugins::AddressbookCompletionJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AddressbookCompletionJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->completionAvailable((*reinterpret_cast< const Plugins::NameEmailList(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AddressbookCompletionJob::*)(const Plugins::NameEmailList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AddressbookCompletionJob::completionAvailable)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Plugins::AddressbookCompletionJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AddressbookJob::staticMetaObject>(),
    qt_meta_stringdata_Plugins__AddressbookCompletionJob.data,
    qt_meta_data_Plugins__AddressbookCompletionJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Plugins::AddressbookCompletionJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Plugins::AddressbookCompletionJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Plugins__AddressbookCompletionJob.stringdata0))
        return static_cast<void*>(this);
    return AddressbookJob::qt_metacast(_clname);
}

int Plugins::AddressbookCompletionJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AddressbookJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Plugins::AddressbookCompletionJob::completionAvailable(const Plugins::NameEmailList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_Plugins__AddressbookNamesJob_t {
    QByteArrayData data[4];
    char stringdata0[74];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Plugins__AddressbookNamesJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Plugins__AddressbookNamesJob_t qt_meta_stringdata_Plugins__AddressbookNamesJob = {
    {
QT_MOC_LITERAL(0, 0, 28), // "Plugins::AddressbookNamesJob"
QT_MOC_LITERAL(1, 29, 30), // "prettyNamesForAddressAvailable"
QT_MOC_LITERAL(2, 60, 0), // ""
QT_MOC_LITERAL(3, 61, 12) // "displayNames"

    },
    "Plugins::AddressbookNamesJob\0"
    "prettyNamesForAddressAvailable\0\0"
    "displayNames"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Plugins__AddressbookNamesJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    3,

       0        // eod
};

void Plugins::AddressbookNamesJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AddressbookNamesJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->prettyNamesForAddressAvailable((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AddressbookNamesJob::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AddressbookNamesJob::prettyNamesForAddressAvailable)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Plugins::AddressbookNamesJob::staticMetaObject = { {
    QMetaObject::SuperData::link<AddressbookJob::staticMetaObject>(),
    qt_meta_stringdata_Plugins__AddressbookNamesJob.data,
    qt_meta_data_Plugins__AddressbookNamesJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Plugins::AddressbookNamesJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Plugins::AddressbookNamesJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Plugins__AddressbookNamesJob.stringdata0))
        return static_cast<void*>(this);
    return AddressbookJob::qt_metacast(_clname);
}

int Plugins::AddressbookNamesJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AddressbookJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Plugins::AddressbookNamesJob::prettyNamesForAddressAvailable(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_Plugins__AddressbookPlugin_t {
    QByteArrayData data[13];
    char stringdata0[198];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Plugins__AddressbookPlugin_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Plugins__AddressbookPlugin_t qt_meta_stringdata_Plugins__AddressbookPlugin = {
    {
QT_MOC_LITERAL(0, 0, 26), // "Plugins::AddressbookPlugin"
QT_MOC_LITERAL(1, 27, 17), // "requestCompletion"
QT_MOC_LITERAL(2, 45, 25), // "AddressbookCompletionJob*"
QT_MOC_LITERAL(3, 71, 0), // ""
QT_MOC_LITERAL(4, 72, 5), // "input"
QT_MOC_LITERAL(5, 78, 7), // "ignores"
QT_MOC_LITERAL(6, 86, 3), // "max"
QT_MOC_LITERAL(7, 90, 28), // "requestPrettyNamesForAddress"
QT_MOC_LITERAL(8, 119, 20), // "AddressbookNamesJob*"
QT_MOC_LITERAL(9, 140, 5), // "email"
QT_MOC_LITERAL(10, 146, 21), // "openAddressbookWindow"
QT_MOC_LITERAL(11, 168, 17), // "openContactWindow"
QT_MOC_LITERAL(12, 186, 11) // "displayName"

    },
    "Plugins::AddressbookPlugin\0requestCompletion\0"
    "AddressbookCompletionJob*\0\0input\0"
    "ignores\0max\0requestPrettyNamesForAddress\0"
    "AddressbookNamesJob*\0email\0"
    "openAddressbookWindow\0openContactWindow\0"
    "displayName"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Plugins__AddressbookPlugin[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   44,    3, 0x0a /* Public */,
       1,    2,   51,    3, 0x2a /* Public | MethodCloned */,
       1,    1,   56,    3, 0x2a /* Public | MethodCloned */,
       7,    1,   59,    3, 0x0a /* Public */,
      10,    0,   62,    3, 0x0a /* Public */,
      11,    2,   63,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2, QMetaType::QString, QMetaType::QStringList, QMetaType::Int,    4,    5,    6,
    0x80000000 | 2, QMetaType::QString, QMetaType::QStringList,    4,    5,
    0x80000000 | 2, QMetaType::QString,    4,
    0x80000000 | 8, QMetaType::QString,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    9,   12,

       0        // eod
};

void Plugins::AddressbookPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AddressbookPlugin *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { AddressbookCompletionJob* _r = _t->requestCompletion((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< AddressbookCompletionJob**>(_a[0]) = std::move(_r); }  break;
        case 1: { AddressbookCompletionJob* _r = _t->requestCompletion((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< AddressbookCompletionJob**>(_a[0]) = std::move(_r); }  break;
        case 2: { AddressbookCompletionJob* _r = _t->requestCompletion((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< AddressbookCompletionJob**>(_a[0]) = std::move(_r); }  break;
        case 3: { AddressbookNamesJob* _r = _t->requestPrettyNamesForAddress((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< AddressbookNamesJob**>(_a[0]) = std::move(_r); }  break;
        case 4: _t->openAddressbookWindow(); break;
        case 5: _t->openContactWindow((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Plugins::AddressbookPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Plugins__AddressbookPlugin.data,
    qt_meta_data_Plugins__AddressbookPlugin,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Plugins::AddressbookPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Plugins::AddressbookPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Plugins__AddressbookPlugin.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Plugins::AddressbookPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
