/* Copyright (C) 2006 - 2014 Jan Kundrát <jkt@flaska.net>

   This file is part of the Trojita Qt IMAP e-mail client,
   http://trojita.flaska.net/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef MSGPARTNETWORKREPLY_H
#define MSGPARTNETWORKREPLY_H

#include <QBuffer>
#include <QModelIndex>
#include <QNetworkReply>

namespace Imap
{
namespace Network
{

class MsgPartNetAccessManager;

/** @short Qt-like access to one MIME message part */
class MsgPartNetworkReply : public QNetworkReply
{
    Q_OBJECT
public:
    MsgPartNetworkReply(MsgPartNetAccessManager *parent, const QPersistentModelIndex &part);
    virtual void abort();
    virtual void close();
    virtual qint64 bytesAvailable() const;
public slots:
    void slotModelDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight);
    void slotMyDataChanged();
protected:
    virtual qint64 readData(char *data, qint64 maxSize);
private:
    void disconnectBufferIfVanished() const;

    QPersistentModelIndex part;
    mutable QBuffer buffer;

    MsgPartNetworkReply(const MsgPartNetworkReply &); // don't implement
    MsgPartNetworkReply &operator=(const MsgPartNetworkReply &); // don't implement
};

}
}

#endif // MSGPARTNETWORKREPLY_H
