/****************************************************************************
** Meta object code from reading C++ file 'FakeMSA.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/MSA/FakeMSA.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'FakeMSA.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MSA__Fake_t {
    QByteArrayData data[3];
    char stringdata0[18];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MSA__Fake_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MSA__Fake_t qt_meta_stringdata_MSA__Fake = {
    {
QT_MOC_LITERAL(0, 0, 9), // "MSA::Fake"
QT_MOC_LITERAL(1, 10, 6), // "cancel"
QT_MOC_LITERAL(2, 17, 0) // ""

    },
    "MSA::Fake\0cancel\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MSA__Fake[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void MSA::Fake::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Fake *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cancel(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject MSA::Fake::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractMSA::staticMetaObject>(),
    qt_meta_stringdata_MSA__Fake.data,
    qt_meta_data_MSA__Fake,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MSA::Fake::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MSA::Fake::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MSA__Fake.stringdata0))
        return static_cast<void*>(this);
    return AbstractMSA::qt_metacast(_clname);
}

int MSA::Fake::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractMSA::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_MSA__FakeFactory_t {
    QByteArrayData data[24];
    char stringdata0[251];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MSA__FakeFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MSA__FakeFactory_t qt_meta_stringdata_MSA__FakeFactory = {
    {
QT_MOC_LITERAL(0, 0, 16), // "MSA::FakeFactory"
QT_MOC_LITERAL(1, 17, 16), // "requestedSending"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 4), // "from"
QT_MOC_LITERAL(4, 40, 2), // "to"
QT_MOC_LITERAL(5, 43, 4), // "data"
QT_MOC_LITERAL(6, 48, 20), // "requestedBurlSending"
QT_MOC_LITERAL(7, 69, 7), // "imapUrl"
QT_MOC_LITERAL(8, 77, 19), // "requestedCancelling"
QT_MOC_LITERAL(9, 97, 10), // "connecting"
QT_MOC_LITERAL(10, 108, 7), // "sending"
QT_MOC_LITERAL(11, 116, 4), // "sent"
QT_MOC_LITERAL(12, 121, 5), // "error"
QT_MOC_LITERAL(13, 127, 7), // "message"
QT_MOC_LITERAL(14, 135, 11), // "progressMax"
QT_MOC_LITERAL(15, 147, 3), // "max"
QT_MOC_LITERAL(16, 151, 8), // "progress"
QT_MOC_LITERAL(17, 160, 3), // "num"
QT_MOC_LITERAL(18, 164, 16), // "doEmitConnecting"
QT_MOC_LITERAL(19, 181, 13), // "doEmitSending"
QT_MOC_LITERAL(20, 195, 10), // "doEmitSent"
QT_MOC_LITERAL(21, 206, 11), // "doEmitError"
QT_MOC_LITERAL(22, 218, 17), // "doEmitProgressMax"
QT_MOC_LITERAL(23, 236, 14) // "doEmitProgress"

    },
    "MSA::FakeFactory\0requestedSending\0\0"
    "from\0to\0data\0requestedBurlSending\0"
    "imapUrl\0requestedCancelling\0connecting\0"
    "sending\0sent\0error\0message\0progressMax\0"
    "max\0progress\0num\0doEmitConnecting\0"
    "doEmitSending\0doEmitSent\0doEmitError\0"
    "doEmitProgressMax\0doEmitProgress"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MSA__FakeFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,   89,    2, 0x06 /* Public */,
       6,    3,   96,    2, 0x06 /* Public */,
       8,    0,  103,    2, 0x06 /* Public */,
       9,    0,  104,    2, 0x06 /* Public */,
      10,    0,  105,    2, 0x06 /* Public */,
      11,    0,  106,    2, 0x06 /* Public */,
      12,    1,  107,    2, 0x06 /* Public */,
      14,    1,  110,    2, 0x06 /* Public */,
      16,    1,  113,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    0,  116,    2, 0x0a /* Public */,
      19,    0,  117,    2, 0x0a /* Public */,
      20,    0,  118,    2, 0x0a /* Public */,
      21,    1,  119,    2, 0x0a /* Public */,
      22,    1,  122,    2, 0x0a /* Public */,
      23,    1,  125,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QByteArray, QMetaType::QByteArrayList, QMetaType::QByteArray,    3,    4,    5,
    QMetaType::Void, QMetaType::QByteArray, QMetaType::QByteArrayList, QMetaType::QByteArray,    3,    4,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   17,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   17,

       0        // eod
};

void MSA::FakeFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FakeFactory *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestedSending((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< const QList<QByteArray>(*)>(_a[2])),(*reinterpret_cast< const QByteArray(*)>(_a[3]))); break;
        case 1: _t->requestedBurlSending((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< const QList<QByteArray>(*)>(_a[2])),(*reinterpret_cast< const QByteArray(*)>(_a[3]))); break;
        case 2: _t->requestedCancelling(); break;
        case 3: _t->connecting(); break;
        case 4: _t->sending(); break;
        case 5: _t->sent(); break;
        case 6: _t->error((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->progressMax((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->progress((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->doEmitConnecting(); break;
        case 10: _t->doEmitSending(); break;
        case 11: _t->doEmitSent(); break;
        case 12: _t->doEmitError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->doEmitProgressMax((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->doEmitProgress((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FakeFactory::*)(const QByteArray & , const QList<QByteArray> & , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FakeFactory::requestedSending)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FakeFactory::*)(const QByteArray & , const QList<QByteArray> & , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FakeFactory::requestedBurlSending)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FakeFactory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FakeFactory::requestedCancelling)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FakeFactory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FakeFactory::connecting)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FakeFactory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FakeFactory::sending)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (FakeFactory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FakeFactory::sent)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (FakeFactory::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FakeFactory::error)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (FakeFactory::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FakeFactory::progressMax)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (FakeFactory::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FakeFactory::progress)) {
                *result = 8;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MSA::FakeFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_MSA__FakeFactory.data,
    qt_meta_data_MSA__FakeFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MSA::FakeFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MSA::FakeFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MSA__FakeFactory.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "MSAFactory"))
        return static_cast< MSAFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int MSA::FakeFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void MSA::FakeFactory::requestedSending(const QByteArray & _t1, const QList<QByteArray> & _t2, const QByteArray & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void MSA::FakeFactory::requestedBurlSending(const QByteArray & _t1, const QList<QByteArray> & _t2, const QByteArray & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void MSA::FakeFactory::requestedCancelling()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MSA::FakeFactory::connecting()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MSA::FakeFactory::sending()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MSA::FakeFactory::sent()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MSA::FakeFactory::error(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void MSA::FakeFactory::progressMax(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void MSA::FakeFactory::progress(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
