/* Copyright (C) 2006 - 2014 Jan Kundrát <jkt@flaska.net>

   This file is part of the Trojita Qt IMAP e-mail client,
   http://trojita.flaska.net/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef GUI_ENVELOPEVIEW_H
#define GUI_ENVELOPEVIEW_H

#include <QModelIndex>
#include <QWidget>

namespace Imap {
namespace Message {
class MailAddress;
}
}

namespace Gui {

class MessageView;

/** @short Widget displaying the message envelope */
class EnvelopeView : public QWidget
{
    Q_OBJECT
public:
    EnvelopeView(QWidget *parent, MessageView *messageView);

    void setMessage(const QModelIndex &index);

private:
    QString headerText(const QModelIndex &index);

    QString htmlizeAddresses(const QList<Imap::Message::MailAddress> &addresses);

    MessageView *m_messageView;

    EnvelopeView(const EnvelopeView &); // don't implement
    EnvelopeView &operator=(const EnvelopeView &); // don't implement
};

}

#endif // GUI_ENVELOPEVIEW_H
