/********************************************************************************
** Form generated from reading UI file 'ProgressPopUp.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROGRESSPOPUP_H
#define UI_PROGRESSPOPUP_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ProgressPopUp
{
public:
    QVBoxLayout *verticalLayout;
    QProgressBar *progressBar;
    QLabel *label;

    void setupUi(QFrame *ProgressPopUp)
    {
        if (ProgressPopUp->objectName().isEmpty())
            ProgressPopUp->setObjectName(QString::fromUtf8("ProgressPopUp"));
        ProgressPopUp->resize(300, 70);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ProgressPopUp->sizePolicy().hasHeightForWidth());
        ProgressPopUp->setSizePolicy(sizePolicy);
        ProgressPopUp->setMinimumSize(QSize(300, 70));
        ProgressPopUp->setMaximumSize(QSize(300, 70));
        ProgressPopUp->setAutoFillBackground(true);
        ProgressPopUp->setFrameShape(QFrame::StyledPanel);
        ProgressPopUp->setFrameShadow(QFrame::Raised);
        verticalLayout = new QVBoxLayout(ProgressPopUp);
        verticalLayout->setSpacing(5);
        verticalLayout->setContentsMargins(10, 10, 10, 10);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        progressBar = new QProgressBar(ProgressPopUp);
        progressBar->setObjectName(QString::fromUtf8("progressBar"));
        progressBar->setMaximum(0);

        verticalLayout->addWidget(progressBar);

        label = new QLabel(ProgressPopUp);
        label->setObjectName(QString::fromUtf8("label"));
        label->setText(QString::fromUtf8("status"));
        label->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(label);


        retranslateUi(ProgressPopUp);

        QMetaObject::connectSlotsByName(ProgressPopUp);
    } // setupUi

    void retranslateUi(QFrame *ProgressPopUp)
    {
        (void)ProgressPopUp;
    } // retranslateUi

};

namespace Ui {
    class ProgressPopUp: public Ui_ProgressPopUp {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROGRESSPOPUP_H
