/****************************************************************************
** Meta object code from reading C++ file 'ClearTextPassword.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Plugins/ClearTextPassword/ClearTextPassword.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ClearTextPassword.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ClearTextPasswordJob_t {
    QByteArrayData data[4];
    char stringdata0[37];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ClearTextPasswordJob_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ClearTextPasswordJob_t qt_meta_stringdata_ClearTextPasswordJob = {
    {
QT_MOC_LITERAL(0, 0, 20), // "ClearTextPasswordJob"
QT_MOC_LITERAL(1, 21, 7), // "doStart"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 6) // "doStop"

    },
    "ClearTextPasswordJob\0doStart\0\0doStop"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ClearTextPasswordJob[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x0a /* Public */,
       3,    0,   25,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ClearTextPasswordJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ClearTextPasswordJob *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->doStart(); break;
        case 1: _t->doStop(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ClearTextPasswordJob::staticMetaObject = { {
    QMetaObject::SuperData::link<PasswordJob::staticMetaObject>(),
    qt_meta_stringdata_ClearTextPasswordJob.data,
    qt_meta_data_ClearTextPasswordJob,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ClearTextPasswordJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ClearTextPasswordJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ClearTextPasswordJob.stringdata0))
        return static_cast<void*>(this);
    return PasswordJob::qt_metacast(_clname);
}

int ClearTextPasswordJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PasswordJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_ClearTextPassword_t {
    QByteArrayData data[9];
    char stringdata0[108];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ClearTextPassword_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ClearTextPassword_t qt_meta_stringdata_ClearTextPassword = {
    {
QT_MOC_LITERAL(0, 0, 17), // "ClearTextPassword"
QT_MOC_LITERAL(1, 18, 15), // "requestPassword"
QT_MOC_LITERAL(2, 34, 12), // "PasswordJob*"
QT_MOC_LITERAL(3, 47, 0), // ""
QT_MOC_LITERAL(4, 48, 9), // "accountId"
QT_MOC_LITERAL(5, 58, 11), // "accountType"
QT_MOC_LITERAL(6, 70, 13), // "storePassword"
QT_MOC_LITERAL(7, 84, 8), // "password"
QT_MOC_LITERAL(8, 93, 14) // "deletePassword"

    },
    "ClearTextPassword\0requestPassword\0"
    "PasswordJob*\0\0accountId\0accountType\0"
    "storePassword\0password\0deletePassword"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ClearTextPassword[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   29,    3, 0x0a /* Public */,
       6,    3,   34,    3, 0x0a /* Public */,
       8,    2,   41,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2, QMetaType::QString, QMetaType::QString,    4,    5,
    0x80000000 | 2, QMetaType::QString, QMetaType::QString, QMetaType::QString,    4,    5,    7,
    0x80000000 | 2, QMetaType::QString, QMetaType::QString,    4,    5,

       0        // eod
};

void ClearTextPassword::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ClearTextPassword *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { PasswordJob* _r = _t->requestPassword((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< PasswordJob**>(_a[0]) = std::move(_r); }  break;
        case 1: { PasswordJob* _r = _t->storePassword((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< PasswordJob**>(_a[0]) = std::move(_r); }  break;
        case 2: { PasswordJob* _r = _t->deletePassword((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< PasswordJob**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ClearTextPassword::staticMetaObject = { {
    QMetaObject::SuperData::link<PasswordPlugin::staticMetaObject>(),
    qt_meta_stringdata_ClearTextPassword.data,
    qt_meta_data_ClearTextPassword,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ClearTextPassword::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ClearTextPassword::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ClearTextPassword.stringdata0))
        return static_cast<void*>(this);
    return PasswordPlugin::qt_metacast(_clname);
}

int ClearTextPassword::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PasswordPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_trojita_plugin_ClearTextPasswordPlugin_t {
    QByteArrayData data[1];
    char stringdata0[39];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_trojita_plugin_ClearTextPasswordPlugin_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_trojita_plugin_ClearTextPasswordPlugin_t qt_meta_stringdata_trojita_plugin_ClearTextPasswordPlugin = {
    {
QT_MOC_LITERAL(0, 0, 38) // "trojita_plugin_ClearTextPassw..."

    },
    "trojita_plugin_ClearTextPasswordPlugin"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_trojita_plugin_ClearTextPasswordPlugin[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void trojita_plugin_ClearTextPasswordPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject trojita_plugin_ClearTextPasswordPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_trojita_plugin_ClearTextPasswordPlugin.data,
    qt_meta_data_trojita_plugin_ClearTextPasswordPlugin,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *trojita_plugin_ClearTextPasswordPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *trojita_plugin_ClearTextPasswordPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_trojita_plugin_ClearTextPasswordPlugin.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PasswordPluginInterface"))
        return static_cast< PasswordPluginInterface*>(this);
    if (!strcmp(_clname, "PasswordPluginInterface"))
        return static_cast< Plugins::PasswordPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int trojita_plugin_ClearTextPasswordPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x2d,  'n',  'e',  't',  '.',  'f', 
    'l',  'a',  's',  'k',  'a',  '.',  't',  'r', 
    'o',  'j',  'i',  't',  'a',  '.',  'p',  'l', 
    'u',  'g',  'i',  'n',  's',  '.',  'p',  'a', 
    's',  's',  'w',  'o',  'r',  'd',  '.',  'c', 
    'l',  'e',  'a',  'r',  't',  'e',  'x',  't', 
    // "className"
    0x03,  0x78,  0x26,  't',  'r',  'o',  'j',  'i', 
    't',  'a',  '_',  'p',  'l',  'u',  'g',  'i', 
    'n',  '_',  'C',  'l',  'e',  'a',  'r',  'T', 
    'e',  'x',  't',  'P',  'a',  's',  's',  'w', 
    'o',  'r',  'd',  'P',  'l',  'u',  'g',  'i', 
    'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(trojita_plugin_ClearTextPasswordPlugin, trojita_plugin_ClearTextPasswordPlugin)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
