/****************************************************************************
** Meta object code from reading C++ file 'MessageView.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/MessageView.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MessageView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gui__MessageView_t {
    QByteArrayData data[39];
    char stringdata0[496];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__MessageView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__MessageView_t qt_meta_stringdata_Gui__MessageView = {
    {
QT_MOC_LITERAL(0, 0, 16), // "Gui::MessageView"
QT_MOC_LITERAL(1, 17, 14), // "messageChanged"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 19), // "messageModelChanged"
QT_MOC_LITERAL(4, 53, 19), // "QAbstractItemModel*"
QT_MOC_LITERAL(5, 73, 5), // "model"
QT_MOC_LITERAL(6, 79, 11), // "linkHovered"
QT_MOC_LITERAL(7, 91, 3), // "url"
QT_MOC_LITERAL(8, 95, 17), // "searchRequestedBy"
QT_MOC_LITERAL(9, 113, 16), // "EmbeddedWebView*"
QT_MOC_LITERAL(10, 130, 7), // "webView"
QT_MOC_LITERAL(11, 138, 13), // "transferError"
QT_MOC_LITERAL(12, 152, 11), // "errorString"
QT_MOC_LITERAL(13, 164, 10), // "setMessage"
QT_MOC_LITERAL(14, 175, 11), // "QModelIndex"
QT_MOC_LITERAL(15, 187, 5), // "index"
QT_MOC_LITERAL(16, 193, 8), // "setEmpty"
QT_MOC_LITERAL(17, 202, 14), // "setHomepageUrl"
QT_MOC_LITERAL(18, 217, 8), // "homepage"
QT_MOC_LITERAL(19, 226, 18), // "stopAutoMarkAsRead"
QT_MOC_LITERAL(20, 245, 6), // "zoomIn"
QT_MOC_LITERAL(21, 252, 7), // "zoomOut"
QT_MOC_LITERAL(22, 260, 12), // "zoomOriginal"
QT_MOC_LITERAL(23, 273, 10), // "markAsRead"
QT_MOC_LITERAL(24, 284, 18), // "externalsRequested"
QT_MOC_LITERAL(25, 303, 18), // "enableExternalData"
QT_MOC_LITERAL(26, 322, 14), // "newLabelAction"
QT_MOC_LITERAL(27, 337, 3), // "tag"
QT_MOC_LITERAL(28, 341, 17), // "deleteLabelAction"
QT_MOC_LITERAL(29, 359, 24), // "partContextMenuRequested"
QT_MOC_LITERAL(30, 384, 5), // "point"
QT_MOC_LITERAL(31, 390, 15), // "partLinkHovered"
QT_MOC_LITERAL(32, 406, 4), // "link"
QT_MOC_LITERAL(33, 411, 5), // "title"
QT_MOC_LITERAL(34, 417, 11), // "textContent"
QT_MOC_LITERAL(35, 429, 21), // "triggerSearchDialogBy"
QT_MOC_LITERAL(36, 451, 1), // "w"
QT_MOC_LITERAL(37, 453, 20), // "onWebViewLoadStarted"
QT_MOC_LITERAL(38, 474, 21) // "onWebViewLoadFinished"

    },
    "Gui::MessageView\0messageChanged\0\0"
    "messageModelChanged\0QAbstractItemModel*\0"
    "model\0linkHovered\0url\0searchRequestedBy\0"
    "EmbeddedWebView*\0webView\0transferError\0"
    "errorString\0setMessage\0QModelIndex\0"
    "index\0setEmpty\0setHomepageUrl\0homepage\0"
    "stopAutoMarkAsRead\0zoomIn\0zoomOut\0"
    "zoomOriginal\0markAsRead\0externalsRequested\0"
    "enableExternalData\0newLabelAction\0tag\0"
    "deleteLabelAction\0partContextMenuRequested\0"
    "point\0partLinkHovered\0link\0title\0"
    "textContent\0triggerSearchDialogBy\0w\0"
    "onWebViewLoadStarted\0onWebViewLoadFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__MessageView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      22,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  124,    2, 0x06 /* Public */,
       3,    1,  125,    2, 0x06 /* Public */,
       6,    1,  128,    2, 0x06 /* Public */,
       8,    1,  131,    2, 0x06 /* Public */,
      11,    1,  134,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    1,  137,    2, 0x0a /* Public */,
      16,    0,  140,    2, 0x0a /* Public */,
      17,    1,  141,    2, 0x0a /* Public */,
      19,    0,  144,    2, 0x0a /* Public */,
      20,    0,  145,    2, 0x0a /* Public */,
      21,    0,  146,    2, 0x0a /* Public */,
      22,    0,  147,    2, 0x0a /* Public */,
      23,    0,  148,    2, 0x08 /* Private */,
      24,    1,  149,    2, 0x08 /* Private */,
      25,    0,  152,    2, 0x08 /* Private */,
      26,    1,  153,    2, 0x08 /* Private */,
      28,    1,  156,    2, 0x08 /* Private */,
      29,    1,  159,    2, 0x08 /* Private */,
      31,    3,  162,    2, 0x08 /* Private */,
      35,    1,  169,    2, 0x08 /* Private */,
      37,    0,  172,    2, 0x08 /* Private */,
      38,    0,  173,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::QString,   12,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   27,
    QMetaType::Void, QMetaType::QString,   27,
    QMetaType::Void, QMetaType::QPoint,   30,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString,   32,   33,   34,
    QMetaType::Void, 0x80000000 | 9,   36,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::MessageView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MessageView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->messageChanged(); break;
        case 1: _t->messageModelChanged((*reinterpret_cast< QAbstractItemModel*(*)>(_a[1]))); break;
        case 2: _t->linkHovered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->searchRequestedBy((*reinterpret_cast< EmbeddedWebView*(*)>(_a[1]))); break;
        case 4: _t->transferError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->setMessage((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 6: _t->setEmpty(); break;
        case 7: _t->setHomepageUrl((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 8: _t->stopAutoMarkAsRead(); break;
        case 9: _t->zoomIn(); break;
        case 10: _t->zoomOut(); break;
        case 11: _t->zoomOriginal(); break;
        case 12: _t->markAsRead(); break;
        case 13: _t->externalsRequested((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 14: _t->enableExternalData(); break;
        case 15: _t->newLabelAction((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->deleteLabelAction((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->partContextMenuRequested((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 18: _t->partLinkHovered((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 19: _t->triggerSearchDialogBy((*reinterpret_cast< EmbeddedWebView*(*)>(_a[1]))); break;
        case 20: _t->onWebViewLoadStarted(); break;
        case 21: _t->onWebViewLoadFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MessageView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MessageView::messageChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MessageView::*)(QAbstractItemModel * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MessageView::messageModelChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (MessageView::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MessageView::linkHovered)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (MessageView::*)(EmbeddedWebView * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MessageView::searchRequestedBy)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (MessageView::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MessageView::transferError)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::MessageView::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Gui__MessageView.data,
    qt_meta_data_Gui__MessageView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::MessageView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::MessageView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__MessageView.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Gui::MessageView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void Gui::MessageView::messageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Gui::MessageView::messageModelChanged(QAbstractItemModel * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Gui::MessageView::linkHovered(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Gui::MessageView::searchRequestedBy(EmbeddedWebView * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Gui::MessageView::transferError(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
