/* Copyright (C) 2006 - 2017 Jan Kundrát <jkt@kde.org>

   This file is part of the Trojita Qt IMAP e-mail client,
   http://trojita.flaska.net/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "Composer/Recipients.h"
#include "Imap/Model/CatenateData.h"

class QDateTime;
class QIODevice;
class QModelIndex;

namespace Composer {

/** @short Something that can provide a message for sending */
class AbstractComposer
{
public:
    virtual ~AbstractComposer();

    virtual bool isReadyForSerialization() const = 0;
    virtual bool asRawMessage(QIODevice *target, QString *errorMessage) const = 0;
    virtual bool asCatenateData(QList<Imap::Mailbox::CatenatePair> &target, QString *errorMessage) const = 0;
    virtual QDateTime timestamp() const = 0;
    virtual QByteArray rawFromAddress() const = 0;
    virtual QList<QByteArray> rawRecipientAddresses() const = 0;
    virtual void setRecipients(const QList<QPair<Composer::RecipientKind, Imap::Message::MailAddress> > &recipients) = 0;
    virtual void setFrom(const Imap::Message::MailAddress &from) = 0;

    virtual void setPreloadEnabled(const bool preload);
    virtual QModelIndex replyingToMessage() const;
    virtual QModelIndex forwardingMessage() const;

    static QByteArray generateMessageId(const Imap::Message::MailAddress &fromAddress);
};

}
