/****************************************************************************
** Meta object code from reading C++ file 'MessageListWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/MessageListWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MessageListWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gui__MessageListWidget_t {
    QByteArrayData data[17];
    char stringdata0[305];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__MessageListWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__MessageListWidget_t qt_meta_stringdata_Gui__MessageListWidget = {
    {
QT_MOC_LITERAL(0, 0, 22), // "Gui::MessageListWidget"
QT_MOC_LITERAL(1, 23, 16), // "requestingSearch"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 10), // "conditions"
QT_MOC_LITERAL(4, 52, 23), // "rawSearchSettingChanged"
QT_MOC_LITERAL(5, 76, 7), // "enabled"
QT_MOC_LITERAL(6, 84, 11), // "focusSearch"
QT_MOC_LITERAL(7, 96, 14), // "focusRawSearch"
QT_MOC_LITERAL(8, 111, 15), // "slotApplySearch"
QT_MOC_LITERAL(9, 127, 27), // "slotAutoEnableDisableSearch"
QT_MOC_LITERAL(10, 155, 17), // "slotSortingFailed"
QT_MOC_LITERAL(11, 173, 22), // "slotComplexSearchInput"
QT_MOC_LITERAL(12, 196, 8), // "QAction*"
QT_MOC_LITERAL(13, 205, 26), // "slotConditionalSearchReset"
QT_MOC_LITERAL(14, 232, 26), // "slotDeActivateSimpleSearch"
QT_MOC_LITERAL(15, 259, 22), // "slotResetSortingFailed"
QT_MOC_LITERAL(16, 282, 22) // "slotUpdateSearchCursor"

    },
    "Gui::MessageListWidget\0requestingSearch\0"
    "\0conditions\0rawSearchSettingChanged\0"
    "enabled\0focusSearch\0focusRawSearch\0"
    "slotApplySearch\0slotAutoEnableDisableSearch\0"
    "slotSortingFailed\0slotComplexSearchInput\0"
    "QAction*\0slotConditionalSearchReset\0"
    "slotDeActivateSimpleSearch\0"
    "slotResetSortingFailed\0slotUpdateSearchCursor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__MessageListWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       4,    1,   77,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   80,    2, 0x0a /* Public */,
       7,    0,   81,    2, 0x0a /* Public */,
       8,    0,   82,    2, 0x09 /* Protected */,
       9,    0,   83,    2, 0x09 /* Protected */,
      10,    0,   84,    2, 0x09 /* Protected */,
      11,    1,   85,    2, 0x08 /* Private */,
      13,    0,   88,    2, 0x08 /* Private */,
      14,    0,   89,    2, 0x08 /* Private */,
      15,    0,   90,    2, 0x08 /* Private */,
      16,    0,   91,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    3,
    QMetaType::Void, QMetaType::Bool,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::MessageListWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MessageListWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestingSearch((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 1: _t->rawSearchSettingChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->focusSearch(); break;
        case 3: _t->focusRawSearch(); break;
        case 4: _t->slotApplySearch(); break;
        case 5: _t->slotAutoEnableDisableSearch(); break;
        case 6: _t->slotSortingFailed(); break;
        case 7: _t->slotComplexSearchInput((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 8: _t->slotConditionalSearchReset(); break;
        case 9: _t->slotDeActivateSimpleSearch(); break;
        case 10: _t->slotResetSortingFailed(); break;
        case 11: _t->slotUpdateSearchCursor(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MessageListWidget::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MessageListWidget::requestingSearch)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MessageListWidget::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MessageListWidget::rawSearchSettingChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::MessageListWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Gui__MessageListWidget.data,
    qt_meta_data_Gui__MessageListWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::MessageListWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::MessageListWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__MessageListWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Gui::MessageListWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void Gui::MessageListWidget::requestingSearch(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Gui::MessageListWidget::rawSearchSettingChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
