/********************************************************************************
** Form generated from reading UI file 'onecontact.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ONECONTACT_H
#define UI_ONECONTACT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_OneContact
{
public:
    QGridLayout *gridLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *name;
    QLabel *mail;
    QLabel *url;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *country;
    QLabel *state;
    QLabel *city;
    QLabel *zip;
    QLabel *address;
    QGroupBox *groupBox_2;
    QFormLayout *formLayout;
    QLabel *label;
    QLabel *workphone;
    QLabel *label_2;
    QLabel *mobile;
    QLabel *label_3;
    QLabel *phone;
    QLabel *label_4;
    QLabel *fax;
    QGroupBox *groupBox_3;
    QFormLayout *formLayout_2;
    QLabel *label_6;
    QLabel *nick;
    QLabel *label_7;
    QLabel *anniversary;
    QLabel *label_8;
    QLabel *notes;
    QSpacerItem *verticalSpacer;
    QLabel *photo;

    void setupUi(QWidget *OneContact)
    {
        if (OneContact->objectName().isEmpty())
            OneContact->setObjectName(QString::fromUtf8("OneContact"));
        OneContact->resize(400, 300);
        gridLayout_2 = new QGridLayout(OneContact);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        name = new QLabel(OneContact);
        name->setObjectName(QString::fromUtf8("name"));
        QFont font;
        font.setBold(true);
        font.setWeight(75);
        name->setFont(font);
        name->setCursor(QCursor(Qt::IBeamCursor));
        name->setFocusPolicy(Qt::TabFocus);
        name->setTextFormat(Qt::PlainText);
        name->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout->addWidget(name);

        mail = new QLabel(OneContact);
        mail->setObjectName(QString::fromUtf8("mail"));
        mail->setCursor(QCursor(Qt::IBeamCursor));
        mail->setFocusPolicy(Qt::TabFocus);
        mail->setTextFormat(Qt::AutoText);
        mail->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout->addWidget(mail);

        url = new QLabel(OneContact);
        url->setObjectName(QString::fromUtf8("url"));
        url->setCursor(QCursor(Qt::IBeamCursor));
        url->setFocusPolicy(Qt::TabFocus);
        url->setTextFormat(Qt::AutoText);
        url->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout->addWidget(url);


        gridLayout_2->addLayout(verticalLayout, 0, 0, 2, 1);

        groupBox = new QGroupBox(OneContact);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        groupBox->setFlat(true);
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        country = new QLabel(groupBox);
        country->setObjectName(QString::fromUtf8("country"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(country->sizePolicy().hasHeightForWidth());
        country->setSizePolicy(sizePolicy);
        country->setCursor(QCursor(Qt::IBeamCursor));
        country->setFocusPolicy(Qt::TabFocus);
        country->setTextFormat(Qt::PlainText);
        country->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        gridLayout->addWidget(country, 2, 1, 1, 1);

        state = new QLabel(groupBox);
        state->setObjectName(QString::fromUtf8("state"));
        state->setCursor(QCursor(Qt::IBeamCursor));
        state->setFocusPolicy(Qt::TabFocus);
        state->setTextFormat(Qt::PlainText);
        state->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        gridLayout->addWidget(state, 2, 0, 1, 1);

        city = new QLabel(groupBox);
        city->setObjectName(QString::fromUtf8("city"));
        sizePolicy.setHeightForWidth(city->sizePolicy().hasHeightForWidth());
        city->setSizePolicy(sizePolicy);
        city->setCursor(QCursor(Qt::IBeamCursor));
        city->setFocusPolicy(Qt::TabFocus);
        city->setTextFormat(Qt::PlainText);
        city->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        gridLayout->addWidget(city, 1, 1, 1, 1);

        zip = new QLabel(groupBox);
        zip->setObjectName(QString::fromUtf8("zip"));
        zip->setCursor(QCursor(Qt::IBeamCursor));
        zip->setFocusPolicy(Qt::TabFocus);
        zip->setTextFormat(Qt::PlainText);
        zip->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        gridLayout->addWidget(zip, 1, 0, 1, 1);

        address = new QLabel(groupBox);
        address->setObjectName(QString::fromUtf8("address"));
        address->setCursor(QCursor(Qt::IBeamCursor));
        address->setFocusPolicy(Qt::TabFocus);
        address->setTextFormat(Qt::PlainText);
        address->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        gridLayout->addWidget(address, 0, 0, 1, 3);


        gridLayout_2->addWidget(groupBox, 2, 0, 1, 1);

        groupBox_2 = new QGroupBox(OneContact);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        groupBox_2->setFlat(true);
        formLayout = new QFormLayout(groupBox_2);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        label = new QLabel(groupBox_2);
        label->setObjectName(QString::fromUtf8("label"));
        label->setTextFormat(Qt::PlainText);

        formLayout->setWidget(0, QFormLayout::LabelRole, label);

        workphone = new QLabel(groupBox_2);
        workphone->setObjectName(QString::fromUtf8("workphone"));
        workphone->setMinimumSize(QSize(64, 0));
        workphone->setCursor(QCursor(Qt::IBeamCursor));
        workphone->setFocusPolicy(Qt::TabFocus);
        workphone->setTextFormat(Qt::PlainText);
        workphone->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        formLayout->setWidget(0, QFormLayout::FieldRole, workphone);

        label_2 = new QLabel(groupBox_2);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setTextFormat(Qt::PlainText);

        formLayout->setWidget(2, QFormLayout::LabelRole, label_2);

        mobile = new QLabel(groupBox_2);
        mobile->setObjectName(QString::fromUtf8("mobile"));
        mobile->setMinimumSize(QSize(64, 0));
        mobile->setCursor(QCursor(Qt::IBeamCursor));
        mobile->setFocusPolicy(Qt::TabFocus);
        mobile->setTextFormat(Qt::PlainText);
        mobile->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        formLayout->setWidget(2, QFormLayout::FieldRole, mobile);

        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setTextFormat(Qt::PlainText);

        formLayout->setWidget(3, QFormLayout::LabelRole, label_3);

        phone = new QLabel(groupBox_2);
        phone->setObjectName(QString::fromUtf8("phone"));
        phone->setMinimumSize(QSize(64, 0));
        phone->setCursor(QCursor(Qt::IBeamCursor));
        phone->setFocusPolicy(Qt::TabFocus);
        phone->setTextFormat(Qt::PlainText);
        phone->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        formLayout->setWidget(3, QFormLayout::FieldRole, phone);

        label_4 = new QLabel(groupBox_2);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setTextFormat(Qt::PlainText);

        formLayout->setWidget(5, QFormLayout::LabelRole, label_4);

        fax = new QLabel(groupBox_2);
        fax->setObjectName(QString::fromUtf8("fax"));
        fax->setMinimumSize(QSize(64, 0));
        fax->setCursor(QCursor(Qt::IBeamCursor));
        fax->setFocusPolicy(Qt::TabFocus);
        fax->setTextFormat(Qt::PlainText);
        fax->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        formLayout->setWidget(5, QFormLayout::FieldRole, fax);


        gridLayout_2->addWidget(groupBox_2, 3, 0, 1, 1);

        groupBox_3 = new QGroupBox(OneContact);
        groupBox_3->setObjectName(QString::fromUtf8("groupBox_3"));
        groupBox_3->setFlat(true);
        formLayout_2 = new QFormLayout(groupBox_3);
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        label_6 = new QLabel(groupBox_3);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        formLayout_2->setWidget(0, QFormLayout::LabelRole, label_6);

        nick = new QLabel(groupBox_3);
        nick->setObjectName(QString::fromUtf8("nick"));
        nick->setMinimumSize(QSize(64, 0));
        nick->setCursor(QCursor(Qt::IBeamCursor));
        nick->setFocusPolicy(Qt::TabFocus);
        nick->setTextFormat(Qt::PlainText);
        nick->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        formLayout_2->setWidget(0, QFormLayout::FieldRole, nick);

        label_7 = new QLabel(groupBox_3);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        formLayout_2->setWidget(1, QFormLayout::LabelRole, label_7);

        anniversary = new QLabel(groupBox_3);
        anniversary->setObjectName(QString::fromUtf8("anniversary"));
        anniversary->setMinimumSize(QSize(64, 0));
        anniversary->setCursor(QCursor(Qt::IBeamCursor));
        anniversary->setFocusPolicy(Qt::TabFocus);
        anniversary->setTextFormat(Qt::PlainText);
        anniversary->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        formLayout_2->setWidget(1, QFormLayout::FieldRole, anniversary);

        label_8 = new QLabel(groupBox_3);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        formLayout_2->setWidget(2, QFormLayout::LabelRole, label_8);

        notes = new QLabel(groupBox_3);
        notes->setObjectName(QString::fromUtf8("notes"));
        notes->setMinimumSize(QSize(64, 0));
        notes->setCursor(QCursor(Qt::IBeamCursor));
        notes->setFocusPolicy(Qt::TabFocus);
        notes->setTextFormat(Qt::PlainText);
        notes->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextEditable|Qt::TextEditorInteraction|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        formLayout_2->setWidget(2, QFormLayout::FieldRole, notes);


        gridLayout_2->addWidget(groupBox_3, 3, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 4, 0, 1, 2);

        photo = new QLabel(OneContact);
        photo->setObjectName(QString::fromUtf8("photo"));
        photo->setFocusPolicy(Qt::StrongFocus);
        photo->setAcceptDrops(true);
        photo->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(photo, 0, 1, 3, 1);

        QWidget::setTabOrder(name, mail);
        QWidget::setTabOrder(mail, url);
        QWidget::setTabOrder(url, address);
        QWidget::setTabOrder(address, zip);
        QWidget::setTabOrder(zip, city);
        QWidget::setTabOrder(city, state);
        QWidget::setTabOrder(state, country);
        QWidget::setTabOrder(country, workphone);
        QWidget::setTabOrder(workphone, mobile);
        QWidget::setTabOrder(mobile, phone);
        QWidget::setTabOrder(phone, fax);
        QWidget::setTabOrder(fax, nick);
        QWidget::setTabOrder(nick, anniversary);
        QWidget::setTabOrder(anniversary, notes);
        QWidget::setTabOrder(notes, photo);

        retranslateUi(OneContact);

        QMetaObject::connectSlotsByName(OneContact);
    } // setupUi

    void retranslateUi(QWidget *OneContact)
    {
        name->setText(QCoreApplication::translate("OneContact", "name", nullptr));
        mail->setText(QCoreApplication::translate("OneContact", "<Mail>", nullptr));
        url->setText(QCoreApplication::translate("OneContact", "<Internet>", nullptr));
        groupBox->setTitle(QCoreApplication::translate("OneContact", "Address", nullptr));
        country->setText(QCoreApplication::translate("OneContact", "<country>", nullptr));
        state->setText(QCoreApplication::translate("OneContact", "<state>", nullptr));
        city->setText(QCoreApplication::translate("OneContact", "<city>", nullptr));
        zip->setText(QCoreApplication::translate("OneContact", "<zip>", nullptr));
        address->setText(QCoreApplication::translate("OneContact", "<address>", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("OneContact", "Phone", nullptr));
        label->setText(QCoreApplication::translate("OneContact", "Office", nullptr));
        workphone->setText(QCoreApplication::translate("OneContact", "workphone", nullptr));
        label_2->setText(QCoreApplication::translate("OneContact", "Mobile", nullptr));
        mobile->setText(QCoreApplication::translate("OneContact", "mobile", nullptr));
        label_3->setText(QCoreApplication::translate("OneContact", "Home", nullptr));
        phone->setText(QCoreApplication::translate("OneContact", "phone", nullptr));
        label_4->setText(QCoreApplication::translate("OneContact", "Fax", nullptr));
        fax->setText(QCoreApplication::translate("OneContact", "fax", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("OneContact", "Notes", nullptr));
        label_6->setText(QCoreApplication::translate("OneContact", "Nick", nullptr));
        nick->setText(QCoreApplication::translate("OneContact", "<nick>", nullptr));
        label_7->setText(QCoreApplication::translate("OneContact", "Anniversary", nullptr));
        anniversary->setText(QCoreApplication::translate("OneContact", "<anniversary>", nullptr));
        label_8->setText(QCoreApplication::translate("OneContact", "Custom notes", nullptr));
        notes->setText(QCoreApplication::translate("OneContact", "notes", nullptr));
        photo->setText(QCoreApplication::translate("OneContact", "Photo", nullptr));
        (void)OneContact;
    } // retranslateUi

};

namespace Ui {
    class OneContact: public Ui_OneContact {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ONECONTACT_H
