/****************************************************************************
** Meta object code from reading C++ file 'ThreadingMsgListModel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Model/ThreadingMsgListModel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ThreadingMsgListModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Mailbox__ThreadingMsgListModel_t {
    QByteArrayData data[53];
    char stringdata0[864];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__ThreadingMsgListModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__ThreadingMsgListModel_t qt_meta_stringdata_Imap__Mailbox__ThreadingMsgListModel = {
    {
QT_MOC_LITERAL(0, 0, 36), // "Imap::Mailbox::ThreadingMsgLi..."
QT_MOC_LITERAL(1, 37, 13), // "sortingFailed"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 7), // "resetMe"
QT_MOC_LITERAL(4, 60, 17), // "handleDataChanged"
QT_MOC_LITERAL(5, 78, 11), // "QModelIndex"
QT_MOC_LITERAL(6, 90, 7), // "topLeft"
QT_MOC_LITERAL(7, 98, 11), // "bottomRight"
QT_MOC_LITERAL(8, 110, 26), // "handleRowsAboutToBeRemoved"
QT_MOC_LITERAL(9, 137, 6), // "parent"
QT_MOC_LITERAL(10, 144, 5), // "start"
QT_MOC_LITERAL(11, 150, 3), // "end"
QT_MOC_LITERAL(12, 154, 17), // "handleRowsRemoved"
QT_MOC_LITERAL(13, 172, 27), // "handleRowsAboutToBeInserted"
QT_MOC_LITERAL(14, 200, 18), // "handleRowsInserted"
QT_MOC_LITERAL(15, 219, 22), // "slotThreadingAvailable"
QT_MOC_LITERAL(16, 242, 7), // "mailbox"
QT_MOC_LITERAL(17, 250, 9), // "algorithm"
QT_MOC_LITERAL(18, 260, 14), // "searchCriteria"
QT_MOC_LITERAL(19, 275, 39), // "QVector<Imap::Responses::Thre..."
QT_MOC_LITERAL(20, 315, 7), // "mapping"
QT_MOC_LITERAL(21, 323, 19), // "slotThreadingFailed"
QT_MOC_LITERAL(22, 343, 14), // "applyThreading"
QT_MOC_LITERAL(23, 358, 20), // "slotSortingAvailable"
QT_MOC_LITERAL(24, 379, 10), // "Imap::Uids"
QT_MOC_LITERAL(25, 390, 4), // "uids"
QT_MOC_LITERAL(26, 395, 17), // "slotSortingFailed"
QT_MOC_LITERAL(27, 413, 28), // "slotSortingIncrementalUpdate"
QT_MOC_LITERAL(28, 442, 50), // "Imap::Responses::ESearch::Inc..."
QT_MOC_LITERAL(29, 493, 7), // "updates"
QT_MOC_LITERAL(30, 501, 9), // "applySort"
QT_MOC_LITERAL(31, 511, 21), // "setUserWantsThreading"
QT_MOC_LITERAL(32, 533, 6), // "enable"
QT_MOC_LITERAL(33, 540, 33), // "setUserSearchingSortingPrefer..."
QT_MOC_LITERAL(34, 574, 16), // "searchConditions"
QT_MOC_LITERAL(35, 591, 13), // "SortCriterium"
QT_MOC_LITERAL(36, 605, 9), // "criterium"
QT_MOC_LITERAL(37, 615, 13), // "Qt::SortOrder"
QT_MOC_LITERAL(38, 629, 5), // "order"
QT_MOC_LITERAL(39, 635, 33), // "slotIncrementalThreadingAvail..."
QT_MOC_LITERAL(40, 669, 46), // "Responses::ESearch::Increment..."
QT_MOC_LITERAL(41, 716, 4), // "data"
QT_MOC_LITERAL(42, 721, 30), // "slotIncrementalThreadingFailed"
QT_MOC_LITERAL(43, 752, 12), // "delayedPrune"
QT_MOC_LITERAL(44, 765, 16), // "currentSortOrder"
QT_MOC_LITERAL(45, 782, 9), // "SORT_NONE"
QT_MOC_LITERAL(46, 792, 12), // "SORT_ARRIVAL"
QT_MOC_LITERAL(47, 805, 7), // "SORT_CC"
QT_MOC_LITERAL(48, 813, 9), // "SORT_DATE"
QT_MOC_LITERAL(49, 823, 9), // "SORT_FROM"
QT_MOC_LITERAL(50, 833, 9), // "SORT_SIZE"
QT_MOC_LITERAL(51, 843, 12), // "SORT_SUBJECT"
QT_MOC_LITERAL(52, 856, 7) // "SORT_TO"

    },
    "Imap::Mailbox::ThreadingMsgListModel\0"
    "sortingFailed\0\0resetMe\0handleDataChanged\0"
    "QModelIndex\0topLeft\0bottomRight\0"
    "handleRowsAboutToBeRemoved\0parent\0"
    "start\0end\0handleRowsRemoved\0"
    "handleRowsAboutToBeInserted\0"
    "handleRowsInserted\0slotThreadingAvailable\0"
    "mailbox\0algorithm\0searchCriteria\0"
    "QVector<Imap::Responses::ThreadingNode>\0"
    "mapping\0slotThreadingFailed\0applyThreading\0"
    "slotSortingAvailable\0Imap::Uids\0uids\0"
    "slotSortingFailed\0slotSortingIncrementalUpdate\0"
    "Imap::Responses::ESearch::IncrementalContextData_t\0"
    "updates\0applySort\0setUserWantsThreading\0"
    "enable\0setUserSearchingSortingPreference\0"
    "searchConditions\0SortCriterium\0criterium\0"
    "Qt::SortOrder\0order\0"
    "slotIncrementalThreadingAvailable\0"
    "Responses::ESearch::IncrementalThreadingData_t\0"
    "data\0slotIncrementalThreadingFailed\0"
    "delayedPrune\0currentSortOrder\0SORT_NONE\0"
    "SORT_ARRIVAL\0SORT_CC\0SORT_DATE\0SORT_FROM\0"
    "SORT_SIZE\0SORT_SUBJECT\0SORT_TO"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__ThreadingMsgListModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       1,  202, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  119,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,  120,    2, 0x0a /* Public */,
       4,    2,  121,    2, 0x0a /* Public */,
       8,    3,  126,    2, 0x0a /* Public */,
      12,    3,  133,    2, 0x0a /* Public */,
      13,    3,  140,    2, 0x0a /* Public */,
      14,    3,  147,    2, 0x0a /* Public */,
      15,    4,  154,    2, 0x0a /* Public */,
      21,    3,  163,    2, 0x0a /* Public */,
      22,    1,  170,    2, 0x0a /* Public */,
      23,    1,  173,    2, 0x0a /* Public */,
      26,    0,  176,    2, 0x0a /* Public */,
      27,    1,  177,    2, 0x0a /* Public */,
      30,    0,  180,    2, 0x0a /* Public */,
      31,    1,  181,    2, 0x0a /* Public */,
      33,    3,  184,    2, 0x0a /* Public */,
      33,    2,  191,    2, 0x2a /* Public | MethodCloned */,
      39,    1,  196,    2, 0x0a /* Public */,
      42,    0,  199,    2, 0x0a /* Public */,
      43,    0,  200,    2, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      44,    0,  201,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5, 0x80000000 | 5,    6,    7,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Int, QMetaType::Int,    9,   10,   11,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Int, QMetaType::Int,    9,   10,   11,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Int, QMetaType::Int,    9,   10,   11,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Int, QMetaType::Int,    9,   10,   11,
    QMetaType::Void, 0x80000000 | 5, QMetaType::QByteArray, QMetaType::QStringList, 0x80000000 | 19,   16,   17,   18,   20,
    QMetaType::Void, 0x80000000 | 5, QMetaType::QByteArray, QMetaType::QStringList,   16,   17,   18,
    QMetaType::Void, 0x80000000 | 19,   20,
    QMetaType::Void, 0x80000000 | 24,   25,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   32,
    QMetaType::Bool, QMetaType::QStringList, 0x80000000 | 35, 0x80000000 | 37,   34,   36,   38,
    QMetaType::Bool, QMetaType::QStringList, 0x80000000 | 35,   34,   36,
    QMetaType::Void, 0x80000000 | 40,   41,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    0x80000000 | 37,

 // enums: name, alias, flags, count, data
      35,   35, 0x0,    8,  207,

 // enum data: key, value
      45, uint(Imap::Mailbox::ThreadingMsgListModel::SORT_NONE),
      46, uint(Imap::Mailbox::ThreadingMsgListModel::SORT_ARRIVAL),
      47, uint(Imap::Mailbox::ThreadingMsgListModel::SORT_CC),
      48, uint(Imap::Mailbox::ThreadingMsgListModel::SORT_DATE),
      49, uint(Imap::Mailbox::ThreadingMsgListModel::SORT_FROM),
      50, uint(Imap::Mailbox::ThreadingMsgListModel::SORT_SIZE),
      51, uint(Imap::Mailbox::ThreadingMsgListModel::SORT_SUBJECT),
      52, uint(Imap::Mailbox::ThreadingMsgListModel::SORT_TO),

       0        // eod
};

void Imap::Mailbox::ThreadingMsgListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ThreadingMsgListModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sortingFailed(); break;
        case 1: _t->resetMe(); break;
        case 2: _t->handleDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 3: _t->handleRowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 4: _t->handleRowsRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 5: _t->handleRowsAboutToBeInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 6: _t->handleRowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 7: _t->slotThreadingAvailable((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])),(*reinterpret_cast< const QVector<Imap::Responses::ThreadingNode>(*)>(_a[4]))); break;
        case 8: _t->slotThreadingFailed((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3]))); break;
        case 9: _t->applyThreading((*reinterpret_cast< const QVector<Imap::Responses::ThreadingNode>(*)>(_a[1]))); break;
        case 10: _t->slotSortingAvailable((*reinterpret_cast< const Imap::Uids(*)>(_a[1]))); break;
        case 11: _t->slotSortingFailed(); break;
        case 12: _t->slotSortingIncrementalUpdate((*reinterpret_cast< const Imap::Responses::ESearch::IncrementalContextData_t(*)>(_a[1]))); break;
        case 13: _t->applySort(); break;
        case 14: _t->setUserWantsThreading((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: { bool _r = _t->setUserSearchingSortingPreference((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const SortCriterium(*)>(_a[2])),(*reinterpret_cast< const Qt::SortOrder(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->setUserSearchingSortingPreference((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const SortCriterium(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->slotIncrementalThreadingAvailable((*reinterpret_cast< const Responses::ESearch::IncrementalThreadingData_t(*)>(_a[1]))); break;
        case 18: _t->slotIncrementalThreadingFailed(); break;
        case 19: _t->delayedPrune(); break;
        case 20: { Qt::SortOrder _r = _t->currentSortOrder();
            if (_a[0]) *reinterpret_cast< Qt::SortOrder*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<Imap::Responses::ThreadingNode> >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<Imap::Responses::ThreadingNode> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ThreadingMsgListModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ThreadingMsgListModel::sortingFailed)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::ThreadingMsgListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractProxyModel::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__ThreadingMsgListModel.data,
    qt_meta_data_Imap__Mailbox__ThreadingMsgListModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::ThreadingMsgListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::ThreadingMsgListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__ThreadingMsgListModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractProxyModel::qt_metacast(_clname);
}

int Imap::Mailbox::ThreadingMsgListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void Imap::Mailbox::ThreadingMsgListModel::sortingFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
