/****************************************************************************
** Meta object code from reading C++ file 'MsgListModel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Model/MsgListModel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MsgListModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Mailbox__MsgListModel_t {
    QByteArrayData data[23];
    char stringdata0[310];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__MsgListModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__MsgListModel_t qt_meta_stringdata_Imap__Mailbox__MsgListModel = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Imap::Mailbox::MsgListModel"
QT_MOC_LITERAL(1, 28, 14), // "messageRemoved"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 14), // "mailboxChanged"
QT_MOC_LITERAL(4, 59, 11), // "QModelIndex"
QT_MOC_LITERAL(5, 71, 7), // "mailbox"
QT_MOC_LITERAL(6, 79, 17), // "messagesAvailable"
QT_MOC_LITERAL(7, 97, 17), // "indexStateChanged"
QT_MOC_LITERAL(8, 115, 7), // "resetMe"
QT_MOC_LITERAL(9, 123, 10), // "setMailbox"
QT_MOC_LITERAL(10, 134, 5), // "index"
QT_MOC_LITERAL(11, 140, 11), // "mailboxName"
QT_MOC_LITERAL(12, 152, 17), // "handleDataChanged"
QT_MOC_LITERAL(13, 170, 7), // "topLeft"
QT_MOC_LITERAL(14, 178, 11), // "bottomRight"
QT_MOC_LITERAL(15, 190, 26), // "handleRowsAboutToBeRemoved"
QT_MOC_LITERAL(16, 217, 6), // "parent"
QT_MOC_LITERAL(17, 224, 5), // "start"
QT_MOC_LITERAL(18, 230, 3), // "end"
QT_MOC_LITERAL(19, 234, 17), // "handleRowsRemoved"
QT_MOC_LITERAL(20, 252, 27), // "handleRowsAboutToBeInserted"
QT_MOC_LITERAL(21, 280, 18), // "handleRowsInserted"
QT_MOC_LITERAL(22, 299, 10) // "itemsValid"

    },
    "Imap::Mailbox::MsgListModel\0messageRemoved\0"
    "\0mailboxChanged\0QModelIndex\0mailbox\0"
    "messagesAvailable\0indexStateChanged\0"
    "resetMe\0setMailbox\0index\0mailboxName\0"
    "handleDataChanged\0topLeft\0bottomRight\0"
    "handleRowsAboutToBeRemoved\0parent\0"
    "start\0end\0handleRowsRemoved\0"
    "handleRowsAboutToBeInserted\0"
    "handleRowsInserted\0itemsValid"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__MsgListModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       1,  122, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       3,    1,   77,    2, 0x06 /* Public */,
       6,    0,   80,    2, 0x06 /* Public */,
       7,    0,   81,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,   82,    2, 0x0a /* Public */,
       9,    1,   83,    2, 0x0a /* Public */,
       9,    1,   86,    2, 0x0a /* Public */,
      12,    2,   89,    2, 0x0a /* Public */,
      15,    3,   94,    2, 0x0a /* Public */,
      19,    3,  101,    2, 0x0a /* Public */,
      20,    3,  108,    2, 0x0a /* Public */,
      21,    3,  115,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::VoidStar,    2,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,   10,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 4,   13,   14,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Int,   16,   17,   18,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Int,   16,   17,   18,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Int,   16,   17,   18,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int, QMetaType::Int,   16,   17,   18,

 // properties: name, type, flags
      22, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       3,

       0        // eod
};

void Imap::Mailbox::MsgListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MsgListModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->messageRemoved((*reinterpret_cast< void*(*)>(_a[1]))); break;
        case 1: _t->mailboxChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 2: _t->messagesAvailable(); break;
        case 3: _t->indexStateChanged(); break;
        case 4: _t->resetMe(); break;
        case 5: _t->setMailbox((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 6: _t->setMailbox((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->handleDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 8: _t->handleRowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 9: _t->handleRowsRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 10: _t->handleRowsAboutToBeInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 11: _t->handleRowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MsgListModel::*)(void * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MsgListModel::messageRemoved)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MsgListModel::*)(const QModelIndex & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MsgListModel::mailboxChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (MsgListModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MsgListModel::messagesAvailable)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (MsgListModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MsgListModel::indexStateChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<MsgListModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->itemsValid(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::MsgListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractProxyModel::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__MsgListModel.data,
    qt_meta_data_Imap__Mailbox__MsgListModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::MsgListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::MsgListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__MsgListModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractProxyModel::qt_metacast(_clname);
}

int Imap::Mailbox::MsgListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Imap::Mailbox::MsgListModel::messageRemoved(void * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Imap::Mailbox::MsgListModel::mailboxChanged(const QModelIndex & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Imap::Mailbox::MsgListModel::messagesAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Imap::Mailbox::MsgListModel::indexStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
