/****************************************************************************
** Meta object code from reading C++ file 'PartWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/PartWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PartWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gui__MultipartAlternativeWidget_t {
    QByteArrayData data[1];
    char stringdata0[32];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__MultipartAlternativeWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__MultipartAlternativeWidget_t qt_meta_stringdata_Gui__MultipartAlternativeWidget = {
    {
QT_MOC_LITERAL(0, 0, 31) // "Gui::MultipartAlternativeWidget"

    },
    "Gui::MultipartAlternativeWidget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__MultipartAlternativeWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Gui::MultipartAlternativeWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::MultipartAlternativeWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QTabWidget::staticMetaObject>(),
    qt_meta_stringdata_Gui__MultipartAlternativeWidget.data,
    qt_meta_data_Gui__MultipartAlternativeWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::MultipartAlternativeWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::MultipartAlternativeWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__MultipartAlternativeWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AbstractPartWidget"))
        return static_cast< AbstractPartWidget*>(this);
    return QTabWidget::qt_metacast(_clname);
}

int Gui::MultipartAlternativeWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTabWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_Gui__PartStatusWidget_t {
    QByteArrayData data[3];
    char stringdata0[35];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__PartStatusWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__PartStatusWidget_t qt_meta_stringdata_Gui__PartStatusWidget = {
    {
QT_MOC_LITERAL(0, 0, 21), // "Gui::PartStatusWidget"
QT_MOC_LITERAL(1, 22, 11), // "showDetails"
QT_MOC_LITERAL(2, 34, 0) // ""

    },
    "Gui::PartStatusWidget\0showDetails\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__PartStatusWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void Gui::PartStatusWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PartStatusWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->showDetails(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::PartStatusWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_Gui__PartStatusWidget.data,
    qt_meta_data_Gui__PartStatusWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::PartStatusWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::PartStatusWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__PartStatusWidget.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int Gui::PartStatusWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_Gui__AsynchronousPartWidget_t {
    QByteArrayData data[17];
    char stringdata0[222];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__AsynchronousPartWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__AsynchronousPartWidget_t qt_meta_stringdata_Gui__AsynchronousPartWidget = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Gui::AsynchronousPartWidget"
QT_MOC_LITERAL(1, 28, 18), // "handleRowsInserted"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 60, 6), // "parent"
QT_MOC_LITERAL(5, 67, 5), // "first"
QT_MOC_LITERAL(6, 73, 4), // "last"
QT_MOC_LITERAL(7, 78, 19), // "handleLayoutChanged"
QT_MOC_LITERAL(8, 98, 28), // "QList<QPersistentModelIndex>"
QT_MOC_LITERAL(9, 127, 7), // "parents"
QT_MOC_LITERAL(10, 135, 11), // "handleError"
QT_MOC_LITERAL(11, 147, 6), // "status"
QT_MOC_LITERAL(12, 154, 7), // "details"
QT_MOC_LITERAL(13, 162, 17), // "handleDataChanged"
QT_MOC_LITERAL(14, 180, 7), // "topLeft"
QT_MOC_LITERAL(15, 188, 11), // "bottomRight"
QT_MOC_LITERAL(16, 200, 21) // "updateStatusIndicator"

    },
    "Gui::AsynchronousPartWidget\0"
    "handleRowsInserted\0\0QModelIndex\0parent\0"
    "first\0last\0handleLayoutChanged\0"
    "QList<QPersistentModelIndex>\0parents\0"
    "handleError\0status\0details\0handleDataChanged\0"
    "topLeft\0bottomRight\0updateStatusIndicator"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__AsynchronousPartWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   39,    2, 0x09 /* Protected */,
       7,    1,   46,    2, 0x09 /* Protected */,
      10,    3,   49,    2, 0x09 /* Protected */,
      13,    2,   56,    2, 0x09 /* Protected */,
      16,    0,   61,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    4,    5,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString, QMetaType::QString,    4,   11,   12,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,   14,   15,
    QMetaType::Void,

       0        // eod
};

void Gui::AsynchronousPartWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AsynchronousPartWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->handleRowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 1: _t->handleLayoutChanged((*reinterpret_cast< const QList<QPersistentModelIndex>(*)>(_a[1]))); break;
        case 2: _t->handleError((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 3: _t->handleDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 4: _t->updateStatusIndicator(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::AsynchronousPartWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_Gui__AsynchronousPartWidget.data,
    qt_meta_data_Gui__AsynchronousPartWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::AsynchronousPartWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::AsynchronousPartWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__AsynchronousPartWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AbstractPartWidget"))
        return static_cast< AbstractPartWidget*>(this);
    return QFrame::qt_metacast(_clname);
}

int Gui::AsynchronousPartWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
struct qt_meta_stringdata_Gui__MultipartSignedEncryptedWidget_t {
    QByteArrayData data[3];
    char stringdata0[59];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__MultipartSignedEncryptedWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__MultipartSignedEncryptedWidget_t qt_meta_stringdata_Gui__MultipartSignedEncryptedWidget = {
    {
QT_MOC_LITERAL(0, 0, 35), // "Gui::MultipartSignedEncrypted..."
QT_MOC_LITERAL(1, 36, 21), // "updateStatusIndicator"
QT_MOC_LITERAL(2, 58, 0) // ""

    },
    "Gui::MultipartSignedEncryptedWidget\0"
    "updateStatusIndicator\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__MultipartSignedEncryptedWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void Gui::MultipartSignedEncryptedWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MultipartSignedEncryptedWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateStatusIndicator(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::MultipartSignedEncryptedWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<AsynchronousPartWidget::staticMetaObject>(),
    qt_meta_stringdata_Gui__MultipartSignedEncryptedWidget.data,
    qt_meta_data_Gui__MultipartSignedEncryptedWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::MultipartSignedEncryptedWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::MultipartSignedEncryptedWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__MultipartSignedEncryptedWidget.stringdata0))
        return static_cast<void*>(this);
    return AsynchronousPartWidget::qt_metacast(_clname);
}

int Gui::MultipartSignedEncryptedWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AsynchronousPartWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_Gui__GenericMultipartWidget_t {
    QByteArrayData data[1];
    char stringdata0[28];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__GenericMultipartWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__GenericMultipartWidget_t qt_meta_stringdata_Gui__GenericMultipartWidget = {
    {
QT_MOC_LITERAL(0, 0, 27) // "Gui::GenericMultipartWidget"

    },
    "Gui::GenericMultipartWidget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__GenericMultipartWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Gui::GenericMultipartWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::GenericMultipartWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Gui__GenericMultipartWidget.data,
    qt_meta_data_Gui__GenericMultipartWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::GenericMultipartWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::GenericMultipartWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__GenericMultipartWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AbstractPartWidget"))
        return static_cast< AbstractPartWidget*>(this);
    return QWidget::qt_metacast(_clname);
}

int Gui::GenericMultipartWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_Gui__Message822Widget_t {
    QByteArrayData data[1];
    char stringdata0[22];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__Message822Widget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__Message822Widget_t qt_meta_stringdata_Gui__Message822Widget = {
    {
QT_MOC_LITERAL(0, 0, 21) // "Gui::Message822Widget"

    },
    "Gui::Message822Widget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__Message822Widget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Gui::Message822Widget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Gui::Message822Widget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Gui__Message822Widget.data,
    qt_meta_data_Gui__Message822Widget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::Message822Widget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::Message822Widget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__Message822Widget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AbstractPartWidget"))
        return static_cast< AbstractPartWidget*>(this);
    return QWidget::qt_metacast(_clname);
}

int Gui::Message822Widget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
