/****************************************************************************
** Meta object code from reading C++ file 'AttachmentView.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/AttachmentView.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AttachmentView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gui__AttachmentView_t {
    QByteArrayData data[17];
    char stringdata0[295];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__AttachmentView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__AttachmentView_t qt_meta_stringdata_Gui__AttachmentView = {
    {
QT_MOC_LITERAL(0, 0, 19), // "Gui::AttachmentView"
QT_MOC_LITERAL(1, 20, 22), // "slotDownloadAttachment"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 18), // "slotOpenAttachment"
QT_MOC_LITERAL(4, 63, 17), // "showMessageSource"
QT_MOC_LITERAL(5, 81, 13), // "indicateHover"
QT_MOC_LITERAL(6, 95, 24), // "openDownloadedAttachment"
QT_MOC_LITERAL(7, 120, 27), // "slotFileNameRequestedOnOpen"
QT_MOC_LITERAL(8, 148, 8), // "QString*"
QT_MOC_LITERAL(9, 157, 8), // "fileName"
QT_MOC_LITERAL(10, 166, 21), // "slotFileNameRequested"
QT_MOC_LITERAL(11, 188, 19), // "enableDownloadAgain"
QT_MOC_LITERAL(12, 208, 12), // "onOpenFailed"
QT_MOC_LITERAL(13, 221, 29), // "updateShowHideAttachmentState"
QT_MOC_LITERAL(14, 251, 8), // "showMenu"
QT_MOC_LITERAL(15, 260, 17), // "showMenuOrPreview"
QT_MOC_LITERAL(16, 278, 16) // "toggleIconCursor"

    },
    "Gui::AttachmentView\0slotDownloadAttachment\0"
    "\0slotOpenAttachment\0showMessageSource\0"
    "indicateHover\0openDownloadedAttachment\0"
    "slotFileNameRequestedOnOpen\0QString*\0"
    "fileName\0slotFileNameRequested\0"
    "enableDownloadAgain\0onOpenFailed\0"
    "updateShowHideAttachmentState\0showMenu\0"
    "showMenuOrPreview\0toggleIconCursor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__AttachmentView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x08 /* Private */,
       3,    0,   80,    2, 0x08 /* Private */,
       4,    0,   81,    2, 0x08 /* Private */,
       5,    0,   82,    2, 0x08 /* Private */,
       6,    0,   83,    2, 0x08 /* Private */,
       7,    1,   84,    2, 0x08 /* Private */,
      10,    1,   87,    2, 0x08 /* Private */,
      11,    0,   90,    2, 0x08 /* Private */,
      12,    0,   91,    2, 0x08 /* Private */,
      13,    0,   92,    2, 0x08 /* Private */,
      14,    0,   93,    2, 0x08 /* Private */,
      15,    0,   94,    2, 0x08 /* Private */,
      16,    0,   95,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::AttachmentView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AttachmentView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotDownloadAttachment(); break;
        case 1: _t->slotOpenAttachment(); break;
        case 2: _t->showMessageSource(); break;
        case 3: _t->indicateHover(); break;
        case 4: _t->openDownloadedAttachment(); break;
        case 5: _t->slotFileNameRequestedOnOpen((*reinterpret_cast< QString*(*)>(_a[1]))); break;
        case 6: _t->slotFileNameRequested((*reinterpret_cast< QString*(*)>(_a[1]))); break;
        case 7: _t->enableDownloadAgain(); break;
        case 8: _t->onOpenFailed(); break;
        case 9: _t->updateShowHideAttachmentState(); break;
        case 10: _t->showMenu(); break;
        case 11: _t->showMenuOrPreview(); break;
        case 12: _t->toggleIconCursor(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::AttachmentView::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_Gui__AttachmentView.data,
    qt_meta_data_Gui__AttachmentView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::AttachmentView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::AttachmentView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__AttachmentView.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "AbstractPartWidget"))
        return static_cast< AbstractPartWidget*>(this);
    return QFrame::qt_metacast(_clname);
}

int Gui::AttachmentView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
