/* Copyright (C) 2006 - 2014 Jan Kundrát <jkt@flaska.net>

   This file is part of the Trojita Qt IMAP e-mail client,
   http://trojita.flaska.net/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "ImapSubmit.h"
#include "Common/InvokeMethod.h"
#include "Imap/Model/Model.h"
#include "Imap/Tasks/UidSubmitTask.h"

namespace MSA
{

ImapSubmit::ImapSubmit(QObject *parent, Imap::Mailbox::Model *model):
    AbstractMSA(parent), m_model(model)
{
}

void ImapSubmit::cancel()
{
    emit error(QStringLiteral("MSA::ImapSubmit::cancel() is not implemented; this is a bug in Trojita."));
    Q_ASSERT(false);
}

bool ImapSubmit::supportsImapSending() const
{
    return true;
}

void ImapSubmit::sendImap(const QString &mailbox, const uint uidValidity, const uint uid,
                          const Imap::Mailbox::UidSubmitOptionsList options)
{
    Imap::Mailbox::UidSubmitTask *submitTask = m_model->sendMailViaUidSubmit(mailbox, uidValidity, uid, options);
    Q_ASSERT(submitTask);
    connect(submitTask, &Imap::Mailbox::ImapTask::completed, this, &AbstractMSA::sent);
    connect(submitTask, &Imap::Mailbox::ImapTask::failed, this, &AbstractMSA::error);
    EMIT_LATER_NOARG(this, sending);
}

ImapSubmitFactory::ImapSubmitFactory(Imap::Mailbox::Model *model):
    m_model(model)
{
}

ImapSubmitFactory::~ImapSubmitFactory()
{
}

AbstractMSA *ImapSubmitFactory::create(QObject *parent) const
{
    return new ImapSubmit(parent, m_model);
}

}
