/****************************************************************************
** Meta object code from reading C++ file 'Window.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/Window.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Window.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gui__MainWindow_t {
    QByteArrayData data[120];
    char stringdata0[2107];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__MainWindow_t qt_meta_stringdata_Gui__MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 15), // "Gui::MainWindow"
QT_MOC_LITERAL(1, 16, 18), // "slotComposeMailUrl"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 3), // "url"
QT_MOC_LITERAL(4, 40, 15), // "slotComposeMail"
QT_MOC_LITERAL(5, 56, 19), // "invokeContactEditor"
QT_MOC_LITERAL(6, 76, 23), // "showContextMenuMboxTree"
QT_MOC_LITERAL(7, 100, 8), // "position"
QT_MOC_LITERAL(8, 109, 26), // "showContextMenuMsgListTree"
QT_MOC_LITERAL(9, 136, 18), // "slotReloadMboxList"
QT_MOC_LITERAL(10, 155, 14), // "slotResyncMbox"
QT_MOC_LITERAL(11, 170, 13), // "alertReceived"
QT_MOC_LITERAL(12, 184, 7), // "message"
QT_MOC_LITERAL(13, 192, 20), // "networkPolicyOffline"
QT_MOC_LITERAL(14, 213, 22), // "networkPolicyExpensive"
QT_MOC_LITERAL(15, 236, 19), // "networkPolicyOnline"
QT_MOC_LITERAL(16, 256, 16), // "slotShowSettings"
QT_MOC_LITERAL(17, 273, 16), // "slotShowImapInfo"
QT_MOC_LITERAL(18, 290, 11), // "slotExpunge"
QT_MOC_LITERAL(19, 302, 9), // "imapError"
QT_MOC_LITERAL(20, 312, 12), // "networkError"
QT_MOC_LITERAL(21, 325, 10), // "cacheError"
QT_MOC_LITERAL(22, 336, 23), // "authenticationRequested"
QT_MOC_LITERAL(23, 360, 22), // "authenticationContinue"
QT_MOC_LITERAL(24, 383, 4), // "pass"
QT_MOC_LITERAL(25, 388, 12), // "errorMessage"
QT_MOC_LITERAL(26, 401, 14), // "checkSslPolicy"
QT_MOC_LITERAL(27, 416, 17), // "slotManageContact"
QT_MOC_LITERAL(28, 434, 13), // "slotEditDraft"
QT_MOC_LITERAL(29, 448, 11), // "slotReplyTo"
QT_MOC_LITERAL(30, 460, 17), // "slotReplyAllButMe"
QT_MOC_LITERAL(31, 478, 12), // "slotReplyAll"
QT_MOC_LITERAL(32, 491, 13), // "slotReplyList"
QT_MOC_LITERAL(33, 505, 14), // "slotReplyGuess"
QT_MOC_LITERAL(34, 520, 23), // "slotForwardAsAttachment"
QT_MOC_LITERAL(35, 544, 10), // "slotResend"
QT_MOC_LITERAL(36, 555, 24), // "slotUpdateMessageActions"
QT_MOC_LITERAL(37, 580, 9), // "handleTag"
QT_MOC_LITERAL(38, 590, 7), // "checked"
QT_MOC_LITERAL(39, 598, 5), // "index"
QT_MOC_LITERAL(40, 604, 16), // "handleMarkAsRead"
QT_MOC_LITERAL(41, 621, 19), // "handleMarkAsDeleted"
QT_MOC_LITERAL(42, 641, 19), // "handleMarkAsFlagged"
QT_MOC_LITERAL(43, 661, 16), // "handleMarkAsJunk"
QT_MOC_LITERAL(44, 678, 19), // "handleMarkAsNotJunk"
QT_MOC_LITERAL(45, 698, 23), // "slotMoveToArchiveFailed"
QT_MOC_LITERAL(46, 722, 5), // "error"
QT_MOC_LITERAL(47, 728, 19), // "handleMoveToArchive"
QT_MOC_LITERAL(48, 748, 14), // "slotNextUnread"
QT_MOC_LITERAL(49, 763, 18), // "slotPreviousUnread"
QT_MOC_LITERAL(50, 782, 14), // "msgListClicked"
QT_MOC_LITERAL(51, 797, 11), // "QModelIndex"
QT_MOC_LITERAL(52, 809, 25), // "openCompleteMessageWidget"
QT_MOC_LITERAL(53, 835, 29), // "slotCreateMailboxBelowCurrent"
QT_MOC_LITERAL(54, 865, 26), // "slotMarkCurrentMailboxRead"
QT_MOC_LITERAL(55, 892, 20), // "slotCreateTopMailbox"
QT_MOC_LITERAL(56, 913, 24), // "slotDeleteCurrentMailbox"
QT_MOC_LITERAL(57, 938, 20), // "handleTrayIconChange"
QT_MOC_LITERAL(58, 959, 27), // "slotSubscribeCurrentMailbox"
QT_MOC_LITERAL(59, 987, 22), // "slotShowOnlySubscribed"
QT_MOC_LITERAL(60, 1010, 18), // "updateMessageFlags"
QT_MOC_LITERAL(61, 1029, 20), // "updateMessageFlagsOf"
QT_MOC_LITERAL(62, 1050, 15), // "scrollMessageUp"
QT_MOC_LITERAL(63, 1066, 23), // "slotMessageModelChanged"
QT_MOC_LITERAL(64, 1090, 19), // "QAbstractItemModel*"
QT_MOC_LITERAL(65, 1110, 5), // "model"
QT_MOC_LITERAL(66, 1116, 20), // "showConnectionStatus"
QT_MOC_LITERAL(67, 1137, 8), // "parserId"
QT_MOC_LITERAL(68, 1146, 21), // "Imap::ConnectionState"
QT_MOC_LITERAL(69, 1168, 5), // "state"
QT_MOC_LITERAL(70, 1174, 18), // "slotShowLinkTarget"
QT_MOC_LITERAL(71, 1193, 4), // "link"
QT_MOC_LITERAL(72, 1198, 20), // "slotShowAboutTrojita"
QT_MOC_LITERAL(73, 1219, 19), // "slotDonateToTrojita"
QT_MOC_LITERAL(74, 1239, 26), // "slotSaveCurrentMessageBody"
QT_MOC_LITERAL(75, 1266, 17), // "slotViewMsgSource"
QT_MOC_LITERAL(76, 1284, 18), // "slotViewMsgHeaders"
QT_MOC_LITERAL(77, 1303, 17), // "slotThreadMsgList"
QT_MOC_LITERAL(78, 1321, 12), // "slotHideRead"
QT_MOC_LITERAL(79, 1334, 28), // "slotSortingPreferenceChanged"
QT_MOC_LITERAL(80, 1363, 20), // "slotSortingConfirmed"
QT_MOC_LITERAL(81, 1384, 6), // "column"
QT_MOC_LITERAL(82, 1391, 13), // "Qt::SortOrder"
QT_MOC_LITERAL(83, 1405, 5), // "order"
QT_MOC_LITERAL(84, 1411, 19), // "slotSearchRequested"
QT_MOC_LITERAL(85, 1431, 16), // "searchConditions"
QT_MOC_LITERAL(86, 1448, 23), // "slotCapabilitiesUpdated"
QT_MOC_LITERAL(87, 1472, 12), // "capabilities"
QT_MOC_LITERAL(88, 1485, 23), // "slotMailboxDeleteFailed"
QT_MOC_LITERAL(89, 1509, 7), // "mailbox"
QT_MOC_LITERAL(90, 1517, 3), // "msg"
QT_MOC_LITERAL(91, 1521, 23), // "slotMailboxCreateFailed"
QT_MOC_LITERAL(92, 1545, 21), // "slotMailboxSyncFailed"
QT_MOC_LITERAL(93, 1567, 18), // "slotMailboxChanged"
QT_MOC_LITERAL(94, 1586, 25), // "slotDownloadTransferError"
QT_MOC_LITERAL(95, 1612, 11), // "errorString"
QT_MOC_LITERAL(96, 1624, 36), // "slotDownloadMessageFileNameRe..."
QT_MOC_LITERAL(97, 1661, 8), // "QString*"
QT_MOC_LITERAL(98, 1670, 8), // "fileName"
QT_MOC_LITERAL(99, 1679, 25), // "slotScrollToUnseenMessage"
QT_MOC_LITERAL(100, 1705, 19), // "slotScrollToCurrent"
QT_MOC_LITERAL(101, 1725, 21), // "slotUpdateWindowTitle"
QT_MOC_LITERAL(102, 1747, 17), // "slotLayoutCompact"
QT_MOC_LITERAL(103, 1765, 14), // "slotLayoutWide"
QT_MOC_LITERAL(104, 1780, 19), // "slotLayoutOneAtTime"
QT_MOC_LITERAL(105, 1800, 17), // "saveSizesAndState"
QT_MOC_LITERAL(106, 1818, 19), // "saveRawStateSetting"
QT_MOC_LITERAL(107, 1838, 7), // "enabled"
QT_MOC_LITERAL(108, 1846, 37), // "possiblyLoadMessageOnSplitter..."
QT_MOC_LITERAL(109, 1884, 23), // "updateNetworkIndication"
QT_MOC_LITERAL(110, 1908, 22), // "desktopGeometryChanged"
QT_MOC_LITERAL(111, 1931, 17), // "slotIconActivated"
QT_MOC_LITERAL(112, 1949, 33), // "QSystemTrayIcon::ActivationRe..."
QT_MOC_LITERAL(113, 1983, 6), // "reason"
QT_MOC_LITERAL(114, 1990, 17), // "slotToggleSysTray"
QT_MOC_LITERAL(115, 2008, 23), // "slotResetReconnectState"
QT_MOC_LITERAL(116, 2032, 18), // "slotPluginsChanged"
QT_MOC_LITERAL(117, 2051, 17), // "showStatusMessage"
QT_MOC_LITERAL(118, 2069, 23), // "slotFavoriteTagsChanged"
QT_MOC_LITERAL(119, 2093, 13) // "recoverDrafts"

    },
    "Gui::MainWindow\0slotComposeMailUrl\0\0"
    "url\0slotComposeMail\0invokeContactEditor\0"
    "showContextMenuMboxTree\0position\0"
    "showContextMenuMsgListTree\0"
    "slotReloadMboxList\0slotResyncMbox\0"
    "alertReceived\0message\0networkPolicyOffline\0"
    "networkPolicyExpensive\0networkPolicyOnline\0"
    "slotShowSettings\0slotShowImapInfo\0"
    "slotExpunge\0imapError\0networkError\0"
    "cacheError\0authenticationRequested\0"
    "authenticationContinue\0pass\0errorMessage\0"
    "checkSslPolicy\0slotManageContact\0"
    "slotEditDraft\0slotReplyTo\0slotReplyAllButMe\0"
    "slotReplyAll\0slotReplyList\0slotReplyGuess\0"
    "slotForwardAsAttachment\0slotResend\0"
    "slotUpdateMessageActions\0handleTag\0"
    "checked\0index\0handleMarkAsRead\0"
    "handleMarkAsDeleted\0handleMarkAsFlagged\0"
    "handleMarkAsJunk\0handleMarkAsNotJunk\0"
    "slotMoveToArchiveFailed\0error\0"
    "handleMoveToArchive\0slotNextUnread\0"
    "slotPreviousUnread\0msgListClicked\0"
    "QModelIndex\0openCompleteMessageWidget\0"
    "slotCreateMailboxBelowCurrent\0"
    "slotMarkCurrentMailboxRead\0"
    "slotCreateTopMailbox\0slotDeleteCurrentMailbox\0"
    "handleTrayIconChange\0slotSubscribeCurrentMailbox\0"
    "slotShowOnlySubscribed\0updateMessageFlags\0"
    "updateMessageFlagsOf\0scrollMessageUp\0"
    "slotMessageModelChanged\0QAbstractItemModel*\0"
    "model\0showConnectionStatus\0parserId\0"
    "Imap::ConnectionState\0state\0"
    "slotShowLinkTarget\0link\0slotShowAboutTrojita\0"
    "slotDonateToTrojita\0slotSaveCurrentMessageBody\0"
    "slotViewMsgSource\0slotViewMsgHeaders\0"
    "slotThreadMsgList\0slotHideRead\0"
    "slotSortingPreferenceChanged\0"
    "slotSortingConfirmed\0column\0Qt::SortOrder\0"
    "order\0slotSearchRequested\0searchConditions\0"
    "slotCapabilitiesUpdated\0capabilities\0"
    "slotMailboxDeleteFailed\0mailbox\0msg\0"
    "slotMailboxCreateFailed\0slotMailboxSyncFailed\0"
    "slotMailboxChanged\0slotDownloadTransferError\0"
    "errorString\0slotDownloadMessageFileNameRequested\0"
    "QString*\0fileName\0slotScrollToUnseenMessage\0"
    "slotScrollToCurrent\0slotUpdateWindowTitle\0"
    "slotLayoutCompact\0slotLayoutWide\0"
    "slotLayoutOneAtTime\0saveSizesAndState\0"
    "saveRawStateSetting\0enabled\0"
    "possiblyLoadMessageOnSplittersChanged\0"
    "updateNetworkIndication\0desktopGeometryChanged\0"
    "slotIconActivated\0QSystemTrayIcon::ActivationReason\0"
    "reason\0slotToggleSysTray\0"
    "slotResetReconnectState\0slotPluginsChanged\0"
    "showStatusMessage\0slotFavoriteTagsChanged\0"
    "recoverDrafts"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      91,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  469,    2, 0x0a /* Public */,
       4,    0,  472,    2, 0x0a /* Public */,
       5,    0,  473,    2, 0x0a /* Public */,
       6,    1,  474,    2, 0x08 /* Private */,
       8,    1,  477,    2, 0x08 /* Private */,
       9,    0,  480,    2, 0x08 /* Private */,
      10,    0,  481,    2, 0x08 /* Private */,
      11,    1,  482,    2, 0x08 /* Private */,
      13,    0,  485,    2, 0x08 /* Private */,
      14,    0,  486,    2, 0x08 /* Private */,
      15,    0,  487,    2, 0x08 /* Private */,
      16,    0,  488,    2, 0x08 /* Private */,
      17,    0,  489,    2, 0x08 /* Private */,
      18,    0,  490,    2, 0x08 /* Private */,
      19,    1,  491,    2, 0x08 /* Private */,
      20,    1,  494,    2, 0x08 /* Private */,
      21,    1,  497,    2, 0x08 /* Private */,
      22,    0,  500,    2, 0x08 /* Private */,
      23,    2,  501,    2, 0x08 /* Private */,
      23,    1,  506,    2, 0x28 /* Private | MethodCloned */,
      26,    0,  509,    2, 0x08 /* Private */,
      27,    1,  510,    2, 0x08 /* Private */,
      28,    0,  513,    2, 0x08 /* Private */,
      29,    0,  514,    2, 0x08 /* Private */,
      30,    0,  515,    2, 0x08 /* Private */,
      31,    0,  516,    2, 0x08 /* Private */,
      32,    0,  517,    2, 0x08 /* Private */,
      33,    0,  518,    2, 0x08 /* Private */,
      34,    0,  519,    2, 0x08 /* Private */,
      35,    0,  520,    2, 0x08 /* Private */,
      36,    0,  521,    2, 0x08 /* Private */,
      37,    2,  522,    2, 0x08 /* Private */,
      40,    1,  527,    2, 0x08 /* Private */,
      41,    1,  530,    2, 0x08 /* Private */,
      42,    1,  533,    2, 0x08 /* Private */,
      43,    1,  536,    2, 0x08 /* Private */,
      44,    1,  539,    2, 0x08 /* Private */,
      45,    1,  542,    2, 0x08 /* Private */,
      47,    0,  545,    2, 0x08 /* Private */,
      48,    0,  546,    2, 0x08 /* Private */,
      49,    0,  547,    2, 0x08 /* Private */,
      50,    1,  548,    2, 0x08 /* Private */,
      52,    0,  551,    2, 0x08 /* Private */,
      53,    0,  552,    2, 0x08 /* Private */,
      54,    0,  553,    2, 0x08 /* Private */,
      55,    0,  554,    2, 0x08 /* Private */,
      56,    0,  555,    2, 0x08 /* Private */,
      57,    0,  556,    2, 0x08 /* Private */,
      58,    0,  557,    2, 0x08 /* Private */,
      59,    0,  558,    2, 0x08 /* Private */,
      60,    0,  559,    2, 0x08 /* Private */,
      61,    1,  560,    2, 0x08 /* Private */,
      62,    0,  563,    2, 0x08 /* Private */,
      63,    1,  564,    2, 0x08 /* Private */,
      66,    2,  567,    2, 0x08 /* Private */,
      70,    1,  572,    2, 0x08 /* Private */,
      72,    0,  575,    2, 0x08 /* Private */,
      73,    0,  576,    2, 0x08 /* Private */,
      74,    0,  577,    2, 0x08 /* Private */,
      75,    0,  578,    2, 0x08 /* Private */,
      76,    0,  579,    2, 0x08 /* Private */,
      77,    0,  580,    2, 0x08 /* Private */,
      78,    0,  581,    2, 0x08 /* Private */,
      79,    0,  582,    2, 0x08 /* Private */,
      80,    2,  583,    2, 0x08 /* Private */,
      84,    1,  588,    2, 0x08 /* Private */,
      86,    1,  591,    2, 0x08 /* Private */,
      88,    2,  594,    2, 0x08 /* Private */,
      91,    2,  599,    2, 0x08 /* Private */,
      92,    2,  604,    2, 0x08 /* Private */,
      93,    1,  609,    2, 0x08 /* Private */,
      94,    1,  612,    2, 0x08 /* Private */,
      96,    1,  615,    2, 0x08 /* Private */,
      99,    0,  618,    2, 0x08 /* Private */,
     100,    0,  619,    2, 0x08 /* Private */,
     101,    0,  620,    2, 0x08 /* Private */,
     102,    0,  621,    2, 0x08 /* Private */,
     103,    0,  622,    2, 0x08 /* Private */,
     104,    0,  623,    2, 0x08 /* Private */,
     105,    0,  624,    2, 0x08 /* Private */,
     106,    1,  625,    2, 0x08 /* Private */,
     108,    0,  628,    2, 0x08 /* Private */,
     109,    0,  629,    2, 0x08 /* Private */,
     110,    0,  630,    2, 0x08 /* Private */,
     111,    1,  631,    2, 0x08 /* Private */,
     114,    0,  634,    2, 0x08 /* Private */,
     115,    0,  635,    2, 0x08 /* Private */,
     116,    0,  636,    2, 0x08 /* Private */,
     117,    1,  637,    2, 0x08 /* Private */,
     118,    0,  640,    2, 0x08 /* Private */,
     119,    0,  641,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::QUrl,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,    7,
    QMetaType::Void, QMetaType::QPoint,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   24,   25,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QUrl,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, QMetaType::Int,   38,   39,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,   46,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 51,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 51,   39,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 64,   65,
    QMetaType::Void, QMetaType::UInt, 0x80000000 | 68,   67,   69,
    QMetaType::Void, QMetaType::QString,   71,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 82,   81,   83,
    QMetaType::Void, QMetaType::QStringList,   85,
    QMetaType::Void, QMetaType::QStringList,   87,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   89,   90,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   89,   90,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   89,   90,
    QMetaType::Void, 0x80000000 | 51,   89,
    QMetaType::Void, QMetaType::QString,   95,
    QMetaType::Void, 0x80000000 | 97,   98,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  107,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 112,  113,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotComposeMailUrl((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 1: _t->slotComposeMail(); break;
        case 2: _t->invokeContactEditor(); break;
        case 3: _t->showContextMenuMboxTree((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 4: _t->showContextMenuMsgListTree((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 5: _t->slotReloadMboxList(); break;
        case 6: _t->slotResyncMbox(); break;
        case 7: _t->alertReceived((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->networkPolicyOffline(); break;
        case 9: _t->networkPolicyExpensive(); break;
        case 10: _t->networkPolicyOnline(); break;
        case 11: _t->slotShowSettings(); break;
        case 12: _t->slotShowImapInfo(); break;
        case 13: _t->slotExpunge(); break;
        case 14: _t->imapError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->networkError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->cacheError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->authenticationRequested(); break;
        case 18: _t->authenticationContinue((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 19: _t->authenticationContinue((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 20: _t->checkSslPolicy(); break;
        case 21: _t->slotManageContact((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 22: _t->slotEditDraft(); break;
        case 23: _t->slotReplyTo(); break;
        case 24: _t->slotReplyAllButMe(); break;
        case 25: _t->slotReplyAll(); break;
        case 26: _t->slotReplyList(); break;
        case 27: _t->slotReplyGuess(); break;
        case 28: _t->slotForwardAsAttachment(); break;
        case 29: _t->slotResend(); break;
        case 30: _t->slotUpdateMessageActions(); break;
        case 31: _t->handleTag((*reinterpret_cast< const bool(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2]))); break;
        case 32: _t->handleMarkAsRead((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 33: _t->handleMarkAsDeleted((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 34: _t->handleMarkAsFlagged((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 35: _t->handleMarkAsJunk((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 36: _t->handleMarkAsNotJunk((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 37: _t->slotMoveToArchiveFailed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 38: _t->handleMoveToArchive(); break;
        case 39: _t->slotNextUnread(); break;
        case 40: _t->slotPreviousUnread(); break;
        case 41: _t->msgListClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 42: _t->openCompleteMessageWidget(); break;
        case 43: _t->slotCreateMailboxBelowCurrent(); break;
        case 44: _t->slotMarkCurrentMailboxRead(); break;
        case 45: _t->slotCreateTopMailbox(); break;
        case 46: _t->slotDeleteCurrentMailbox(); break;
        case 47: _t->handleTrayIconChange(); break;
        case 48: _t->slotSubscribeCurrentMailbox(); break;
        case 49: _t->slotShowOnlySubscribed(); break;
        case 50: _t->updateMessageFlags(); break;
        case 51: _t->updateMessageFlagsOf((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 52: _t->scrollMessageUp(); break;
        case 53: _t->slotMessageModelChanged((*reinterpret_cast< QAbstractItemModel*(*)>(_a[1]))); break;
        case 54: _t->showConnectionStatus((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< Imap::ConnectionState(*)>(_a[2]))); break;
        case 55: _t->slotShowLinkTarget((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 56: _t->slotShowAboutTrojita(); break;
        case 57: _t->slotDonateToTrojita(); break;
        case 58: _t->slotSaveCurrentMessageBody(); break;
        case 59: _t->slotViewMsgSource(); break;
        case 60: _t->slotViewMsgHeaders(); break;
        case 61: _t->slotThreadMsgList(); break;
        case 62: _t->slotHideRead(); break;
        case 63: _t->slotSortingPreferenceChanged(); break;
        case 64: _t->slotSortingConfirmed((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Qt::SortOrder(*)>(_a[2]))); break;
        case 65: _t->slotSearchRequested((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 66: _t->slotCapabilitiesUpdated((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 67: _t->slotMailboxDeleteFailed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 68: _t->slotMailboxCreateFailed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 69: _t->slotMailboxSyncFailed((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 70: _t->slotMailboxChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 71: _t->slotDownloadTransferError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 72: _t->slotDownloadMessageFileNameRequested((*reinterpret_cast< QString*(*)>(_a[1]))); break;
        case 73: _t->slotScrollToUnseenMessage(); break;
        case 74: _t->slotScrollToCurrent(); break;
        case 75: _t->slotUpdateWindowTitle(); break;
        case 76: _t->slotLayoutCompact(); break;
        case 77: _t->slotLayoutWide(); break;
        case 78: _t->slotLayoutOneAtTime(); break;
        case 79: _t->saveSizesAndState(); break;
        case 80: _t->saveRawStateSetting((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 81: _t->possiblyLoadMessageOnSplittersChanged(); break;
        case 82: _t->updateNetworkIndication(); break;
        case 83: _t->desktopGeometryChanged(); break;
        case 84: _t->slotIconActivated((*reinterpret_cast< const QSystemTrayIcon::ActivationReason(*)>(_a[1]))); break;
        case 85: _t->slotToggleSysTray(); break;
        case 86: _t->slotResetReconnectState(); break;
        case 87: _t->slotPluginsChanged(); break;
        case 88: _t->showStatusMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 89: _t->slotFavoriteTagsChanged(); break;
        case 90: _t->recoverDrafts(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 53:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractItemModel* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Gui__MainWindow.data,
    qt_meta_data_Gui__MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int Gui::MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 91)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 91;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 91)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 91;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
