/****************************************************************************
** Meta object code from reading C++ file 'ComposerAttachmentsList.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/ComposerAttachmentsList.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ComposerAttachmentsList.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ComposerAttachmentsList_t {
    QByteArrayData data[11];
    char stringdata0[189];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ComposerAttachmentsList_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ComposerAttachmentsList_t qt_meta_stringdata_ComposerAttachmentsList = {
    {
QT_MOC_LITERAL(0, 0, 23), // "ComposerAttachmentsList"
QT_MOC_LITERAL(1, 24, 14), // "itemDroppedOut"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 20), // "slotRemoveAttachment"
QT_MOC_LITERAL(4, 61, 35), // "slotToggledContentDisposition..."
QT_MOC_LITERAL(5, 97, 7), // "checked"
QT_MOC_LITERAL(6, 105, 20), // "slotRenameAttachment"
QT_MOC_LITERAL(7, 126, 25), // "onAttachmentNumberChanged"
QT_MOC_LITERAL(8, 152, 16), // "onCurrentChanged"
QT_MOC_LITERAL(9, 169, 15), // "showContextMenu"
QT_MOC_LITERAL(10, 185, 3) // "pos"

    },
    "ComposerAttachmentsList\0itemDroppedOut\0"
    "\0slotRemoveAttachment\0"
    "slotToggledContentDispositionInline\0"
    "checked\0slotRenameAttachment\0"
    "onAttachmentNumberChanged\0onCurrentChanged\0"
    "showContextMenu\0pos"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ComposerAttachmentsList[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   50,    2, 0x0a /* Public */,
       4,    1,   51,    2, 0x0a /* Public */,
       6,    0,   54,    2, 0x0a /* Public */,
       7,    0,   55,    2, 0x0a /* Public */,
       8,    0,   56,    2, 0x0a /* Public */,
       9,    1,   57,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   10,

       0        // eod
};

void ComposerAttachmentsList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ComposerAttachmentsList *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemDroppedOut(); break;
        case 1: _t->slotRemoveAttachment(); break;
        case 2: _t->slotToggledContentDispositionInline((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->slotRenameAttachment(); break;
        case 4: _t->onAttachmentNumberChanged(); break;
        case 5: _t->onCurrentChanged(); break;
        case 6: _t->showContextMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ComposerAttachmentsList::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ComposerAttachmentsList::itemDroppedOut)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ComposerAttachmentsList::staticMetaObject = { {
    QMetaObject::SuperData::link<QListView::staticMetaObject>(),
    qt_meta_stringdata_ComposerAttachmentsList.data,
    qt_meta_data_ComposerAttachmentsList,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ComposerAttachmentsList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ComposerAttachmentsList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ComposerAttachmentsList.stringdata0))
        return static_cast<void*>(this);
    return QListView::qt_metacast(_clname);
}

int ComposerAttachmentsList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void ComposerAttachmentsList::itemDroppedOut()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
