/****************************************************************************
** Meta object code from reading C++ file 'ProtocolLoggerWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Gui/ProtocolLoggerWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ProtocolLoggerWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gui__ProtocolLoggerWidget_t {
    QByteArrayData data[17];
    char stringdata0[227];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gui__ProtocolLoggerWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gui__ProtocolLoggerWidget_t qt_meta_stringdata_Gui__ProtocolLoggerWidget = {
    {
QT_MOC_LITERAL(0, 0, 25), // "Gui::ProtocolLoggerWidget"
QT_MOC_LITERAL(1, 26, 24), // "persistentLoggingChanged"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 6), // "active"
QT_MOC_LITERAL(4, 59, 3), // "log"
QT_MOC_LITERAL(5, 63, 12), // "connectionId"
QT_MOC_LITERAL(6, 76, 18), // "Common::LogMessage"
QT_MOC_LITERAL(7, 95, 7), // "message"
QT_MOC_LITERAL(8, 103, 18), // "onConnectionClosed"
QT_MOC_LITERAL(9, 122, 21), // "Imap::ConnectionState"
QT_MOC_LITERAL(10, 144, 5), // "state"
QT_MOC_LITERAL(11, 150, 24), // "slotSetPersistentLogging"
QT_MOC_LITERAL(12, 175, 7), // "enabled"
QT_MOC_LITERAL(13, 183, 8), // "closeTab"
QT_MOC_LITERAL(14, 192, 5), // "index"
QT_MOC_LITERAL(15, 198, 15), // "clearLogDisplay"
QT_MOC_LITERAL(16, 214, 12) // "slotShowLogs"

    },
    "Gui::ProtocolLoggerWidget\0"
    "persistentLoggingChanged\0\0active\0log\0"
    "connectionId\0Common::LogMessage\0message\0"
    "onConnectionClosed\0Imap::ConnectionState\0"
    "state\0slotSetPersistentLogging\0enabled\0"
    "closeTab\0index\0clearLogDisplay\0"
    "slotShowLogs"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gui__ProtocolLoggerWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    2,   52,    2, 0x0a /* Public */,
       8,    2,   57,    2, 0x0a /* Public */,
      11,    1,   62,    2, 0x0a /* Public */,
      13,    1,   65,    2, 0x08 /* Private */,
      15,    0,   68,    2, 0x08 /* Private */,
      16,    0,   69,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::UInt, 0x80000000 | 6,    5,    7,
    QMetaType::Void, QMetaType::UInt, 0x80000000 | 9,    5,   10,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Gui::ProtocolLoggerWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ProtocolLoggerWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->persistentLoggingChanged((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 1: _t->log((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< Common::LogMessage(*)>(_a[2]))); break;
        case 2: _t->onConnectionClosed((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< Imap::ConnectionState(*)>(_a[2]))); break;
        case 3: _t->slotSetPersistentLogging((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 4: _t->closeTab((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->clearLogDisplay(); break;
        case 6: _t->slotShowLogs(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ProtocolLoggerWidget::*)(const bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ProtocolLoggerWidget::persistentLoggingChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gui::ProtocolLoggerWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Gui__ProtocolLoggerWidget.data,
    qt_meta_data_Gui__ProtocolLoggerWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gui::ProtocolLoggerWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gui::ProtocolLoggerWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gui__ProtocolLoggerWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Gui::ProtocolLoggerWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Gui::ProtocolLoggerWidget::persistentLoggingChanged(const bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
