/****************************************************************************
** Meta object code from reading C++ file 'Account.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/MSA/Account.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Account.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MSA__Account_t {
    QByteArrayData data[50];
    char stringdata0[714];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MSA__Account_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MSA__Account_t qt_meta_stringdata_MSA__Account = {
    {
QT_MOC_LITERAL(0, 0, 12), // "MSA::Account"
QT_MOC_LITERAL(1, 13, 23), // "submissionMethodChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 13), // "serverChanged"
QT_MOC_LITERAL(4, 52, 11), // "portChanged"
QT_MOC_LITERAL(5, 64, 15), // "usernameChanged"
QT_MOC_LITERAL(6, 80, 26), // "authenticateEnabledChanged"
QT_MOC_LITERAL(7, 107, 30), // "reuseImapAuthenticationChanged"
QT_MOC_LITERAL(8, 138, 21), // "pathToSendmailChanged"
QT_MOC_LITERAL(9, 160, 17), // "saveToImapChanged"
QT_MOC_LITERAL(10, 178, 22), // "sentMailboxNameChanged"
QT_MOC_LITERAL(11, 201, 14), // "useBurlChanged"
QT_MOC_LITERAL(12, 216, 15), // "showPortWarning"
QT_MOC_LITERAL(13, 232, 7), // "message"
QT_MOC_LITERAL(14, 240, 13), // "settingsSaved"
QT_MOC_LITERAL(15, 254, 16), // "settingsRestored"
QT_MOC_LITERAL(16, 271, 19), // "setSubmissionMethod"
QT_MOC_LITERAL(17, 291, 6), // "Method"
QT_MOC_LITERAL(18, 298, 6), // "method"
QT_MOC_LITERAL(19, 305, 9), // "setServer"
QT_MOC_LITERAL(20, 315, 6), // "server"
QT_MOC_LITERAL(21, 322, 7), // "setPort"
QT_MOC_LITERAL(22, 330, 4), // "port"
QT_MOC_LITERAL(23, 335, 11), // "setUsername"
QT_MOC_LITERAL(24, 347, 8), // "username"
QT_MOC_LITERAL(25, 356, 22), // "setAuthenticateEnabled"
QT_MOC_LITERAL(26, 379, 4), // "auth"
QT_MOC_LITERAL(27, 384, 26), // "setReuseImapAuthentication"
QT_MOC_LITERAL(28, 411, 13), // "reuseImapAuth"
QT_MOC_LITERAL(29, 425, 17), // "setPathToSendmail"
QT_MOC_LITERAL(30, 443, 14), // "pathToSendmail"
QT_MOC_LITERAL(31, 458, 13), // "setSaveToImap"
QT_MOC_LITERAL(32, 472, 8), // "selected"
QT_MOC_LITERAL(33, 481, 18), // "setSentMailboxName"
QT_MOC_LITERAL(34, 500, 8), // "location"
QT_MOC_LITERAL(35, 509, 10), // "setUseBurl"
QT_MOC_LITERAL(36, 520, 20), // "maybeShowPortWarning"
QT_MOC_LITERAL(37, 541, 12), // "saveSettings"
QT_MOC_LITERAL(38, 554, 15), // "restoreSettings"
QT_MOC_LITERAL(39, 570, 19), // "authenticateEnabled"
QT_MOC_LITERAL(40, 590, 23), // "reuseImapAuthentication"
QT_MOC_LITERAL(41, 614, 10), // "saveToImap"
QT_MOC_LITERAL(42, 625, 15), // "sentMailboxName"
QT_MOC_LITERAL(43, 641, 7), // "useBurl"
QT_MOC_LITERAL(44, 649, 16), // "submissionMethod"
QT_MOC_LITERAL(45, 666, 4), // "SMTP"
QT_MOC_LITERAL(46, 671, 5), // "SSMTP"
QT_MOC_LITERAL(47, 677, 13), // "SMTP_STARTTLS"
QT_MOC_LITERAL(48, 691, 8), // "SENDMAIL"
QT_MOC_LITERAL(49, 700, 13) // "IMAP_SENDMAIL"

    },
    "MSA::Account\0submissionMethodChanged\0"
    "\0serverChanged\0portChanged\0usernameChanged\0"
    "authenticateEnabledChanged\0"
    "reuseImapAuthenticationChanged\0"
    "pathToSendmailChanged\0saveToImapChanged\0"
    "sentMailboxNameChanged\0useBurlChanged\0"
    "showPortWarning\0message\0settingsSaved\0"
    "settingsRestored\0setSubmissionMethod\0"
    "Method\0method\0setServer\0server\0setPort\0"
    "port\0setUsername\0username\0"
    "setAuthenticateEnabled\0auth\0"
    "setReuseImapAuthentication\0reuseImapAuth\0"
    "setPathToSendmail\0pathToSendmail\0"
    "setSaveToImap\0selected\0setSentMailboxName\0"
    "location\0setUseBurl\0maybeShowPortWarning\0"
    "saveSettings\0restoreSettings\0"
    "authenticateEnabled\0reuseImapAuthentication\0"
    "saveToImap\0sentMailboxName\0useBurl\0"
    "submissionMethod\0SMTP\0SSMTP\0SMTP_STARTTLS\0"
    "SENDMAIL\0IMAP_SENDMAIL"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MSA__Account[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
      10,  192, // properties
       1,  232, // enums/sets
       0,    0, // constructors
       0,       // flags
      13,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  144,    2, 0x06 /* Public */,
       3,    0,  145,    2, 0x06 /* Public */,
       4,    0,  146,    2, 0x06 /* Public */,
       5,    0,  147,    2, 0x06 /* Public */,
       6,    0,  148,    2, 0x06 /* Public */,
       7,    0,  149,    2, 0x06 /* Public */,
       8,    0,  150,    2, 0x06 /* Public */,
       9,    0,  151,    2, 0x06 /* Public */,
      10,    0,  152,    2, 0x06 /* Public */,
      11,    0,  153,    2, 0x06 /* Public */,
      12,    1,  154,    2, 0x06 /* Public */,
      14,    0,  157,    2, 0x06 /* Public */,
      15,    0,  158,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    1,  159,    2, 0x0a /* Public */,
      19,    1,  162,    2, 0x0a /* Public */,
      21,    1,  165,    2, 0x0a /* Public */,
      23,    1,  168,    2, 0x0a /* Public */,
      25,    1,  171,    2, 0x0a /* Public */,
      27,    1,  174,    2, 0x0a /* Public */,
      29,    1,  177,    2, 0x0a /* Public */,
      31,    1,  180,    2, 0x0a /* Public */,
      33,    1,  183,    2, 0x0a /* Public */,
      35,    1,  186,    2, 0x0a /* Public */,
      36,    0,  189,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      37,    0,  190,    2, 0x02 /* Public */,
      38,    0,  191,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void, QMetaType::UShort,   22,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void, QMetaType::Bool,   28,
    QMetaType::Void, QMetaType::QString,   30,
    QMetaType::Void, QMetaType::Bool,   32,
    QMetaType::Void, QMetaType::QString,   34,
    QMetaType::Void, QMetaType::Bool,   32,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      20, QMetaType::QString, 0x00495103,
      22, QMetaType::Int, 0x00495103,
      24, QMetaType::QString, 0x00495103,
      39, QMetaType::Bool, 0x00495103,
      40, QMetaType::Bool, 0x00495103,
      30, QMetaType::QString, 0x00495103,
      41, QMetaType::Bool, 0x00495103,
      42, QMetaType::QString, 0x00495103,
      43, QMetaType::Bool, 0x00495103,
      44, 0x80000000 | 17, 0x0049510b,

 // properties: notify_signal_id
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
       0,

 // enums: name, alias, flags, count, data
      17,   17, 0x2,    5,  237,

 // enum data: key, value
      45, uint(MSA::Account::Method::SMTP),
      46, uint(MSA::Account::Method::SSMTP),
      47, uint(MSA::Account::Method::SMTP_STARTTLS),
      48, uint(MSA::Account::Method::SENDMAIL),
      49, uint(MSA::Account::Method::IMAP_SENDMAIL),

       0        // eod
};

void MSA::Account::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Account *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->submissionMethodChanged(); break;
        case 1: _t->serverChanged(); break;
        case 2: _t->portChanged(); break;
        case 3: _t->usernameChanged(); break;
        case 4: _t->authenticateEnabledChanged(); break;
        case 5: _t->reuseImapAuthenticationChanged(); break;
        case 6: _t->pathToSendmailChanged(); break;
        case 7: _t->saveToImapChanged(); break;
        case 8: _t->sentMailboxNameChanged(); break;
        case 9: _t->useBurlChanged(); break;
        case 10: _t->showPortWarning((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->settingsSaved(); break;
        case 12: _t->settingsRestored(); break;
        case 13: _t->setSubmissionMethod((*reinterpret_cast< const Method(*)>(_a[1]))); break;
        case 14: _t->setServer((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->setPort((*reinterpret_cast< const quint16(*)>(_a[1]))); break;
        case 16: _t->setUsername((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->setAuthenticateEnabled((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 18: _t->setReuseImapAuthentication((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 19: _t->setPathToSendmail((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 20: _t->setSaveToImap((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 21: _t->setSentMailboxName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 22: _t->setUseBurl((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 23: _t->maybeShowPortWarning(); break;
        case 24: _t->saveSettings(); break;
        case 25: _t->restoreSettings(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::submissionMethodChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::serverChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::portChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::usernameChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::authenticateEnabledChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::reuseImapAuthenticationChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::pathToSendmailChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::saveToImapChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::sentMailboxNameChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::useBurlChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Account::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::showPortWarning)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::settingsSaved)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Account::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Account::settingsRestored)) {
                *result = 12;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Account *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->server(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->port(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->username(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->authenticateEnabled(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->reuseImapAuthentication(); break;
        case 5: *reinterpret_cast< QString*>(_v) = _t->pathToSendmail(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->saveToImap(); break;
        case 7: *reinterpret_cast< QString*>(_v) = _t->sentMailboxName(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->useBurl(); break;
        case 9: *reinterpret_cast< Method*>(_v) = _t->submissionMethod(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Account *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setServer(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setPort(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setUsername(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setAuthenticateEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setReuseImapAuthentication(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setPathToSendmail(*reinterpret_cast< QString*>(_v)); break;
        case 6: _t->setSaveToImap(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setSentMailboxName(*reinterpret_cast< QString*>(_v)); break;
        case 8: _t->setUseBurl(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setSubmissionMethod(*reinterpret_cast< Method*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject MSA::Account::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_MSA__Account.data,
    qt_meta_data_MSA__Account,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MSA::Account::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MSA::Account::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MSA__Account.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MSA::Account::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 26;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void MSA::Account::submissionMethodChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MSA::Account::serverChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MSA::Account::portChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MSA::Account::usernameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MSA::Account::authenticateEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MSA::Account::reuseImapAuthenticationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MSA::Account::pathToSendmailChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MSA::Account::saveToImapChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void MSA::Account::sentMailboxNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void MSA::Account::useBurlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void MSA::Account::showPortWarning(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void MSA::Account::settingsSaved()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void MSA::Account::settingsRestored()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
