/****************************************************************************
** Meta object code from reading C++ file 'ModelWatcher.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Model/ModelWatcher.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ModelWatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Mailbox__ModelWatcher_t {
    QByteArrayData data[34];
    char stringdata0[493];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Mailbox__ModelWatcher_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Mailbox__ModelWatcher_t qt_meta_stringdata_Imap__Mailbox__ModelWatcher = {
    {
QT_MOC_LITERAL(0, 0, 27), // "Imap::Mailbox::ModelWatcher"
QT_MOC_LITERAL(1, 28, 24), // "columnsAboutToBeInserted"
QT_MOC_LITERAL(2, 53, 0), // ""
QT_MOC_LITERAL(3, 54, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 66, 23), // "columnsAboutToBeRemoved"
QT_MOC_LITERAL(5, 90, 15), // "columnsInserted"
QT_MOC_LITERAL(6, 106, 14), // "columnsRemoved"
QT_MOC_LITERAL(7, 121, 11), // "dataChanged"
QT_MOC_LITERAL(8, 133, 17), // "headerDataChanged"
QT_MOC_LITERAL(9, 151, 15), // "Qt::Orientation"
QT_MOC_LITERAL(10, 167, 22), // "layoutAboutToBeChanged"
QT_MOC_LITERAL(11, 190, 13), // "layoutChanged"
QT_MOC_LITERAL(12, 204, 19), // "modelAboutToBeReset"
QT_MOC_LITERAL(13, 224, 10), // "modelReset"
QT_MOC_LITERAL(14, 235, 21), // "rowsAboutToBeInserted"
QT_MOC_LITERAL(15, 257, 20), // "rowsAboutToBeRemoved"
QT_MOC_LITERAL(16, 278, 12), // "rowsInserted"
QT_MOC_LITERAL(17, 291, 11), // "rowsRemoved"
QT_MOC_LITERAL(18, 303, 18), // "rowsAboutToBeMoved"
QT_MOC_LITERAL(19, 322, 12), // "sourceParent"
QT_MOC_LITERAL(20, 335, 11), // "sourceStart"
QT_MOC_LITERAL(21, 347, 9), // "sourceEnd"
QT_MOC_LITERAL(22, 357, 17), // "destinationParent"
QT_MOC_LITERAL(23, 375, 14), // "destinationRow"
QT_MOC_LITERAL(24, 390, 9), // "rowsMoved"
QT_MOC_LITERAL(25, 400, 6), // "parent"
QT_MOC_LITERAL(26, 407, 5), // "start"
QT_MOC_LITERAL(27, 413, 3), // "end"
QT_MOC_LITERAL(28, 417, 11), // "destination"
QT_MOC_LITERAL(29, 429, 3), // "row"
QT_MOC_LITERAL(30, 433, 21), // "columnsAboutToBeMoved"
QT_MOC_LITERAL(31, 455, 17), // "destinationColumn"
QT_MOC_LITERAL(32, 473, 12), // "columnsMoved"
QT_MOC_LITERAL(33, 486, 6) // "column"

    },
    "Imap::Mailbox::ModelWatcher\0"
    "columnsAboutToBeInserted\0\0QModelIndex\0"
    "columnsAboutToBeRemoved\0columnsInserted\0"
    "columnsRemoved\0dataChanged\0headerDataChanged\0"
    "Qt::Orientation\0layoutAboutToBeChanged\0"
    "layoutChanged\0modelAboutToBeReset\0"
    "modelReset\0rowsAboutToBeInserted\0"
    "rowsAboutToBeRemoved\0rowsInserted\0"
    "rowsRemoved\0rowsAboutToBeMoved\0"
    "sourceParent\0sourceStart\0sourceEnd\0"
    "destinationParent\0destinationRow\0"
    "rowsMoved\0parent\0start\0end\0destination\0"
    "row\0columnsAboutToBeMoved\0destinationColumn\0"
    "columnsMoved\0column"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Mailbox__ModelWatcher[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,  104,    2, 0x08 /* Private */,
       4,    3,  111,    2, 0x08 /* Private */,
       5,    3,  118,    2, 0x08 /* Private */,
       6,    3,  125,    2, 0x08 /* Private */,
       7,    2,  132,    2, 0x08 /* Private */,
       8,    3,  137,    2, 0x08 /* Private */,
      10,    0,  144,    2, 0x08 /* Private */,
      11,    0,  145,    2, 0x08 /* Private */,
      12,    0,  146,    2, 0x08 /* Private */,
      13,    0,  147,    2, 0x08 /* Private */,
      14,    3,  148,    2, 0x08 /* Private */,
      15,    3,  155,    2, 0x08 /* Private */,
      16,    3,  162,    2, 0x08 /* Private */,
      17,    3,  169,    2, 0x08 /* Private */,
      18,    5,  176,    2, 0x08 /* Private */,
      24,    5,  187,    2, 0x08 /* Private */,
      30,    5,  198,    2, 0x08 /* Private */,
      32,    5,  209,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    2,    2,
    QMetaType::Void, 0x80000000 | 9, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int, 0x80000000 | 3, QMetaType::Int,   19,   20,   21,   22,   23,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int, 0x80000000 | 3, QMetaType::Int,   25,   26,   27,   28,   29,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int, 0x80000000 | 3, QMetaType::Int,   19,   20,   21,   22,   31,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int, 0x80000000 | 3, QMetaType::Int,   25,   26,   27,   28,   33,

       0        // eod
};

void Imap::Mailbox::ModelWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ModelWatcher *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->columnsAboutToBeInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 1: _t->columnsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 2: _t->columnsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 3: _t->columnsRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 4: _t->dataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 5: _t->headerDataChanged((*reinterpret_cast< Qt::Orientation(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 6: _t->layoutAboutToBeChanged(); break;
        case 7: _t->layoutChanged(); break;
        case 8: _t->modelAboutToBeReset(); break;
        case 9: _t->modelReset(); break;
        case 10: _t->rowsAboutToBeInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 11: _t->rowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 12: _t->rowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 13: _t->rowsRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 14: _t->rowsAboutToBeMoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QModelIndex(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 15: _t->rowsMoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QModelIndex(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 16: _t->columnsAboutToBeMoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QModelIndex(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 17: _t->columnsMoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QModelIndex(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Imap::Mailbox::ModelWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Imap__Mailbox__ModelWatcher.data,
    qt_meta_data_Imap__Mailbox__ModelWatcher,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Mailbox::ModelWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Mailbox::ModelWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Mailbox__ModelWatcher.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Imap::Mailbox::ModelWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
