/****************************************************************************
** Meta object code from reading C++ file 'FileDownloadManager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Network/FileDownloadManager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'FileDownloadManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Imap__Network__FileDownloadManager_t {
    QByteArrayData data[19];
    char stringdata0[280];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Imap__Network__FileDownloadManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Imap__Network__FileDownloadManager_t qt_meta_stringdata_Imap__Network__FileDownloadManager = {
    {
QT_MOC_LITERAL(0, 0, 34), // "Imap::Network::FileDownloadMa..."
QT_MOC_LITERAL(1, 35, 13), // "transferError"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 12), // "errorMessage"
QT_MOC_LITERAL(4, 63, 17), // "fileNameRequested"
QT_MOC_LITERAL(5, 81, 8), // "QString*"
QT_MOC_LITERAL(6, 90, 8), // "fileName"
QT_MOC_LITERAL(7, 99, 9), // "succeeded"
QT_MOC_LITERAL(8, 109, 9), // "cancelled"
QT_MOC_LITERAL(9, 119, 20), // "onPartDataTransfered"
QT_MOC_LITERAL(10, 140, 20), // "onReplyTransferError"
QT_MOC_LITERAL(11, 161, 23), // "onCombinerTransferError"
QT_MOC_LITERAL(12, 185, 7), // "message"
QT_MOC_LITERAL(13, 193, 11), // "deleteReply"
QT_MOC_LITERAL(14, 205, 14), // "QNetworkReply*"
QT_MOC_LITERAL(15, 220, 5), // "reply"
QT_MOC_LITERAL(16, 226, 12), // "downloadPart"
QT_MOC_LITERAL(17, 239, 15), // "downloadMessage"
QT_MOC_LITERAL(18, 255, 24) // "onMessageDataTransferred"

    },
    "Imap::Network::FileDownloadManager\0"
    "transferError\0\0errorMessage\0"
    "fileNameRequested\0QString*\0fileName\0"
    "succeeded\0cancelled\0onPartDataTransfered\0"
    "onReplyTransferError\0onCombinerTransferError\0"
    "message\0deleteReply\0QNetworkReply*\0"
    "reply\0downloadPart\0downloadMessage\0"
    "onMessageDataTransferred"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Imap__Network__FileDownloadManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,
       4,    1,   72,    2, 0x06 /* Public */,
       7,    0,   75,    2, 0x06 /* Public */,
       8,    0,   76,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   77,    2, 0x08 /* Private */,
      10,    0,   78,    2, 0x08 /* Private */,
      11,    1,   79,    2, 0x08 /* Private */,
      13,    1,   82,    2, 0x08 /* Private */,
      16,    0,   85,    2, 0x0a /* Public */,
      17,    0,   86,    2, 0x0a /* Public */,
      18,    0,   87,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Imap::Network::FileDownloadManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileDownloadManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->transferError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->fileNameRequested((*reinterpret_cast< QString*(*)>(_a[1]))); break;
        case 2: _t->succeeded(); break;
        case 3: _t->cancelled(); break;
        case 4: _t->onPartDataTransfered(); break;
        case 5: _t->onReplyTransferError(); break;
        case 6: _t->onCombinerTransferError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->deleteReply((*reinterpret_cast< QNetworkReply*(*)>(_a[1]))); break;
        case 8: _t->downloadPart(); break;
        case 9: _t->downloadMessage(); break;
        case 10: _t->onMessageDataTransferred(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNetworkReply* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileDownloadManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDownloadManager::transferError)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileDownloadManager::*)(QString * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDownloadManager::fileNameRequested)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileDownloadManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDownloadManager::succeeded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileDownloadManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileDownloadManager::cancelled)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Imap::Network::FileDownloadManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Imap__Network__FileDownloadManager.data,
    qt_meta_data_Imap__Network__FileDownloadManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Imap::Network::FileDownloadManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Imap::Network::FileDownloadManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Imap__Network__FileDownloadManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Imap::Network::FileDownloadManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void Imap::Network::FileDownloadManager::transferError(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Imap::Network::FileDownloadManager::fileNameRequested(QString * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Imap::Network::FileDownloadManager::succeeded()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Imap::Network::FileDownloadManager::cancelled()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
