/****************************************************************************
** Meta object code from reading C++ file 'kdescendantsproxymodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Imap/Model/kdeui-itemviews/kdescendantsproxymodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kdescendantsproxymodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KDescendantsProxyModel_t {
    QByteArrayData data[16];
    char stringdata0[322];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDescendantsProxyModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDescendantsProxyModel_t qt_meta_stringdata_KDescendantsProxyModel = {
    {
QT_MOC_LITERAL(0, 0, 22), // "KDescendantsProxyModel"
QT_MOC_LITERAL(1, 23, 27), // "sourceRowsAboutToBeInserted"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 64, 18), // "sourceRowsInserted"
QT_MOC_LITERAL(5, 83, 26), // "sourceRowsAboutToBeRemoved"
QT_MOC_LITERAL(6, 110, 17), // "sourceRowsRemoved"
QT_MOC_LITERAL(7, 128, 24), // "sourceRowsAboutToBeMoved"
QT_MOC_LITERAL(8, 153, 15), // "sourceRowsMoved"
QT_MOC_LITERAL(9, 169, 25), // "sourceModelAboutToBeReset"
QT_MOC_LITERAL(10, 195, 16), // "sourceModelReset"
QT_MOC_LITERAL(11, 212, 28), // "sourceLayoutAboutToBeChanged"
QT_MOC_LITERAL(12, 241, 19), // "sourceLayoutChanged"
QT_MOC_LITERAL(13, 261, 17), // "sourceDataChanged"
QT_MOC_LITERAL(14, 279, 20), // "sourceModelDestroyed"
QT_MOC_LITERAL(15, 300, 21) // "processPendingParents"

    },
    "KDescendantsProxyModel\0"
    "sourceRowsAboutToBeInserted\0\0QModelIndex\0"
    "sourceRowsInserted\0sourceRowsAboutToBeRemoved\0"
    "sourceRowsRemoved\0sourceRowsAboutToBeMoved\0"
    "sourceRowsMoved\0sourceModelAboutToBeReset\0"
    "sourceModelReset\0sourceLayoutAboutToBeChanged\0"
    "sourceLayoutChanged\0sourceDataChanged\0"
    "sourceModelDestroyed\0processPendingParents"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDescendantsProxyModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   79,    2, 0x08 /* Private */,
       4,    3,   86,    2, 0x08 /* Private */,
       5,    3,   93,    2, 0x08 /* Private */,
       6,    3,  100,    2, 0x08 /* Private */,
       7,    5,  107,    2, 0x08 /* Private */,
       8,    5,  118,    2, 0x08 /* Private */,
       9,    0,  129,    2, 0x08 /* Private */,
      10,    0,  130,    2, 0x08 /* Private */,
      11,    0,  131,    2, 0x08 /* Private */,
      12,    0,  132,    2, 0x08 /* Private */,
      13,    2,  133,    2, 0x08 /* Private */,
      14,    0,  138,    2, 0x08 /* Private */,
      15,    0,  139,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int, 0x80000000 | 3, QMetaType::Int,    2,    2,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int, QMetaType::Int, 0x80000000 | 3, QMetaType::Int,    2,    2,    2,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    2,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void KDescendantsProxyModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<KDescendantsProxyModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->d_func()->sourceRowsAboutToBeInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 1: _t->d_func()->sourceRowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 2: _t->d_func()->sourceRowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 3: _t->d_func()->sourceRowsRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 4: _t->d_func()->sourceRowsAboutToBeMoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QModelIndex(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 5: _t->d_func()->sourceRowsMoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QModelIndex(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 6: _t->d_func()->sourceModelAboutToBeReset(); break;
        case 7: _t->d_func()->sourceModelReset(); break;
        case 8: _t->d_func()->sourceLayoutAboutToBeChanged(); break;
        case 9: _t->d_func()->sourceLayoutChanged(); break;
        case 10: _t->d_func()->sourceDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 11: _t->d_func()->sourceModelDestroyed(); break;
        case 12: _t->d_func()->processPendingParents(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KDescendantsProxyModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractProxyModel::staticMetaObject>(),
    qt_meta_stringdata_KDescendantsProxyModel.data,
    qt_meta_data_KDescendantsProxyModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KDescendantsProxyModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDescendantsProxyModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KDescendantsProxyModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractProxyModel::qt_metacast(_clname);
}

int KDescendantsProxyModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
