/* Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
   All rights reserved.
   Contact: Nokia Corporation (qt-info@nokia.com)

   This file is part of the Trojita Qt IMAP e-mail client,
   http://trojita.flaska.net/

   It was taken from the examples of the Qt Toolkit.

   $QT_BEGIN_LICENSE:LGPL$
   Commercial Usage
   Licensees holding valid Qt Commercial licenses may use this file in
   accordance with the Qt Commercial License Agreement provided with the
   Software or, alternatively, in accordance with the terms contained in
   a written agreement between you and Nokia.

   GNU Lesser General Public License Usage
   Alternatively, this file may be used under the terms of the GNU Lesser
   General Public License version 2.1 as published by the Free Software
   Foundation and appearing in the file LICENSE.LGPL included in the
   packaging of this file.  Please review the following information to
   ensure the GNU Lesser General Public License version 2.1 requirements
   will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.

   In addition, as a special exception, Nokia gives you certain additional
   rights.  These rights are described in the Nokia Qt LGPL Exception
   version 1.1, included in the file LGPL_EXCEPTION.txt in this package.

   GNU General Public License Usage
   Alternatively, this file may be used under the terms of the GNU
   General Public License version 3.0 as published by the Free Software
   Foundation and appearing in the file LICENSE.GPL included in the
   packaging of this file.  Please review the following information to
   ensure the GNU General Public License version 3.0 requirements will be
   met: http://www.gnu.org/copyleft/gpl.html.

   If you have questions regarding the use of this file, please contact
   Nokia at qt-info@nokia.com.
   $QT_END_LICENSE$
*/

#ifndef GUI_FLOWLAYOUT_H
#define GUI_FLOWLAYOUT_H

#include <QLayout>
#include <QRect>
#include <QStyle>
#include <QWidgetItem>

namespace Gui
{

class FlowLayout : public QLayout
{
public:
    explicit FlowLayout(QWidget *parent, int margin = -1, int hSpacing = -1, int vSpacing = -1);
    explicit FlowLayout(int margin = -1, int hSpacing = -1, int vSpacing = -1);
    ~FlowLayout();

    void addItem(QLayoutItem *item);
    int horizontalSpacing() const;
    int verticalSpacing() const;
    Qt::Orientations expandingDirections() const;
    bool hasHeightForWidth() const;
    int heightForWidth(int) const;
    int count() const;
    QLayoutItem *itemAt(int index) const;
    QSize minimumSize() const;
    void setGeometry(const QRect &rect);
    QSize sizeHint() const;
    QLayoutItem *takeAt(int index);

private:
    int doLayout(const QRect &rect, bool testOnly) const;
    int smartSpacing(QStyle::PixelMetric pm) const;

    QList<QLayoutItem *> itemList;
    int m_hSpace;
    int m_vSpace;
};

} // namespace Gui

#endif // GUI_FLOWLAYOUT_H
