/****************************************************************************
** Meta object code from reading C++ file 'Submission.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/Composer/Submission.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Submission.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Composer__Submission_t {
    QByteArrayData data[43];
    char stringdata0[602];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Composer__Submission_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Composer__Submission_t qt_meta_stringdata_Composer__Submission = {
    {
QT_MOC_LITERAL(0, 0, 20), // "Composer::Submission"
QT_MOC_LITERAL(1, 21, 11), // "progressMin"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 3), // "min"
QT_MOC_LITERAL(4, 38, 11), // "progressMax"
QT_MOC_LITERAL(5, 50, 3), // "max"
QT_MOC_LITERAL(6, 54, 8), // "progress"
QT_MOC_LITERAL(7, 63, 19), // "updateStatusMessage"
QT_MOC_LITERAL(8, 83, 7), // "message"
QT_MOC_LITERAL(9, 91, 6), // "failed"
QT_MOC_LITERAL(10, 98, 9), // "succeeded"
QT_MOC_LITERAL(11, 108, 17), // "passwordRequested"
QT_MOC_LITERAL(12, 126, 4), // "user"
QT_MOC_LITERAL(13, 131, 4), // "host"
QT_MOC_LITERAL(14, 136, 11), // "gotPassword"
QT_MOC_LITERAL(15, 148, 8), // "password"
QT_MOC_LITERAL(16, 157, 8), // "canceled"
QT_MOC_LITERAL(17, 166, 6), // "logged"
QT_MOC_LITERAL(18, 173, 15), // "Common::LogKind"
QT_MOC_LITERAL(19, 189, 4), // "kind"
QT_MOC_LITERAL(20, 194, 6), // "source"
QT_MOC_LITERAL(21, 201, 11), // "setPassword"
QT_MOC_LITERAL(22, 213, 14), // "cancelPassword"
QT_MOC_LITERAL(23, 228, 8), // "gotError"
QT_MOC_LITERAL(24, 237, 5), // "error"
QT_MOC_LITERAL(25, 243, 4), // "sent"
QT_MOC_LITERAL(26, 248, 18), // "slotAppendUidKnown"
QT_MOC_LITERAL(27, 267, 11), // "uidValidity"
QT_MOC_LITERAL(28, 279, 3), // "uid"
QT_MOC_LITERAL(29, 283, 22), // "slotGenUrlAuthReceived"
QT_MOC_LITERAL(30, 306, 3), // "url"
QT_MOC_LITERAL(31, 310, 19), // "slotAppendSucceeded"
QT_MOC_LITERAL(32, 330, 16), // "slotAppendFailed"
QT_MOC_LITERAL(33, 347, 35), // "onUpdatingFlagsOfReplyingToSu..."
QT_MOC_LITERAL(34, 383, 33), // "onUpdatingFlagsOfReplyingToFa..."
QT_MOC_LITERAL(35, 417, 36), // "onUpdatingFlagsOfForwardingSu..."
QT_MOC_LITERAL(36, 454, 33), // "onUpdatingFlagsOfForwardingFa..."
QT_MOC_LITERAL(37, 488, 24), // "slotMessageDataAvailable"
QT_MOC_LITERAL(38, 513, 13), // "slotAskForUrl"
QT_MOC_LITERAL(39, 527, 16), // "slotInvokeMsaNow"
QT_MOC_LITERAL(40, 544, 23), // "onMsaProgressMaxChanged"
QT_MOC_LITERAL(41, 568, 27), // "onMsaProgressCurrentChanged"
QT_MOC_LITERAL(42, 596, 5) // "value"

    },
    "Composer::Submission\0progressMin\0\0min\0"
    "progressMax\0max\0progress\0updateStatusMessage\0"
    "message\0failed\0succeeded\0passwordRequested\0"
    "user\0host\0gotPassword\0password\0canceled\0"
    "logged\0Common::LogKind\0kind\0source\0"
    "setPassword\0cancelPassword\0gotError\0"
    "error\0sent\0slotAppendUidKnown\0uidValidity\0"
    "uid\0slotGenUrlAuthReceived\0url\0"
    "slotAppendSucceeded\0slotAppendFailed\0"
    "onUpdatingFlagsOfReplyingToSucceded\0"
    "onUpdatingFlagsOfReplyingToFailed\0"
    "onUpdatingFlagsOfForwardingSucceeded\0"
    "onUpdatingFlagsOfForwardingFailed\0"
    "slotMessageDataAvailable\0slotAskForUrl\0"
    "slotInvokeMsaNow\0onMsaProgressMaxChanged\0"
    "onMsaProgressCurrentChanged\0value"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Composer__Submission[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      27,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  149,    2, 0x06 /* Public */,
       4,    1,  152,    2, 0x06 /* Public */,
       6,    1,  155,    2, 0x06 /* Public */,
       7,    1,  158,    2, 0x06 /* Public */,
       9,    1,  161,    2, 0x06 /* Public */,
      10,    0,  164,    2, 0x06 /* Public */,
      11,    2,  165,    2, 0x06 /* Public */,
      14,    1,  170,    2, 0x06 /* Public */,
      16,    0,  173,    2, 0x06 /* Public */,
      17,    3,  174,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      21,    1,  181,    2, 0x0a /* Public */,
      22,    0,  184,    2, 0x0a /* Public */,
      23,    1,  185,    2, 0x08 /* Private */,
      25,    0,  188,    2, 0x08 /* Private */,
      26,    2,  189,    2, 0x08 /* Private */,
      29,    1,  194,    2, 0x08 /* Private */,
      31,    0,  197,    2, 0x08 /* Private */,
      32,    1,  198,    2, 0x08 /* Private */,
      33,    0,  201,    2, 0x08 /* Private */,
      34,    0,  202,    2, 0x08 /* Private */,
      35,    0,  203,    2, 0x08 /* Private */,
      36,    0,  204,    2, 0x08 /* Private */,
      37,    0,  205,    2, 0x08 /* Private */,
      38,    0,  206,    2, 0x08 /* Private */,
      39,    0,  207,    2, 0x08 /* Private */,
      40,    1,  208,    2, 0x08 /* Private */,
      41,    1,  211,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   12,   13,
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18, QMetaType::QString, QMetaType::QString,   19,   20,    8,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UInt, QMetaType::UInt,   27,   28,
    QMetaType::Void, QMetaType::QString,   30,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,   42,

       0        // eod
};

void Composer::Submission::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Submission *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->progressMin((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 1: _t->progressMax((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 2: _t->progress((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 3: _t->updateStatusMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->failed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->succeeded(); break;
        case 6: _t->passwordRequested((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 7: _t->gotPassword((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->canceled(); break;
        case 9: _t->logged((*reinterpret_cast< const Common::LogKind(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 10: _t->setPassword((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->cancelPassword(); break;
        case 12: _t->gotError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->sent(); break;
        case 14: _t->slotAppendUidKnown((*reinterpret_cast< const uint(*)>(_a[1])),(*reinterpret_cast< const uint(*)>(_a[2]))); break;
        case 15: _t->slotGenUrlAuthReceived((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->slotAppendSucceeded(); break;
        case 17: _t->slotAppendFailed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->onUpdatingFlagsOfReplyingToSucceded(); break;
        case 19: _t->onUpdatingFlagsOfReplyingToFailed(); break;
        case 20: _t->onUpdatingFlagsOfForwardingSucceeded(); break;
        case 21: _t->onUpdatingFlagsOfForwardingFailed(); break;
        case 22: _t->slotMessageDataAvailable(); break;
        case 23: _t->slotAskForUrl(); break;
        case 24: _t->slotInvokeMsaNow(); break;
        case 25: _t->onMsaProgressMaxChanged((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 26: _t->onMsaProgressCurrentChanged((*reinterpret_cast< const int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Submission::*)(const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Submission::progressMin)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Submission::*)(const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Submission::progressMax)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Submission::*)(const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Submission::progress)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Submission::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Submission::updateStatusMessage)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Submission::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Submission::failed)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Submission::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Submission::succeeded)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Submission::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Submission::passwordRequested)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Submission::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Submission::gotPassword)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Submission::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Submission::canceled)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Submission::*)(const Common::LogKind , const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Submission::logged)) {
                *result = 9;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Composer::Submission::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Composer__Submission.data,
    qt_meta_data_Composer__Submission,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Composer::Submission::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Composer::Submission::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Composer__Submission.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Composer::Submission::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void Composer::Submission::progressMin(const int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Composer::Submission::progressMax(const int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Composer::Submission::progress(const int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Composer::Submission::updateStatusMessage(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Composer::Submission::failed(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void Composer::Submission::succeeded()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Composer::Submission::passwordRequested(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void Composer::Submission::gotPassword(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void Composer::Submission::canceled()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Composer::Submission::logged(const Common::LogKind _t1, const QString & _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
